/*
* CLIF is a Load Injection Framework
* Copyright (C) 2008 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.lib.graph;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.Map.Entry;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.ow2.clif.analyze.api.graph.DataSupplier;
import org.ow2.clif.analyze.api.graph.virtualdata.FilterDataControl;
import org.ow2.clif.storage.api.BladeEvent;
import org.ow2.clif.storage.api.EventFilter;
import org.ow2.clif.supervisor.api.ClifException;

/**
 * The Class FilterDataImpl
 * @author Olivier Liu
 */

public class FilterDataImpl implements BindingController, FilterDataControl {
	/** The instance. */
    private static FilterDataImpl instance= new FilterDataImpl();
    
    /** The interface list. */
    static private String[] itfList = new String[] { 
    	DataSupplier.DATA_SUPPLIER
    };

    // field for client binding (access to stored data)
    /** The data supplier interface. */
    private DataSupplier dsItf;
	
    private String OPERATOR_EQUALS = "equals";
    private String OPERATOR_NOT_EQUALS = "not equals";
    private String OPERATOR_CONTAINS = "contains";
    private String OPERATOR_NOT_CONTAINS = "not contains";
    private String OPERATOR_BEGINS_WITH = "begins with";
    private String OPERATOR_ENDS_WITH = "ends with";
    private String OPERATOR_GREATER_THAN = "greater than";
    private String OPERATOR_LESS_THAN = "less than";
    private String OPERATOR_GREATER_THAN_OR_EQUALS = "greater than or equals";
    private String OPERATOR_LESS_THAN_OR_EQUALS = "less than or equals";
    
    /**
     * Instantiates a new FilterDataImpl
     */
    public FilterDataImpl()
    { 
       
    }
    
    
    /**
     * Gets the single instance of FilterDataImpl
     * 
     * @return single instance of FilterDataImpl
     */
    public FilterDataImpl getInstance()
    {
        return instance;
    }


    /////////////////////////////////
    // interface BindingController //
    /////////////////////////////////
    
    /* (non-Javadoc)
     * @see org.ow2.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
     */
    public void bindFc(String clientItfName, Object serverItf) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
    {
        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
        {
            dsItf = (DataSupplier)serverItf;
        }
    }
    

    /* (non-Javadoc)
     * @see org.ow2.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() 
    {
        return itfList;
    }

    
    /* (non-Javadoc)
     * @see org.ow2.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String clientItfName) throws NoSuchInterfaceException 
    {
        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
        {
            return dsItf;
        }
        else
        {
            return null;
        }
    }

    
    /* (non-Javadoc)
     * @see org.ow2.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
    {
        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
        {
            dsItf = null;
        }
    }
    

    /////////////////////////////////
    // interface FilterDataControl //
    /////////////////////////////////
    
    /* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.FilterDataControl#getBladeByTime(java.util.list, Integer, Integer)
	 */
	public List<BladeEvent> getBladeByTime(List<BladeEvent> bladeEventList,
			int start, int end) {
		//blade events list result
		List<BladeEvent> bELR = new ArrayList<BladeEvent>();
		Iterator<BladeEvent> iterator = bladeEventList.iterator();
		while (iterator.hasNext()) {
			BladeEvent bladeEvent = (BladeEvent) iterator.next();
			if((bladeEvent.getDate() > start) 
					&& (bladeEvent.getDate() < end)){
				bELR.add(bladeEvent);
			}
		}
		return bELR;
	}
    
    
    
    /* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.FilterDataControl#getBladeOneEventValues(java.lang.String, java.lang.String, java.lang.String)
	 */
    public List<BladeEvent> getBladeOneEventValues(
    		String testName,
    		String bladeId, 
    		String eventTypeLabel) 
    		throws ClifException {
    	try {
    		//get all the values from a bladeEvent
    		BladeEvent[] bladeEventArray = 
        		dsItf.getBladeEvents(testName, bladeId, eventTypeLabel, null, 0, -1);
        	List<BladeEvent> bladeEventList = new ArrayList<BladeEvent>();
        	int bladeEventArrayLength = bladeEventArray.length;
        	//tab to array list
        	for (int i = 0; i < bladeEventArrayLength; i++) {
        		bladeEventList.add(bladeEventArray[i]);
        	}
        	return bladeEventList;
		} catch (Exception e) {
			
		}
		return null;
    }

    /* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.FilterDataControl#getBladeOneEventValues(java.lang.String, java.lang.String, java.lang.String)
	 */
    public Map<String, List<BladeEvent>> getBladeAllEventsValues(
    		String testName, 
    		String bladeId, 
    		String eventTypeLabel) 
    		throws ClifException {
    	try {
    		String[] allEventsArray = 
        		dsItf.getEventsTypeLabels(testName, bladeId);
    		//retrieve the blade data in a map<eventTypeLabel, values>
        	Map<String, List<BladeEvent>> allEventsValues = 
        		new HashMap<String, List<BladeEvent>>();
        	for (int i = 0; i < allEventsArray.length; i++) {
        		allEventsValues.put(allEventsArray[i], getBladeOneEventValues(testName, bladeId, allEventsArray[i]));
        	}
        	return allEventsValues;
		} catch (Exception e) {
			
		}
		return null;
    }
   
    
    /* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.FilterDataControl#getBladeOneEventValues(java.lang.String, java.lang.String, java.lang.String, java.util.map, java.lang.string)
	 */
    public List<BladeEvent> getBladeAllEventsValuesFiltered(
    		String testName, 
    		String bladeId, 
    		String eventTypeLabel,
    		Map<String, Map<String, String>> filter,
    		String linkingOp){
    	if(linkingOp.equals("and")){
    		try {
    			//get blade event list
    			List<BladeEvent> bladeEventList = getBladeOneEventValues(testName, bladeId, eventTypeLabel);
    			return getBladeAllEventsValuesFilteredAnd(
    					filter,
    					bladeEventList);
    		} catch (Exception e) {
    		
    		}
    	}
    	else if (linkingOp.equals("or")){
    		try {
    			Set<Entry<String, Map<String, String>>> set = filter.entrySet();
    			Iterator<Entry<String, Map<String, String>>> iterator = set.iterator();
    			Map<String, String> operatorFilter = new HashMap<String, String>(); 
    			List<BladeEvent> bladeEventListCurrent = new ArrayList<BladeEvent>();
    			List<List<BladeEvent>> bladeEventListList = new ArrayList<List<BladeEvent>>();
    			String fieldLabel;
    			String fieldFilterKeyword;
    			String filterOperator;
    			while(iterator.hasNext()){
    				Map.Entry<String,Map<String, String>> me = 
    					(Map.Entry<String,Map<String, String>>)iterator.next();
    				fieldLabel = me.getKey();
    				operatorFilter = me.getValue();
    				Set<Entry<String, String>> setF = operatorFilter.entrySet();
    				Iterator<Entry<String, String>> iteratorF = setF.iterator();
    				while (iteratorF.hasNext()) {
    					Map.Entry<String, String> entry = 
    						(Map.Entry<String, String>) iteratorF.next();
    					fieldFilterKeyword = entry.getKey();
    					filterOperator = entry.getValue();
    					bladeEventListCurrent = fieldFilter(
    							testName, 
    							bladeId, 
    							eventTypeLabel, 
    							fieldLabel, 
    							fieldFilterKeyword, 
    							filterOperator);
    					bladeEventListList.add(bladeEventListCurrent);
    				}
    			}
    				return  getBladeAllEventsValuesFilteredOr(
    						testName,
    						bladeId,
    						eventTypeLabel,
    						bladeEventListList,
    						linkingOp);
    		}
    		catch (Exception e) {
    		
    		}
    	}
    	return null;
    }
    
    
    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.virtualdata.FilterDataControl#getBladeAllEventsValuesFiltered(java.util.List, java.util.Map, java.lang.String)
     */
    public List<BladeEvent> getBladeAllEventsValuesFiltered(
    		List<BladeEvent> bladeEventList,
    		Map<String, Map<String, String>> filter,
    		String linkingOp){
    	if(linkingOp.equals("and")){
    		try {
    			return getBladeAllEventsValuesFilteredAnd(
    					filter,
    					bladeEventList);
    		} catch (Exception e) {
    		
    		}
    	}
    	else if (linkingOp.equals("or")){
    		try {
    			Set<Entry<String, Map<String, String>>> set = filter.entrySet();
    			Iterator<Entry<String, Map<String, String>>> iterator = set.iterator();
    			Map<String, String> operatorFilter = new HashMap<String, String>(); 
    			List<BladeEvent> bladeEventListCurrent = new ArrayList<BladeEvent>();
    			List<List<BladeEvent>> bladeEventListList = new ArrayList<List<BladeEvent>>();
    			String fieldLabel;
    			String fieldFilterKeyword;
    			String filterOperator;
    			while(iterator.hasNext()){
    				Map.Entry<String,Map<String, String>> me = 
    					(Map.Entry<String,Map<String, String>>)iterator.next();
    				fieldLabel = me.getKey();
    				operatorFilter = me.getValue();
    				Set<Entry<String, String>> setF = operatorFilter.entrySet();
    				Iterator<Entry<String, String>> iteratorF = setF.iterator();
    				while (iteratorF.hasNext()) {
    					Map.Entry<String, String> entry = 
    						(Map.Entry<String, String>) iteratorF.next();
    					fieldFilterKeyword = entry.getKey();
    					filterOperator = entry.getValue();
    					bladeEventListCurrent = fieldFilter(
    							bladeEventList,
    							fieldLabel, 
    							fieldFilterKeyword, 
    							filterOperator);
    					bladeEventListList.add(bladeEventListCurrent);
    				}
    			}
    				return  getBladeAllEventsValuesFilteredOr(
    						bladeEventListList,
    						linkingOp);
    		}
    		catch (Exception e) {
    		
    		}
    	}
    	return null;
    }
    
    
    /* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.FilterDataControl#getBladeOneEventValues(java.lang.String, java.lang.String, java.lang.String, java.util.map, java.util.list)
	 */
    //TO MODIFY
    public List<BladeEvent> getBladeAllEventsValuesFilteredAnd(
    		String testName,
    		String bladeId,
    		String eventTypeLabel,
    		Map<String, Map<String, String>> filter,
    		List<BladeEvent> bladeEventList){
    	try {
        	Set<Entry<String, Map<String, String>>> set = filter.entrySet();
    		Iterator<Entry<String, Map<String, String>>> iterator = set.iterator();
    		Map<String, String> operatorFilter = new HashMap<String, String>(); 
    		List<BladeEvent> bladeEventListCurrent = new ArrayList<BladeEvent>();
//    		Set<BladeEvent> bladeEventListResult = new HashSet<BladeEvent>(); 
    		String fieldLabel;
    		String fieldFilterKeyword;
    		String filterOperator;
    		bladeEventListCurrent = bladeEventList;
    		
    		while(iterator.hasNext()){
    			Map.Entry<String,Map<String, String>> me = 
    				(Map.Entry<String,Map<String, String>>)iterator.next();
    			//field
    			fieldLabel = me.getKey();
    			//operator : keyword and filter operator
    			operatorFilter = me.getValue();
    			Set<Entry<String, String>> setF = operatorFilter.entrySet();
    			Iterator<Entry<String, String>> iteratorF = setF.iterator();
    			while (iteratorF.hasNext()) {
    				Map.Entry<String, String> entry = 
    					(Map.Entry<String, String>) iteratorF.next();
    				fieldFilterKeyword = entry.getKey();
    				filterOperator = entry.getValue();
    				//recursive appel allow to filter with "and"
    				bladeEventListCurrent = fieldFilter(
    						bladeEventListCurrent, 
    						fieldLabel, 
    						fieldFilterKeyword, 
    						filterOperator);
//    				bladeEventListResult.addAll(bladeEventListCurrent);
    			}
    		}
//    		List<BladeEvent> beList = new ArrayList<BladeEvent>();
//    		Iterator<BladeEvent> beIte = bladeEventListResult.iterator();
//    		while (beIte.hasNext()) {
//				BladeEvent bladeEvent = (BladeEvent) beIte.next();
//				beList.add(bladeEvent);
//			}
//        	return beList;
    		return bladeEventListCurrent;
		} catch (Exception e) {
			
		}
    	return null;
    }
    
    
    public List<BladeEvent> getBladeAllEventsValuesFilteredAnd(
    		Map<String, Map<String, String>> filter,
    		List<BladeEvent> bladeEventList){
    	try {
        	Set<Entry<String, Map<String, String>>> set = filter.entrySet();
    		Iterator<Entry<String, Map<String, String>>> iterator = set.iterator();
    		Map<String, String> operatorFilter = new HashMap<String, String>(); 
    		List<BladeEvent> bladeEventListCurrent = new ArrayList<BladeEvent>();
//    		Set<BladeEvent> bladeEventListResult = new HashSet<BladeEvent>(); 
    		String fieldLabel;
    		String fieldFilterKeyword;
    		String filterOperator;
    		bladeEventListCurrent = bladeEventList;
    		
    		while(iterator.hasNext()){
    			Map.Entry<String,Map<String, String>> me = 
    				(Map.Entry<String,Map<String, String>>)iterator.next();
    			//field
    			fieldLabel = me.getKey();
    			//operator : keyword and filter operator
    			operatorFilter = me.getValue();
    			Set<Entry<String, String>> setF = operatorFilter.entrySet();
    			Iterator<Entry<String, String>> iteratorF = setF.iterator();
    			while (iteratorF.hasNext()) {
    				Map.Entry<String, String> entry = 
    					(Map.Entry<String, String>) iteratorF.next();
    				fieldFilterKeyword = entry.getKey();
    				filterOperator = entry.getValue();
    				//recursive appel allow to filter with "and"
    				bladeEventListCurrent = fieldFilter(
    						bladeEventListCurrent, 
    						fieldLabel, 
    						fieldFilterKeyword, 
    						filterOperator);
//    				bladeEventListResult.addAll(bladeEventListCurrent);
    			}
    		}
//    		List<BladeEvent> beList = new ArrayList<BladeEvent>();
//    		Iterator<BladeEvent> beIte = bladeEventListResult.iterator();
//    		while (beIte.hasNext()) {
//				BladeEvent bladeEvent = (BladeEvent) beIte.next();
//				beList.add(bladeEvent);
//			}
//        	return beList;
    		return bladeEventListCurrent;
		} catch (Exception e) {
			
		}
    	return null;
    }
    
    /* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.FilterDataControl#getBladeOneEventValues(java.lang.String, java.lang.String, java.lang.String, java.util.list, java.lang.string)
	 */
    public List<BladeEvent> getBladeAllEventsValuesFilteredOr(
    		String testName,
    		String bladeId,
    		String eventTypeLabel,
    		List<List<BladeEvent>> bladeEventListList,
    		String linkingOp){
    	try {
    		Comparator<BladeEvent> SORT_BY_TIME = new DateComp();
    		TreeSet<BladeEvent> bladeTreeSet = new TreeSet<BladeEvent>(SORT_BY_TIME);
    		Iterator <List<BladeEvent>> iterator = bladeEventListList.iterator();
        	while (iterator.hasNext()) {
    			List<BladeEvent> bladeEventList = (List<BladeEvent>) iterator.next();
    			Iterator<BladeEvent> bladeEventIterator = bladeEventList.iterator();
    			while (bladeEventIterator.hasNext()) {
					BladeEvent bladeEvent = (BladeEvent) bladeEventIterator.next();
					bladeTreeSet.add(bladeEvent);
				}
    		}
        	Iterator<BladeEvent> iterator2 = bladeTreeSet.iterator();
        	List<BladeEvent> bladeEventList = new ArrayList<BladeEvent>();
        	while (iterator2.hasNext()) {
				BladeEvent bladeEvent = (BladeEvent) iterator2.next();
				bladeEventList.add(bladeEvent);
			}
        	return bladeEventList;
		} catch (Exception e) {
			
		}
    	return null;
    }
    
    /**
     * Filter by or operator
     * 
     * @param bladeEventListList
     * @param linkingOp
     * @return bladeEvent list filtered by or operator
     */
    public List<BladeEvent> getBladeAllEventsValuesFilteredOr(
    		List<List<BladeEvent>> bladeEventListList,
    		String linkingOp){
    	try {
    		Comparator<BladeEvent> SORT_BY_TIME = new DateComp();
    		TreeSet<BladeEvent> bladeTreeSet = new TreeSet<BladeEvent>(SORT_BY_TIME);
    		Iterator <List<BladeEvent>> iterator = bladeEventListList.iterator();
        	while (iterator.hasNext()) {
    			List<BladeEvent> bladeEventList = (List<BladeEvent>) iterator.next();
    			Iterator<BladeEvent> bladeEventIterator = bladeEventList.iterator();
    			while (bladeEventIterator.hasNext()) {
					BladeEvent bladeEvent = (BladeEvent) bladeEventIterator.next();
					bladeTreeSet.add(bladeEvent);
				}
    		}
        	Iterator<BladeEvent> iterator2 = bladeTreeSet.iterator();
        	List<BladeEvent> bladeEventList = new ArrayList<BladeEvent>();
        	while (iterator2.hasNext()) {
				BladeEvent bladeEvent = (BladeEvent) iterator2.next();
				bladeEventList.add(bladeEvent);
			}
        	return bladeEventList;
		} catch (Exception e) {
			
		}
    	return null;
    }
   
    
	/* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.FilterDataControl#getCommonMinTime(java.lang.String)
	 */
	public long getCommonMinTime(String testName) {
		int commonMinTime = 0;
		String[] bladeIds = dsItf.getBladesIds(testName);
		int currentMinTime;
		if(bladeIds[0] != null){
			commonMinTime = (int) dsItf.getLifecycleRunning(testName, bladeIds[0]);
		}
		for (int i = 0; i < bladeIds.length; i++) {
			currentMinTime = (int) dsItf.getLifecycleRunning(testName, bladeIds[i]);
			if(currentMinTime > commonMinTime){
				commonMinTime = currentMinTime;
			}
		}
		return commonMinTime;
	} 

	/* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.FilterDataControl#getCommonMaxTime(java.lang.String)
	 */
	public long getCommonMaxTime(String testName) {
		int commonMaxTime = 0;
		String[] bladeIds = dsItf.getBladesIds(testName);
		int currentMaxTime;
		if(bladeIds[0] != null){
			commonMaxTime = (int) dsItf.getLifecycleCompleted(testName, bladeIds[0]);
		}
		for (int i = 0; i < bladeIds.length; i++) {
			currentMaxTime = (int) dsItf.getLifecycleCompleted(testName, bladeIds[i]);
			if(currentMaxTime < commonMaxTime){
				commonMaxTime = currentMaxTime;
			}
		}
		return commonMaxTime;
	}
	   
	/* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.FilterDataControl#getBladeEvents(java.lang.String, java.lang.String, java.lang.String, filter, long, int)
	 */
	public BladeEvent[] getBladeEvents(
			String testName,	
			String bladeId, 
			String eventTypeLabel,
			EventFilter filter, 
			long fromIndex,
			int count
			)throws ClifException{
		return dsItf.getBladeEvents(testName, bladeId, eventTypeLabel, filter, fromIndex, count);
	} 
	
	/* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.FilterDataControl#getBladeEventsValues(BladeEvent[], java.lang.String, java.lang.String)
	 */
	public Object[][] getBladeEventsValues(BladeEvent[] bladeEventList, String XFieldLabel, String YFieldLabel){
		return dsItf.getBladeEventsValues(bladeEventList, XFieldLabel, YFieldLabel);
	}	
	
	
    ///////////////////////
    // private functions //
    ///////////////////////
	
	/* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.FilterDataControl#fieldFilter(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	public List<BladeEvent> fieldFilter(
			String testName, 
			String bladeId, 
			String eventTypeLabel, 
			String fieldLabel, 
			String fieldFilterKeyword, 
			String filterOperator){
		if(filterOperator.equals(OPERATOR_EQUALS)){
			return fieldFilterEquals(testName, bladeId, eventTypeLabel, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_NOT_EQUALS)){
			return fieldFilterNotEquals(testName, bladeId, eventTypeLabel, fieldLabel, fieldFilterKeyword);
		}
		else 
			if(filterOperator.equals(OPERATOR_CONTAINS)){
			return fieldFilterContains(testName, bladeId, eventTypeLabel, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_NOT_CONTAINS)){
			return fieldFilterNotContains(testName, bladeId, eventTypeLabel, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_BEGINS_WITH)){
			return fieldFilterBeginsWith(testName, bladeId, eventTypeLabel, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_ENDS_WITH)){
			return fieldFilterEndsWith(testName, bladeId, eventTypeLabel, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_GREATER_THAN)){
			return fieldFilterGreaterThan(testName, bladeId, eventTypeLabel, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_LESS_THAN)){
			return fieldFilterLessThan(testName, bladeId, eventTypeLabel, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_GREATER_THAN_OR_EQUALS)){
			return fieldFilterGreaterThanOrEquals(testName, bladeId, eventTypeLabel, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_LESS_THAN_OR_EQUALS)){
			return fieldFilterLessThanOrEquals(testName, bladeId, eventTypeLabel, fieldLabel, fieldFilterKeyword);
		}
		else return null;
		
	}

	/* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.FilterDataControl#fieldFilter(java.util.list, java.lang.String, java.lang.String, java.lang.String)
	 */
	public List<BladeEvent> fieldFilter(
			List<BladeEvent> bladeEventList,
			String fieldLabel, 
			String fieldFilterKeyword, 
			String filterOperator){
		if(filterOperator.equals(OPERATOR_EQUALS)){
			return fieldFilterEquals(bladeEventList, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_NOT_EQUALS)){
			return fieldFilterNotEquals(bladeEventList, fieldLabel, fieldFilterKeyword);
		}
		else 
			if(filterOperator.equals(OPERATOR_CONTAINS)){
			return fieldFilterContains(bladeEventList, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_NOT_CONTAINS)){
			return fieldFilterNotContains(bladeEventList, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_BEGINS_WITH)){
			return fieldFilterBeginsWith(bladeEventList, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_ENDS_WITH)){
			return fieldFilterEndsWith(bladeEventList, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_GREATER_THAN)){
			return fieldFilterGreaterThan(bladeEventList, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_LESS_THAN)){
			return fieldFilterLessThan(bladeEventList, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_GREATER_THAN_OR_EQUALS)){
			return fieldFilterGreaterThanOrEquals(bladeEventList, fieldLabel, fieldFilterKeyword);
		}
		else if(filterOperator.equals(OPERATOR_LESS_THAN_OR_EQUALS)){
			return fieldFilterLessThanOrEquals(bladeEventList, fieldLabel, fieldFilterKeyword);
		}
		else return null;
		
	}
	
	
	

	private List<BladeEvent> fieldFilterLessThanOrEquals(String testName, String bladeId,
			String eventTypeLabel, String fieldLabel, String fieldFilterKeyword) {
		BladeEvent[] bladeEventList = null;
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		try{
			bladeEventList = getBladeEvents(testName, bladeId, eventTypeLabel, null, 0, -1);
		}
		catch(ClifException ce){

		}
		if(bladeEventList != null){
			for (int i = 0; i < bladeEventList.length; i++) {
				fieldValue = (bladeEventList[i].getFieldValue(fieldLabel)).toString();
				try {
					double intFieldValue = Double.parseDouble(fieldValue);
					double intFieldFilterKeyword = Double.parseDouble(fieldFilterKeyword);
					if(intFieldValue <= intFieldFilterKeyword){
						bladeEventSetListFilter.add(bladeEventList[i]);
					}
				}
				catch (Exception e) {
					
				}
			}
		}
		return bladeEventSetListFilter;
	}


	private List<BladeEvent> fieldFilterGreaterThanOrEquals(String testName,
			String bladeId, String eventTypeLabel, String fieldLabel,
			String fieldFilterKeyword) {
		BladeEvent[] bladeEventList = null;
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		try{
			bladeEventList = getBladeEvents(testName, bladeId, eventTypeLabel, null, 0, -1);
		}
		catch(ClifException ce){

		}
		if(bladeEventList != null){
			for (int i = 0; i < bladeEventList.length; i++) {
				fieldValue = (bladeEventList[i].getFieldValue(fieldLabel)).toString();
				try {
					double intFieldValue = Double.parseDouble(fieldValue);
					double intFieldFilterKeyword = Double.parseDouble(fieldFilterKeyword);
					if(intFieldValue >= intFieldFilterKeyword){
						bladeEventSetListFilter.add(bladeEventList[i]);
					}
				}
				catch (Exception e) {
		
				}
			}
		}
		return bladeEventSetListFilter;
	}


	private List<BladeEvent> fieldFilterLessThan(String testName, String bladeId,
			String eventTypeLabel, String fieldLabel, String fieldFilterKeyword) {
		BladeEvent[] bladeEventList = null;
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		try{
			bladeEventList = getBladeEvents(testName, bladeId, eventTypeLabel, null, 0, -1);
		}
		catch(ClifException ce){

		}
		if(bladeEventList != null){
			for (int i = 0; i < bladeEventList.length; i++) {
				fieldValue = (bladeEventList[i].getFieldValue(fieldLabel)).toString();
				try {
					double intFieldValue = Double.parseDouble(fieldValue);
					double intFieldFilterKeyword = Double.parseDouble(fieldFilterKeyword);
					if(intFieldValue < intFieldFilterKeyword){
						bladeEventSetListFilter.add(bladeEventList[i]);
					}
				}
				catch (Exception e) {
					
				}
			}
		}
		return bladeEventSetListFilter;
	}


	private List<BladeEvent> fieldFilterGreaterThan(String testName, String bladeId,
			String eventTypeLabel, String fieldLabel, String fieldFilterKeyword) {
		BladeEvent[] bladeEventList = null;
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		try{
			bladeEventList = getBladeEvents(testName, bladeId, eventTypeLabel, null, 0, -1);
		}
		catch(ClifException ce){

		}
		if(bladeEventList != null){
			for (int i = 0; i < bladeEventList.length; i++) {
				fieldValue = (bladeEventList[i].getFieldValue(fieldLabel)).toString();
				try {
					double intFieldValue = Double.parseDouble(fieldValue);
					double intFieldFilterKeyword = Double.parseDouble(fieldFilterKeyword);
					if(intFieldValue > intFieldFilterKeyword){
						bladeEventSetListFilter.add(bladeEventList[i]);
					}
				}
				catch (Exception e) {
					
				}
			}
		}
		return bladeEventSetListFilter;
	}


	private List<BladeEvent> fieldFilterEndsWith(String testName, String bladeId,
			String eventTypeLabel, String fieldLabel, String fieldFilterKeyword) {
		BladeEvent[] bladeEventList = null;
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		try{
			bladeEventList = getBladeEvents(testName, bladeId, eventTypeLabel, null, 0, -1);
		}
		catch(ClifException ce){

		}
		if(bladeEventList != null){
			for (int i = 0; i < bladeEventList.length; i++) {
				fieldValue = (bladeEventList[i].getFieldValue(fieldLabel)).toString();
				boolean test = fieldValue.endsWith(fieldFilterKeyword);
				if(test == true){
					bladeEventSetListFilter.add(bladeEventList[i]);
				}
			}
		}
		return bladeEventSetListFilter;
	}


	private List<BladeEvent> fieldFilterBeginsWith(String testName, String bladeId,
			String eventTypeLabel, String fieldLabel, String fieldFilterKeyword) {
		BladeEvent[] bladeEventList = null;
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		try{
			bladeEventList = getBladeEvents(testName, bladeId, eventTypeLabel, null, 0, -1);
		}
		catch(ClifException ce){

		}
		if(bladeEventList != null){
			for (int i = 0; i < bladeEventList.length; i++) {
				fieldValue = (bladeEventList[i].getFieldValue(fieldLabel)).toString();
				boolean test = fieldValue.startsWith(fieldFilterKeyword);
				if(test == true){
					bladeEventSetListFilter.add(bladeEventList[i]);
				}
			}
		}
		return bladeEventSetListFilter;
	}


	private List<BladeEvent> fieldFilterNotContains(String testName, String bladeId,
			String eventTypeLabel, String fieldLabel, String fieldFilterKeyword) {
		BladeEvent[] bladeEventList = null;
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		try{
			bladeEventList = getBladeEvents(testName, bladeId, eventTypeLabel, null, 0, -1);
		}
		catch(ClifException ce){

		}
		if(bladeEventList != null){
			for (int i = 0; i < bladeEventList.length; i++) {
				fieldValue = (bladeEventList[i].getFieldValue(fieldLabel)).toString();
				boolean test = fieldValue.contains(fieldFilterKeyword);
				if(test == false){
					bladeEventSetListFilter.add(bladeEventList[i]);
				}
			}
		}
		return bladeEventSetListFilter;
	}


	private List<BladeEvent> fieldFilterContains(String testName, String bladeId,
			String eventTypeLabel, String fieldLabel, String fieldFilterKeyword) {
		BladeEvent[] bladeEventList = null;
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		try{
			bladeEventList = getBladeEvents(testName, bladeId, eventTypeLabel, null, 0, -1);
		}
		catch(ClifException ce){

		}
		if(bladeEventList != null){
			for (int i = 0; i < bladeEventList.length; i++) {
				fieldValue = (bladeEventList[i].getFieldValue(fieldLabel)).toString();
				boolean test = fieldValue.contains(fieldFilterKeyword);
				if(test == true){
					bladeEventSetListFilter.add(bladeEventList[i]);
				}
			}
		}
		return bladeEventSetListFilter;
	}


	private List<BladeEvent> fieldFilterNotEquals(String testName, String bladeId,
			String eventTypeLabel, String fieldLabel, String fieldFilterKeyword) {
		BladeEvent[] bladeEventList = null;
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		Pattern pattern;
		Matcher matcher;
		try{
			bladeEventList = getBladeEvents(testName, bladeId, eventTypeLabel, null, 0, -1);
		}
		catch(ClifException ce){

		}
		if(bladeEventList != null){
			for (int i = 0; i < bladeEventList.length; i++) {
				fieldValue = (bladeEventList[i].getFieldValue(fieldLabel)).toString();
				pattern = Pattern.compile(fieldValue);
				matcher = pattern.matcher(fieldFilterKeyword);
				boolean test = matcher.matches();
				if(test == false){
					bladeEventSetListFilter.add(bladeEventList[i]);
				}
			}
		}
		return bladeEventSetListFilter;
	}


	private List<BladeEvent> fieldFilterEquals(String testName, String bladeId,
			String eventTypeLabel, String fieldLabel, String fieldFilterKeyword) {
		BladeEvent[] bladeEventList = null;
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		Pattern pattern;
		Matcher matcher;
		try{
			bladeEventList = getBladeEvents(testName, bladeId, eventTypeLabel, null, 0, -1);
		}
		catch(ClifException ce){

		}
		if(bladeEventList != null){
			for (int i = 0; i < bladeEventList.length; i++) {
				fieldValue = (bladeEventList[i].getFieldValue(fieldLabel)).toString();
				pattern = Pattern.compile(fieldValue);
				matcher = pattern.matcher(fieldFilterKeyword);
				boolean test = matcher.matches();
				if(test == true){
					bladeEventSetListFilter.add(bladeEventList[i]);
				}
			}
		}
		return bladeEventSetListFilter;
	}
	
	
    
	private List<BladeEvent> fieldFilterLessThanOrEquals(List<BladeEvent> bladeEventList,String fieldLabel, String fieldFilterKeyword) {
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		Iterator<BladeEvent> iterator = bladeEventList.iterator();
		while (iterator.hasNext()) {
			BladeEvent bladeEvent = (BladeEvent) iterator.next();
			fieldValue = bladeEvent.getFieldValue(fieldLabel).toString();
			try {
				double intFieldValue = Double.parseDouble(fieldValue);
				double intFieldFilterKeyword = Double.parseDouble(fieldFilterKeyword);
				if(intFieldValue <= intFieldFilterKeyword){
					bladeEventSetListFilter.add(bladeEvent);
				}
			}
			catch (Exception e) {
				
			}
		}
		return bladeEventSetListFilter;
	}


	private List<BladeEvent> fieldFilterGreaterThanOrEquals(List<BladeEvent> bladeEventList,String fieldLabel, String fieldFilterKeyword) {
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		Iterator<BladeEvent> iterator = bladeEventList.iterator();
		while (iterator.hasNext()) {
			BladeEvent bladeEvent = (BladeEvent) iterator.next();
			fieldValue = bladeEvent.getFieldValue(fieldLabel).toString();
			try {
				double intFieldValue = Double.parseDouble(fieldValue);
				double intFieldFilterKeyword = Double.parseDouble(fieldFilterKeyword);
				if(intFieldValue >= intFieldFilterKeyword){
					bladeEventSetListFilter.add(bladeEvent);
				}
			}
			catch (Exception e) {
				
			}
		}
		return bladeEventSetListFilter;
	}

	private List<BladeEvent> fieldFilterLessThan(List<BladeEvent> bladeEventList,String fieldLabel, String fieldFilterKeyword) {
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		Iterator<BladeEvent> iterator = bladeEventList.iterator();
		while (iterator.hasNext()) {
			BladeEvent bladeEvent = (BladeEvent) iterator.next();
			fieldValue = bladeEvent.getFieldValue(fieldLabel).toString();
			try {
				double intFieldValue = Double.parseDouble(fieldValue);
				double intFieldFilterKeyword = Double.parseDouble(fieldFilterKeyword);
				if(intFieldValue < intFieldFilterKeyword){
					bladeEventSetListFilter.add(bladeEvent);
				}
			}
			catch (Exception e) {
				
			}
		}
		return bladeEventSetListFilter;
	}

	private List<BladeEvent> fieldFilterGreaterThan(List<BladeEvent> bladeEventList,String fieldLabel, String fieldFilterKeyword) {
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		Iterator<BladeEvent> iterator = bladeEventList.iterator();
		while (iterator.hasNext()) {
			BladeEvent bladeEvent = (BladeEvent) iterator.next();
			fieldValue = bladeEvent.getFieldValue(fieldLabel).toString();
			try {
				double intFieldValue = Double.parseDouble(fieldValue);
				double intFieldFilterKeyword = Double.parseDouble(fieldFilterKeyword);
				if(intFieldValue > intFieldFilterKeyword){
					bladeEventSetListFilter.add(bladeEvent);
				}
			}
			catch (Exception e) {
				
			}
		}
		return bladeEventSetListFilter;
	}
	
	private List<BladeEvent> fieldFilterEndsWith(List<BladeEvent> bladeEventList,String fieldLabel, String fieldFilterKeyword) {
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		Iterator<BladeEvent> iterator = bladeEventList.iterator();
		while (iterator.hasNext()) {
			BladeEvent bladeEvent = (BladeEvent) iterator.next();
			fieldValue = bladeEvent.getFieldValue(fieldLabel).toString();
			boolean test = fieldValue.endsWith(fieldFilterKeyword);
			if(test == true){
				bladeEventSetListFilter.add(bladeEvent);
			}
		}
		return bladeEventSetListFilter;
	}



	private List<BladeEvent> fieldFilterBeginsWith(List<BladeEvent> bladeEventList,String fieldLabel, String fieldFilterKeyword) {
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		Iterator<BladeEvent> iterator = bladeEventList.iterator();
		while (iterator.hasNext()) {
			BladeEvent bladeEvent = (BladeEvent) iterator.next();
			fieldValue = bladeEvent.getFieldValue(fieldLabel).toString();
			boolean test = fieldValue.startsWith(fieldFilterKeyword);
			if(test == true){
				bladeEventSetListFilter.add(bladeEvent);
			}
		}
		return bladeEventSetListFilter;
	}
	

	private List<BladeEvent> fieldFilterNotContains(List<BladeEvent> bladeEventList,String fieldLabel, String fieldFilterKeyword) {
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		Iterator<BladeEvent> iterator = bladeEventList.iterator();
		while (iterator.hasNext()) {
			BladeEvent bladeEvent = (BladeEvent) iterator.next();
			fieldValue = bladeEvent.getFieldValue(fieldLabel).toString();
			boolean test = fieldValue.contains(fieldFilterKeyword);
			if(test == false){
				bladeEventSetListFilter.add(bladeEvent);
			}
		}
		return bladeEventSetListFilter;
	}
	
	
	private List<BladeEvent> fieldFilterContains(List<BladeEvent> bladeEventList,String fieldLabel, String fieldFilterKeyword) {
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		Iterator<BladeEvent> iterator = bladeEventList.iterator();
		while (iterator.hasNext()) {
			BladeEvent bladeEvent = (BladeEvent) iterator.next();
			fieldValue = bladeEvent.getFieldValue(fieldLabel).toString();
			boolean test = fieldValue.contains(fieldFilterKeyword);
			if(test == true){
				bladeEventSetListFilter.add(bladeEvent);
			}
		}
		return bladeEventSetListFilter;
	}

	
	private List<BladeEvent> fieldFilterNotEquals(List<BladeEvent> bladeEventList,String fieldLabel, String fieldFilterKeyword) {
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		Pattern pattern;
		Matcher matcher;
		Iterator<BladeEvent> iterator = bladeEventList.iterator();
		while (iterator.hasNext()) {
			BladeEvent bladeEvent = (BladeEvent) iterator.next();
			fieldValue = bladeEvent.getFieldValue(fieldLabel).toString();
			pattern = Pattern.compile(fieldValue);
			matcher = pattern.matcher(fieldFilterKeyword);
			boolean test = matcher.matches();
			if(test == false){
				bladeEventSetListFilter.add(bladeEvent);
			}
		}
		return bladeEventSetListFilter;
	}


	private List<BladeEvent> fieldFilterEquals(List<BladeEvent> bladeEventList,String fieldLabel, String fieldFilterKeyword) {
		String fieldValue = null;
		List<BladeEvent> bladeEventSetListFilter = new ArrayList<BladeEvent>();
		Pattern pattern;
		Matcher matcher;
		Iterator<BladeEvent> iterator = bladeEventList.iterator();
		while (iterator.hasNext()) {
			BladeEvent bladeEvent = (BladeEvent) iterator.next();
			fieldValue = bladeEvent.getFieldValue(fieldLabel).toString();
			pattern = Pattern.compile(fieldValue);
			matcher = pattern.matcher(fieldFilterKeyword);
			boolean test = matcher.matches();
			if(test == true){
				bladeEventSetListFilter.add(bladeEvent);
			}
		}
		return bladeEventSetListFilter;
	}
	
	/////////////////////////////////
    // Class DateComp //
    /////////////////////////////////
    private class DateComp implements Comparator<BladeEvent> {
    	//sort by time
    	public int compare(BladeEvent bladeEvent1, BladeEvent bladeEvent2){
    		if(bladeEvent1.getDate()<bladeEvent2.getDate()){
    			return -1;
    		}
    		else if(bladeEvent1.getDate()>bladeEvent2.getDate()){
    			return 1;
    		}
    		else{
    			return 0;
    		}
    	}
    }


}
