/*
* CLIF is a Load Injection Framework
* Copyright (C) 2008 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.lib.graph;

import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;
import org.apache.commons.math.stat.StatUtils;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.ow2.clif.analyze.api.graph.DataSupplier;
import org.ow2.clif.analyze.api.graph.virtualdata.StatisticDataControl;
import org.ow2.clif.analyze.statistics.util.data.ListOfLong;
import org.ow2.clif.analyze.statistics.util.data.LongStatistics;
import org.ow2.clif.analyze.statistics.util.data.StatOnLongs;

/**
 * @author Olivier Liu
 */
public class StatisticDataImpl implements BindingController, StatisticDataControl {
	/**
	 * The Class StatisticDataImpl
	 */

	    /** The instance. */
	    private static StatisticDataImpl instance= new StatisticDataImpl();
	    
	    /** The interface list. */
	    static private String[] itfList = new String[] { DataSupplier.DATA_SUPPLIER };

	    // field for client binding (access to stored data)
	    /** The data supplier interface. */
	    private DataSupplier dsItf;
	    
	    private String testName;
	    
	    private String bladeId;
	    
	    //name complete for this blade 
	    private String bladeIdAlias = "";
	    
	    //0 : simpledataset, 1 : multiple dataset, 2 : aggregate dataset
	    private int datasetType;
	    
	    private String eventType;
	    
	    private String field;
	    
	    private StatOnLongs stolDate;
	    
	    private StatOnLongs stol;
	    
	    private List<FieldFilterAndOp> ffo;
	    
	    private double minimum;
	    
	    private double maximum;
	    
	    private double median;
	    
	    private double mean;
	    
	    private double standardDeviation;
	    
	    private String logicalOp;
	    
	    private int timeIntervalStart;
	    
	    private int timeIntervalEnd;
	    
	    /**
	     * Instantiates a new chart engine impl.
	     */
	    public StatisticDataImpl()
	    { 
	       //empty
	    }
	    
	    public StatisticDataImpl(
	    		String testName, 
	    		String bladeId, 
	    		String eventType, 
	    		String field, 
	    		StatOnLongs stolDate, 
	    		StatOnLongs stol, 
	    		List<FieldFilterAndOp> ffo,
	    		String logicalOp,
	    		String bladeIdAlias,
	    		int datasetType,
	    		int timeIntervalStart,
	    		int timeIntervalEnd){
	    	this.testName = testName;
	    	this.bladeId = bladeId;
	    	this.eventType = eventType;
	    	this.field = field;
	    	this.stolDate = stolDate;
	    	this.stol = stol;
	    	this.ffo = ffo;
	    	this.logicalOp = logicalOp;
	    	this.bladeIdAlias = bladeIdAlias;
	    	this.datasetType = datasetType;
	    	this.timeIntervalStart = timeIntervalStart;
	    	this.timeIntervalEnd = timeIntervalEnd;
	    	setMinimum();
	    	setMaximum();
	    	setMedian();
	    	setMean();
	    	setStdDvt();
	    }
	    
	    /**
	     * Gets the single instance of FilterDataImpl.
	     * 
	     * @return single instance of FilterDataImpl
	     */
	    public StatisticDataImpl getInstance()
	    {
	        return instance;
	    }


	    /////////////////////////////////
	    // interface BindingController //
	    /////////////////////////////////


	    
	    /* (non-Javadoc)
	     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
	     */
	    public void bindFc(String clientItfName, Object serverItf) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
	    {
	        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
	        {
	            dsItf = (DataSupplier)serverItf;
	        }
	    }
	    

	    /* (non-Javadoc)
	     * @see org.objectweb.fractal.api.control.BindingController#listFc()
	     */
	    public String[] listFc() 
	    {
	        return itfList;
	    }

	    
	    /* (non-Javadoc)
	     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
	     */
	    public Object lookupFc(String clientItfName) throws NoSuchInterfaceException 
	    {
	        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
	        {
	            return dsItf;
	        }
	        else
	        {
	            return null;
	        }
	    }

	    
	    /* (non-Javadoc)
	     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
	     */
	    public void unbindFc(String clientItfName) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
	    {
	        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
	        {
	            dsItf = null;
	        }
	    }

	    ////////////////////////////////////
	    // GLOBAL STAT                 //
	    ////////////////////////////////////
	    
	    /**
	     * Get throughput
	     * 
	     * @return throughput
	     */
	    public double getThroughput(){
	    	//calculate throughput per second
	    	double min = this.stolDate.getMin();
	    	double max = this.stolDate.getMax();
	    	double numberOfValues = this.stolDate.size();
	    	double throughput = numberOfValues*1000/(max - min);
	    	return throughput;
	    }
	    
	    /**
	     * Number of values
	     * 
	     * @return number of values for this 
	     */
	    public double getNumberOfValues(){
	    	return this.stolDate.size();
	    }
	    
	    /**
	     * Get minimum
	     * 
	     * @return minimum
	     */
	    public double getMinimum() {
			return minimum;
		}

		/**
		 * Get maximum
		 * 
		 * @return maximum
		 */
		public double getMaximum() {
			return maximum;
		}
		
		/**
		 * Get median
		 * 
		 * @return median
		 */
		public double getMedian() {
			return median;
		}
		
		/**
		 * Get mean
		 * 
		 * @return mean
		 */
		public double getMean(){
			return mean;
		}
		
		/**
		 * Get standard deviation
		 * 
		 * @return standard deviation
		 */
		public double getStdDvt(){
			return standardDeviation;
		}
		

		/**
		 * Set minimum 
		 */
		public void setMinimum() {
			this.minimum = this.stol.getMin();
		}

		
		/**
		 * Set maximum
		 */
		public void setMaximum() {
			this.maximum = this.stol.getMax();
		}
		
		/**
		 * Set median
		 */
		public void setMedian(){
			// to modify : with one values it doesn't work
			try {
				this.median = this.stol.getMedian();
			} catch (Exception e) {
				this.median = 0;
			}
		}
		
		/**
		 * Set mean
		 */
		public void setMean(){
			this.mean = this.stol.getMean();
		}
		
		/**
		 * Set standard deviation
		 */
		public void setStdDvt(){
			this.standardDeviation = this.stol.getStd();
		}

	    ////////////////////////////////////
	    // MOVING STAT                 //
	    ////////////////////////////////////

	    /**
	     * Get moving minimum according to time window and the step given
	     * @param timeWindow
	     * @param step
	     * @return moving minimum
	     */
	    public Object[][] getMovingMinimum(int timeWindow, int step) {
	    	 Object[][] minValues = null;
	         double[] vals = toDoubleArray(this.stol.getData());
	         Long[] splitTime = split(toDoubleArray(this.stolDate.getData()), timeWindow, step);
	         minValues = new Object[2][splitTime.length];
	         
	         for (int i = 0; i < splitTime.length; i++) {
	             minValues[0][i] = timeWindow + i * step;
	             if (i == splitTime.length-1)
	             {
	                 minValues[1][i] = StatUtils.min(vals, splitTime[i].intValue(), vals.length-splitTime[i].intValue());
	             }
	             else
	             {
	                 minValues[1][i] = StatUtils.min(vals, splitTime[i].intValue(), (splitTime[i+1].intValue()-splitTime[i].intValue()));
	             }
	         }
	         return minValues;
		}
		
	    /**
	     * Get moving maximum according to time window and the step given
	     * @param timeWindow
	     * @param step
	     * @return moving maximum
	     */
	    public Object[][] getMovingMaximum(int timeWindow, int step) 
	    {
	    	Object[][] maxValues = null;
	        double[] vals = toDoubleArray(this.stol.getData());
	        Long[] splitTime = split(toDoubleArray(this.stolDate.getData()), timeWindow, step);
	        maxValues = new Object[2][splitTime.length];
	        
	        for (int i = 0; i < splitTime.length; i++) {
	            maxValues[0][i] = timeWindow + i * step;
	            if (i == splitTime.length-1)
	            {
	                maxValues[1][i] = StatUtils.max(vals, splitTime[i].intValue(), vals.length-splitTime[i].intValue());
	            }
	            else
	            {
	                maxValues[1][i] = StatUtils.max(vals, splitTime[i].intValue(), (splitTime[i+1].intValue()-splitTime[i].intValue()));
	            }
	        }
	        
	        return maxValues;
	    }
	    
	    
	    /**
	     * Get moving mean according to time window and the step given
	     * @param timeWindow
	     * @param step
	     * @return moving mean
	     */
	    public Object[][] getMovingMeans(int timeWindow, int step) 
	    {
	        Object[][] meanValues = null;
	        double[] vals = toDoubleArray(this.stol.getData());
	        Long[] splitTime = split(toDoubleArray(this.stolDate.getData()), timeWindow, step);
	        meanValues = new Object[2][splitTime.length];
	        
	        for (int i = 0; i < splitTime.length; i++) {
	            meanValues[0][i] = timeWindow + i * step;
	            if (i == splitTime.length-1)
	            {
	                meanValues[1][i] = StatUtils.mean(vals, splitTime[i].intValue(), vals.length-splitTime[i].intValue());
	            }
	            else
	            {
	                meanValues[1][i] = StatUtils.mean(vals, splitTime[i].intValue(), (splitTime[i+1].intValue()-splitTime[i].intValue()));
	            }
	        }
	        
	        return meanValues;
	    }
	    
	    
	    /**
	     * Get moving standard deviation according to time window and the step given
	     * @return moving standard deviation
	     */
	    public Object[][] getMovingStandardDeviations(int timeWindow, int step) 
	    {
	        Object[][] stdDevValues = null;
	        double[] vals = toDoubleArray(this.stol.getData());
	        Long[] splitTime = split(toDoubleArray(this.stolDate.getData()), timeWindow, step);
	        stdDevValues = new Object[2][splitTime.length];
	        
	        for (int i = 0; i < splitTime.length; i++) {
	            stdDevValues[0][i] = timeWindow + i * step;
	            if (i == splitTime.length-1)
	            {
	                stdDevValues[1][i] = Math.sqrt(StatUtils.variance(vals, splitTime[i].intValue(), (vals.length-splitTime[i].intValue())));
	            }
	            else
	            {
	                stdDevValues[1][i] = Math.sqrt(StatUtils.variance(vals, splitTime[i].intValue(), (splitTime[i+1].intValue()-splitTime[i].intValue())));
	            }
	        }
	        
	        for (int i = 0; i < splitTime.length-1; i++) {
	            stdDevValues[0][i] = timeWindow + i * step;
	            stdDevValues[1][i] = Math.sqrt(StatUtils.variance(vals, splitTime[i].intValue(), (splitTime[i+1].intValue()-splitTime[i].intValue())));
	        }
	        
	        return stdDevValues;
	    }
	    
	    /**
	     * Get moving throughput according to time window and the step given
	     * @param timeWindow
	     * @param step
	     * @return moving throughput
	     */
	    public Object[][] getMovingThroughput(int timeWindow, int step) 
	    {
	    	// moving throughput per second
	    	Object[][] throughputValues = null;
	    	Long[] splitTime = split(toDoubleArray(this.stolDate.getData()), timeWindow, step);
	    	throughputValues = new Object[2][splitTime.length];
	    	double numberOfValues;
	    	int timeStartCurrent;
	    	int timeEndCurrent;
	    	
	    	for (int i = 0; i < splitTime.length; i++) {
	    		throughputValues[0][i] = timeWindow + i * step;
	            if (i < splitTime.length-1)
	            {
	            	timeStartCurrent = timeWindow + i * step;
	            	timeEndCurrent = timeWindow + 2 * i * step;
	            	numberOfValues = getNumberOfValues(this.stolDate.getData(), timeStartCurrent, timeEndCurrent);
	            	throughputValues[1][i] = numberOfValues*1000/(timeEndCurrent - timeStartCurrent);
	            }
	        }
	        return throughputValues;
	    }
		
	    
	    /**
	     * Get the number of values in interval
	     * @param data
	     * @param timeStartCurrent
	     * @param timeEndCurrent
	     * @return number of values in interval
	     */
	    private double getNumberOfValues(
	    		ListOfLong data, 
	    		int timeStartCurrent,
				int timeEndCurrent) {
	    	double numberOfValues = 0;
	    	boolean boo = true;
			Iterator<Long> dataIt = data.iterator();
			while (dataIt.hasNext() && boo == true) {
				Long longTmp = (Long) dataIt.next();
				if((longTmp >= timeStartCurrent) && (longTmp < timeEndCurrent)){
					numberOfValues ++;
				}
				if(longTmp >= timeEndCurrent){
					boo = false;
				}
			}
			return numberOfValues;
		}

		/**
	     * Get frequencies in a map that contains value for key and frequency for number of occurencies
	     * @param values
	     * @return frequencies for this values
	     */
	    public Map<Integer, Integer> frequency(double[] values){
	    	Map<Integer, Integer> mapResult = new TreeMap<Integer, Integer>();
	    	int key;
	    	int frequency;
	    	for (int i = 0; i < values.length; i++) {
	    		//give floor value
				key = (int) values[i];
				if(mapResult.get(key) == null){
					mapResult.put(key, 1);
				}
				else{
					frequency = mapResult.get(key);
					mapResult.put(key, frequency + 1);
				}
			}
	    	return mapResult;
	    }
	    
	    /**
	     * @param ls LongStatistics
	     * @param n the number of slices
	     * @return value for each slice
	     */
	    public double[] rawDataFrequency(LongStatistics ls, int n){
	    	ls = (LongStatistics) this.getStol();
	    	
	    	System.out.println("median" + ls.getMedian());
	    	
	    	return null;
	    }
	    
	    /**
	     * Get quantiles 
	     * @param n
	     * @param values
	     * @return array that contains the quantiles
	     */
	    public double[] quantiles(int n, double[] values) {
	    	values = (double[])values.clone();
	    	Arrays.sort(values);
	    	double[] qtls = new double[n+1];
	    	for ( int i=0; i<=n; ++i ) {
	    		qtls[i] = values[((values.length-1)*i)/n];
	    	}
	    	return qtls;
	    }

		public String getTestName() {
			return testName;
		}

		public String getBladeId() {
			return bladeId;
		}

		public String getEventType() {
			return eventType;
		}

		public String getField() {
			return field;
		}
		
		//number of values 
		public int getStolSize(){
			return this.stolDate.size();
		}

		public StatOnLongs getStolDate() {
			return stolDate;
		}

		public void setStolDate(StatOnLongs stolDate) {
			this.stolDate = stolDate;
		}

		public StatOnLongs getStol() {
			return stol;
		}

		public void setStol(StatOnLongs stol) {
			this.stol = stol;
		}
		
		public String getBladeIdAlias() {
			return bladeIdAlias;
		}
		
		public int getDatasetType() {
			return datasetType;
		}
		
		public List<FieldFilterAndOp> getFfo() {
			return ffo;
		}
		
		public String getLogicalOp() {
			return logicalOp;
		}

		
		////////////////////////////////////
	    // PRIVATE //
	    ////////////////////////////////////
		
	    /**
	     * Convert long list to double array
	     * 
	     * @param longList
	     * @return double array
	     */
	    private double[] toDoubleArray(List<Long> longList){
	    	int size = longList.size();
	    	double[] longArray = new double[size];
	    	Iterator<Long> iterator = longList.iterator();
	    	int count = 0;
	    	while (iterator.hasNext()) {
	    		double currentValue = (double) iterator.next();
				longArray[count] = currentValue;
				count++;
			}
	    	return longArray;
	    }
	    
	    /**
	     * Split.
	     * 
	     * @param values the values
	     * @param timeWindow the time window
	     * @param step the step
	     * 
	     * @return the long[] 
	     */
	    private Long[] split(double[] values, int timeWindow, int step){
	        long prev=0;
	        int k=0;
	        int i = 0;
	        SortedSet<Long> res = new TreeSet<Long>();
	        
	        while(i<values.length)
	        {
	            while( (i<values.length) && ((((Number)values[i]).longValue())<(k*step)+timeWindow) )
	            {
	                prev = i;
	                i++;
	            }
	            res.add(prev);
	            k++;
	            prev = i;
	            i++;
	        }
	        return res.toArray(new Long[0]);
	    }

		public void settimeIntervalStart(int timeIntervalStart)
		{
			this.timeIntervalStart = timeIntervalStart;
		}

		public int gettimeIntervalStart()
		{
			return timeIntervalStart;
		}

		public void settimeIntervalEnd(int timeIntervalEnd)
		{
			this.timeIntervalEnd = timeIntervalEnd;
		}

		public int gettimeIntervalEnd()
		{
			return timeIntervalEnd;
		}

	    ////////////////////////////////////
	    // interface StatisticDataControl //
	    ////////////////////////////////////
		
}
