/*
* CLIF is a Load Injection Framework
* Copyright (C) 2004 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF $Name: not supported by cvs2svn $
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.deploy;


import org.objectweb.fractal.api.Component;
import org.ow2.clif.server.api.ClifServerControl;


/**
 * Asynchronously and remotely instantiates a Blade component in a CLIF server.
 * @author Bruno Dillenseger
 */
public class BladeRemove extends Thread
{
	protected Component clifApp;
	protected ClifServerControl server;


	/**
	 * Removes all blades of a given Clif server, and from the given clif application composite.
	 * @param clifApp the Clif application distributed component the blades must be removed from
	 * @param server the controller of the CLIF server where the blades are deployed
	 */
	public BladeRemove(Component clifApp, ClifServerControl server)
	{
		super("Blades removal on " + server);
		this.clifApp = clifApp;
		this.server = server;
	}


	/**
	 * Actually proceeds with the blades removal.
	 */
	public void run()
	{
		try
		{
			server.removeAllBlades(clifApp);
		}
		catch (Exception ex)
		{
			System.out.println("Ignoring blade removal on unreachable CLIF server: " + ex);
		}
	}
}
