/*
* CLIF is a Load Injection Framework
* Copyright (C) 2004 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF $Name: not supported by cvs2svn $
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.probe.disk;

import org.objectweb.lewys.probe.Probe;
import org.objectweb.lewys.common.exceptions.NoResourceToProbeException;
import org.ow2.clif.probe.util.AbstractDumbInsert;
import org.ow2.clif.storage.api.ProbeEvent;
import org.ow2.clif.supervisor.api.ClifException;

/**
 * Field  0 -- # of reads issued
 *     This is the total number of reads issued to this partition or disk.
 * Field  1 -- # of sectors read
 *     This is the total number of sectors requested to be read from this
 *     partition  or disk.
 * Field  2 -- # of writes issued
 *     This is the total number of writes issued to this partition or disk.
 * Field  3 -- # of sectors written
 *     This is the total number of sectors requested to be written to
 *     this partition  or disk.
 * Fields 4 to 6 give io information in milliseconds
 * 
 * insert for Disk probe
 * @author Jean-Francois AHIBA
 */
public class Insert extends AbstractDumbInsert 
{

	private Probe probe;
	private int[] resourceIds;
    int nbresources = 0;
    private int NBRESOURCE = 7;
	private long[] previousValues = null;
	public Insert()
		throws ClifException
	{       
            try
            {   
                if (System.getProperty("os.name").equalsIgnoreCase("linux"))
                {   
                    if (System.getProperty("os.version").startsWith("2.4")) 
                    {
                        probe = new org.objectweb.lewys.probe.linux.DiskProbe ();
                    }
                    else if (System.getProperty("os.version").startsWith("2.6")) 
                    {
                        probe = new org.objectweb.lewys.probe.linux.Linux2_6DiskProbe();
                    }
                    else 
                        throw new ClifException("No Disk probe available for " + 
                                System.getProperty("os.name") + System.getProperty("os.version"));
                     
                    
                }
                else if (System.getProperty("os.name").startsWith("Windows"))
                {
                    probe = new org.objectweb.lewys.probe.windows.DiskProbe();
                }
                else if (System.getProperty("os.name").equals("Mac OS X"))
                {
                    probe = new org.objectweb.lewys.probe.macosx.DiskProbe();
                }
                else
                {
                    throw new ClifException("No disk probe available for " + System.getProperty("os.name"));
                }
            }
            catch (NoResourceToProbeException ex)
            {
                throw new ClifException("Can't set disk probe", ex);
            }
            
		eventStorageStatesMap.put("store-lifecycle-events", storeLifeCycleEvents);
		eventStorageStatesMap.put("store-alarm-events", storeAlarmEvents);
		eventStorageStatesMap.put("store-" + DiskEvent.EVENT_TYPE_LABEL + "-events", storeProbeEvents);
	}
	protected void configProbewithArray() throws ClifException{ 
        // The configuration is done thanks to the Map arg_probe_config in the super class.
           
        String [] resourceNames = probe.getResourceNames();
        resourceIds = new int[NBRESOURCE];
        // the ArrayList specifies the inner parameters of the probe
        if (!arg_probe_config.isEmpty()) {
        	// parameter 'diskname' appears in position zero.
            String diskname = (String)arg_probe_config.get(0);
            // locate the disk index in LeWYS resourceNames Array in order to get ressource ids.
            int disk_index = 0;
            while (disk_index < resourceNames.length && !(resourceNames[disk_index].endsWith(diskname+" reads issued")))
                {	
            		disk_index++;
                }
            if (disk_index < resourceNames.length) {
	            
            		resourceIds[0] = 0 + disk_index;
	                resourceIds[1] = 2 + disk_index;
	                resourceIds[2] = 4 + disk_index;
	                resourceIds[3] = 6 + disk_index;
	                resourceIds[4] = 9 + disk_index;
	                resourceIds[5] = 3 + disk_index;
	                resourceIds[6] = 7 + disk_index;
                
            return;
            }
        }
        // If this section is reached then Probe arguments are not correct. 
    	
    	int totalNbResource = 0;
    	if (System.getProperty("os.name").equalsIgnoreCase("linux")) {
        	totalNbResource = 11;
    	} else if (System.getProperty("os.name").startsWith("Windows")) {
    		totalNbResource = 14;
    	} else if (System.getProperty("os.name").equals("Mac OS X")) {
    		totalNbResource = 13;
        }
        
    	String exceptionMessage = "";
    	exceptionMessage+="\n\n. Bad probe parameters: A correct diskname must be provided in section BladeArgument [pollingPeriod pollingTime diskname]!!\n.\n";
    	exceptionMessage+=". Disk names for this host are:\n";
    	if (totalNbResource != 0)
        	for( int j = 0 ; j < resourceNames.length; j+= totalNbResource) {
        		String name = resourceNames[j].substring(0,resourceNames[j].length()-"reads issued".length());
        		exceptionMessage+=". "+name+"\n";
        	}
    	exceptionMessage+=	".\n. HINT: If a DiskName is too long, provide only the last discriminating characters.\n"+
			    			". For instance:\n\n" +
			    			".            \"lun0/partition6\"\n" +
			    			".               in place of \n" +
			    			". \"scsi/host0/bus0/target0/lun0/partition6\"\n"+
			    			".\n. ATTENTION: The provided DiskName must not comport character \";\" .\n";
    	throw new ClifException (exceptionMessage);
    	
    }
	public ProbeEvent doProbe()
	{
		DiskEvent event = null;
		try
		{	
			long[] values = probe.getValue(resourceIds);
			if (previousValues == null)
			{
				previousValues = values;
			} else {
					long [] result = new long [values.length];
					result [0] = values[0]-previousValues[0]; 
					result [1] = values[1]-previousValues[1];
					result [2] = values[2]-previousValues[2];
					result [3] = values[3]-previousValues[3];
					result [4] = values[4]-previousValues[4];
					result [5] = values[5]-previousValues[5];
					result [6] = values[6]-previousValues[6];
					event =  new DiskEvent(System.currentTimeMillis(), result);
					previousValues = values;
			}

		} catch (Exception ex)
		{
			ex.printStackTrace(System.err);
		}
		return event;
	}
}
