/*
* CLIF is a Load Injection Framework
* Copyright (C) 2004 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF $Name: not supported by cvs2svn $
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.probe.memory;

import org.objectweb.lewys.common.exceptions.NoResourceToProbeException;
import org.objectweb.lewys.probe.Probe;
import org.ow2.clif.probe.memory.linux.GenericProbe;
import org.ow2.clif.probe.util.AbstractDumbInsert;
import org.ow2.clif.storage.api.ProbeEvent;
import org.ow2.clif.supervisor.api.ClifException;


/**
 * Insert for memory probe
 * @author Bruno Dillenseger
 * @author Emmanuel Varoquaux
 * @author Francois AHIBA
 */
public class Insert extends AbstractDumbInsert
{
	static private final String[] RESOURCES = new String[] {
		"MemTotal",
		"MemFree",
		"SwapTotal",
		"SwapFree",
		"Cached",
		"Buffers"
	};

	private Probe probe = null;
	private GenericProbe gProbe = null;
	private int[] resourceIds;


	public Insert()
		throws ClifException
	{
		try
		{
			if (System.getProperty("os.name").equalsIgnoreCase("linux"))
			{	
				if (System.getProperty("os.version").startsWith("2.4"))
                {
					probe = new org.objectweb.lewys.probe.linux.MemoryProbe();
                    resourceIds = new int[] { 0, 2, 6, 8, 5, 4 };
                }
				else if (System.getProperty("os.version").startsWith("2.6"))
                {
/*					probe = new org.objectweb.lewys.probe.linux.Linux2_6MemoryProbe();
                    resourceIds = new int[] { 0, 1, 11, 12, 3, 2 };
*/					gProbe = new GenericProbe(RESOURCES);
					}
				else 
					throw new ClifException("No memory probe available for " + 
							System.getProperty("os.name") + System.getProperty("os.version"));
				
				
			}
			else if (System.getProperty("os.name").startsWith("Windows"))
			{
				probe = new org.objectweb.lewys.probe.windows.MemoryProbe();
                resourceIds = new int[] { 0, 2, 6, 8, 5, 4 };
			}
			else if (System.getProperty("os.name").equals("Mac OS X"))
			{
				probe = new org.objectweb.lewys.probe.macosx.MemoryProbe();
				resourceIds = new int[] { 0, 2, 6, 8, 5, 4 };
			}
			else
			{
				throw new ClifException("No memory probe available for " + System.getProperty("os.name"));
			}
		}
		catch (Exception ex)
		{
			throw new ClifException("Can't set memory probe", ex);
		}		
		eventStorageStatesMap.put("store-lifecycle-events", storeLifeCycleEvents);
		eventStorageStatesMap.put("store-alarm-events", storeAlarmEvents);
		eventStorageStatesMap.put("store-" + MemoryEvent.EVENT_TYPE_LABEL + "-events", storeProbeEvents);	
	}


	public ProbeEvent doProbe()
	{
		MemoryEvent event = null;
		try
		{
			long [] values;
			if (probe == null)
			{
				values = gProbe.getData();
			}
			else
			{
				values = probe.getValue(resourceIds);
			}
			long[] result = new long[6];
			if (System.getProperty("os.name").equalsIgnoreCase("linux") && System.getProperty("os.version").startsWith("2.4")
					|| System.getProperty("os.name").startsWith("Windows") || System.getProperty("os.name").equals("Mac OS X"))
			{
				// data is issued in bytes: must be converted to Kbytes
				for (int j = 0; j <  values.length; j++)
				{
					values[j] /= 1024;
				}
			}
			long free = values[1] + values[4] + values[5];
			result[0] = (100 * (values[0] - free)) / values[0] ;
			result[1] = (values[0] - free) / 1000;
			result[2] = values[4] / 1000;
			result[3] = values[5] / 1000;
			result[4] = (100 * (values[2] - values[3])) / values[2] ;
			result[5] = (values[2] - values[3]) / 1000;
			event = new MemoryEvent( System.currentTimeMillis(),result);
		}
		catch (Exception ex)
		{
			ex.printStackTrace(System.err);
		}
		return event;
	}
}
