/*
* CLIF is a Load Injection Framework
* Copyright (C) 2004, 2009 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF $Name: not supported by cvs2svn $
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.probe.network;

import org.objectweb.lewys.common.exceptions.NoResourceToProbeException;
import org.objectweb.lewys.common.exceptions.NoSuchResourceException;
import org.objectweb.lewys.probe.Probe;
import org.ow2.clif.probe.util.AbstractDumbInsert;
import org.ow2.clif.storage.api.ProbeEvent;
import org.ow2.clif.supervisor.api.ClifException;


/**
 * insert component for Network probe
 * @author Francois AHIBA
 * @author Bruno Dillenseger
 */
public class Insert extends AbstractDumbInsert
{
	private Probe probe;
	private int[] resourceIds;
	private int NBRESOURCE = 4;
	private long[] previousValues = null;

	public Insert()
		throws ClifException
	{
		try
		{
			if (System.getProperty("os.name").equalsIgnoreCase("linux"))
			{    
				probe = new org.objectweb.lewys.probe.linux.NetworkProbe();
			}
			else if (System.getProperty("os.name").startsWith("Windows"))
			{
				probe = new org.objectweb.lewys.probe.windows.NetworkProbe();
			}
			else if (System.getProperty("os.name").equals("Mac OS X"))
			{
				probe = new org.objectweb.lewys.probe.macosx.NetworkProbe();
			}
			else
			{
				throw new ClifException("No Network probe available for " + System.getProperty("os.name"));
			}
		}
		catch (NoResourceToProbeException ex)
		{
			throw new ClifException("Can't set Network probe", ex);
		}
		eventStorageStatesMap.put("store-lifecycle-events", storeLifeCycleEvents);
		eventStorageStatesMap.put("store-alarm-events", storeAlarmEvents);
		eventStorageStatesMap.put("store-" + NetworkEvent.EVENT_TYPE_LABEL + "-events", storeProbeEvents);
	}


	protected void configProbewithArray() throws ClifException
	{
        resourceIds = new int[NBRESOURCE];
        if (!arg_probe_config.isEmpty())
        {
        	// network adapter name parameter is the first argument
        	String networkcardName = (String)arg_probe_config.get(0);
        	try
        	{
        		// locate the network adapter index in LeWYS resourceNames array in order to get resource ids
        		int networkcard_index = probe.getResourceId(networkcardName + " bytes received");
            	resourceIds[0]  =  0 + networkcard_index;
            	resourceIds[1]  =  1 + networkcard_index;
            	resourceIds[2]  =  8 + networkcard_index;
            	resourceIds[3]  =  9 + networkcard_index;
            }
        	catch (NoSuchResourceException ex)
        	{
        		throw new ClifException("Network probe can't find network adapter " + networkcardName + " among these:\n" + adapterList());
            }
        }
        else
        {
        	throw new ClifException("Network probe needs a network adapter name to look for, as an argument.\nPlease choose one of these:" + adapterList());
        }
    }


    /**
     * Gives the list of available network adapters' names
     * @return a string containing all network adapters names separated with each other by a newline character
     */
	private String adapterList()
	{
        String [] resourceNames = probe.getResourceNames();
        int resourcesPerAdapter = System.getProperty("os.name").equals("Mac OS X") ? 17 : 16;
        StringBuilder result = new StringBuilder();
    	for (int j = 0 ; j < resourceNames.length; j+= resourcesPerAdapter)
    	{
    		result.append(resourceNames[j].substring(0, resourceNames[j].length() - " bytes received".length()) + "\n");
    	}
    	return result.toString();
	}


	public ProbeEvent doProbe()
	{	
		NetworkEvent event = null;
		try
		{	
			long[] values = probe.getValue(resourceIds);
			if (previousValues == null)
			{
				previousValues = values;
			}
			else
			{
				long [] result = new long [values.length];
				result [0] = (values[0]-previousValues[0]) >> 10; // Convert Bytes to KBytes
				result [1] = values[1]-previousValues[1];
				result [2] = (values[2]-previousValues[2]) >> 10; // Convert Bytes to KBytes
				result [3] = values[3]-previousValues[3];
				event =  new NetworkEvent(System.currentTimeMillis(),result);
				previousValues = values;
			}
		}
		catch (Exception ex)
		{	
			ex.printStackTrace(System.err);
		}
		return event;
	}
}
