/*
 * CLIF is a Load Injection Framework Copyright (C) 2005 France Telecom R&D
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * CLIF
 * 
 * Contact: clif@ow2.org
 */
package org.ow2.clif.scenario.isac.engine.nodes;

import java.util.Map;
import java.util.ArrayList;
import java.util.Stack;

import org.jdom.Element;
import org.ow2.clif.scenario.isac.engine.PlugIn;
import org.ow2.clif.scenario.isac.engine.instructions.Instruction;
import org.ow2.clif.scenario.isac.engine.instructions.SampleInstruction;
import org.ow2.clif.scenario.isac.engine.instructions.TestInstruction;

/**
 * @author Emmanuel Varoquaux
 */
public class SampleNode extends UsePlugInNode implements InstructionNode {
	
	public SampleNode(Element element) throws NodeException {
		super(element);
	}
	
	protected int getMethodNumber(Map plugIns) throws Exception {
		if (!plugIns.containsKey(use))
			throw new Exception("Plug-in not found: " + use);
		PlugIn plugIn = (PlugIn)plugIns.get(use);
		if (!plugIn.sampleConversionMap.containsKey(name))
			throw new Exception("sample not found in plug-in " + use + ": " + name);
		return ((Integer)plugIn.sampleConversionMap.get(name)).intValue();
	}

	public StringBuilder indentedToString(int indent, StringBuilder str)
	{
		Util.stringIndent(indent, str);
		str.append("sample: " + super.toString() + "\n");
		return str;
	}

	public void compile(Map<String,PlugIn> plugIns, ArrayList<Instruction> code, Stack<TestInstruction> conditions) throws Exception {
		checkConditions(code, conditions);
		code.add(new SampleInstruction(use, getMethodNumber(plugIns), split(params)));
	}
}
