/*
* CLIF is a Load Injection Framework
* Copyright (C) 2006, 2007, 2008 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF $Name$
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.util;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * Miscellaneous String utilities
 * @author Fabrice Rivart
 * @author Bruno Dillenseger
 */

public abstract class StringHelper
{
    /**
	 * Returns a valid Java identifier by removing non valid character.
	 * @param string string to convert into a valid Java identifier
	 * @return a valid Java identifier
	 */
	static public String convertJavaIdentifier(String string)
	{	
		StringBuilder sb = new StringBuilder();
		boolean first = true;
		char [] tabs = string.toCharArray();

		for (int i = 0; i < tabs.length; i++)
		{	
			if (first && Character.isJavaIdentifierStart(tabs[i]))
			{
				sb.append(tabs[i]);
				first = false;
			}
			else if (!first)
			{
				if (Character.isJavaIdentifierPart(tabs[i]))
				{
					sb.append(tabs[i]);
				}
			}
		}	
		return sb.toString();
	}


	/**
	 * @return a string representing current date in a format that makes
	 * chronological order and alphabetic order be the same
	 */
	static public String getCurrentDate()
	{
		GregorianCalendar calendar = new GregorianCalendar();
		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd_HH'h'mm'm'ss");
		format.setCalendar(calendar);
		return format.format(new Date(System.currentTimeMillis()));
	}
}
