/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2008 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Contact: clif@ow2.org
 */

package org.ow2.clif.analyze.lib.graph.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionAdapter;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.Vector;
import java.util.Map.Entry;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultCellEditor;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.jdom.Attribute;
import org.jdom.Element;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.ow2.clif.analyze.api.graph.DataSupplier;
import org.ow2.clif.analyze.lib.graph.PageOptions;
import org.ow2.clif.analyze.lib.graph.StatisticDataImpl;
import org.ow2.clif.analyze.lib.graph.ChartData;
import org.ow2.clif.analyze.lib.graph.DatasetOptions;
import org.ow2.clif.analyze.statistics.util.data.StatOnLongs;
import org.ow2.clif.storage.api.BladeEvent;
import org.ow2.clif.supervisor.api.ClifException;
import org.ow2.clif.analyze.lib.graph.FieldFilterAndOp;

/**
 * DataQuickViewContainer provides the main body for CLIF SWING application
 * @author Jordan BRUNIER
 * @author Gregory CALONNIER
 * @author Olivier LIU
 * @author Tomas Perez Segovia
 */
public class DataQuickViewContainer extends JPanel {
	private static final long serialVersionUID = -5125484790184687017L;

	// DEBUG mode
	private boolean DEBUG = true;
	
	// Constants
	private static final Color MODIFIED_AREA_COLOR = new Color(255, 204, 0);		// YELLOW
	private static final Color ERROR_AREA_COLOR = new Color(255, 0, 0);				// RED
	private static final Color DEFAULT_TEXTFIELD_COLOR = new Color(255, 255, 255);	// WHITE
	private static final Color FORE_AREA_COLOR = new Color(0, 0, 0);				// BLACK
	private static final Color BACK_AREA_COLOR = new Color(225, 238, 211);			// CREAMY

	// not used
	private static final int DEFAULT_CHART_TYPE = 1; // Index in "chartsTypes" property
	
	// Variables for the dataset table
	private final int CHART_ID  = 0;
//	private final int TEST_NAME = 1; 
//	private final int BLADE_ID = 2; 
//	private final int EVENT_TYPE = 3;
	private final int FIELD = 4;
	private final int PAGE = 5;
	private final int TO_DRAW = 6;
	private final int DELETE = 7;

	// Icons
//	private final ImageIcon VALIDATE_BUTTON_ICON = new ImageIcon(getClass().getClassLoader().getResource("icons/validate.gif"));
	private final ImageIcon REFRESH_ICON = new ImageIcon(getClass().getClassLoader().getResource("icons/refresh.gif"));
	private final ImageIcon DELETE_ICON = new ImageIcon(getClass().getClassLoader().getResource("icons/delete.gif"));
	private final ImageIcon ADD_Y_AXIS_ICON = new ImageIcon(getClass().getClassLoader().getResource("icons/addYAxis.png"));

	// Properties concerning the chart (chart engine)
//	private boolean isFirstStart = true;
//	private long timeInterval = 0;
//	private int minTime = 0;
	private long maxTime = 0;
	private boolean isFirstYAxis = true;
	
	// yAxisProperties = [ key=FieldTreePath, value=[YLabel, LineColor, ChartType, DataType] ]
	private Map<String, ArrayList<String>> yAxisProperties = new HashMap<String, ArrayList<String>>();
	// yAxisAndNode = [ key=FieldTreePath, value=[NodeTreePath] ]
	private Map<String, TreePath> yAxisAndNode = new HashMap<String, TreePath>();
	// chartsTypes = [ key=chartTypeLabel, value=[chartTypeID] ]
	private Map<String, String> chartsTypes = new HashMap<String, String>();
	// chartsTypes = [ key=FieldTreePath, value=[MapOfStatistics] ]
	private Map<String, Map<String, Double>> statistics = new HashMap<String, Map<String, Double>>();
	private DefaultTreeModel executionsTreeModel = null;
	private DefaultTableModel yAxisTableModel = null;
	private DefaultTableModel chartTableModel = null;
	
	//generatedId to link values and chartTableModel
	private int generatedId = 0;
	
	//simpleDatasetList = [ key=datasetId, value=simpleDataset]
	private Map<Integer, SimpleDataset> simpleDatasetMap = new HashMap<Integer, SimpleDataset>();
	//datasetIdAndPage = [ key=datasetId, value=pageId]
	private Map<Integer, Integer> datasetIdAndPage = new HashMap<Integer, Integer>();
	// if true page exists, so tab exists, pageExistence = [ key=pageId, boolean]
	private Map<Integer, Boolean> pageExistence = new HashMap<Integer, Boolean>();
	// statisticMap = [ key=datasetId, value=statistics]
	private Map<Integer, List<StatisticDataImpl>> statisticMap = new HashMap<Integer, List<StatisticDataImpl>>();
	// associate a datasetId and his graph options, datasetIdAndGraphOptions = [ key=datasetId, value=graphOptions]
	private Map<Integer, DatasetOptions> datasetIdAndDatasetOptions = new HashMap<Integer, DatasetOptions>();
	// associate pageId and the JFreeChart, pageIdAndJFreeChart = [ key=datasetId, value=graphOptions]
	private Map<Integer, JFreeChart> pageIdAndJFreeChart = new HashMap<Integer, JFreeChart>();
	//	pageIdAndDatasetIds = [ key=pageId, value=List of dataset Ids]
	private Map<Integer, List<Integer>> pageIdAndDatasetIds = new HashMap<Integer, List<Integer>>();
	// chartDataList = [ key=pageId, value=chartData associated]
	private Map<Integer, ChartData> chartDataList = new HashMap<Integer, ChartData>();
	// associate treepath[](report, page, datasetId) converted in String with a comment
	// mapTreeComment = [ key=path in treeComment, value=comment associated]
	Comparator<String> REVERSE_ORDER = new CommentComp();
	private Map<String, String> mapTreeComment = new TreeMap<String, String>(REVERSE_ORDER);
	//  pageOptionsMap = [ key=pageId, value=PageOptions]
	private Map<Integer, PageOptions> pageOptionsMap = new HashMap<Integer, PageOptions>();
		
	
	// Parent
	private SwingGUI globalGUI;

	private SimpleDataset simpleDataset; 
	
	//current dataset
	private int currentIdDataset;
	
	private int maxPointsToDisplay = -1;
	//initial values for moving statistics
	private int timeWindow = 100000;
	private int step = 50000;
	
	/** Creates new form DataQuickViewContainer */
	public DataQuickViewContainer() {
		initComponents();
	}

	/**
	 * Instantiates a new data quick view container.
	 * @param _globalGUI the _global GUI
	 */
	public DataQuickViewContainer(SwingGUI _globalGUI) {
		this();
		this.globalGUI = _globalGUI;
	}

	/** End Constructors */
	
	// ------------------------ Private Methods ----------------------//
	/**
	 * Create the new table model 
	 */
	private void initChartTableModel() {

		chartTableModel = new DefaultTableModel(
				new Object[][] {},
				new String[] { 
						"Chart ID",
						"Test Name", 
						"Blade Id", 
						"Event Type", 
						"Field", 
						"Page",
						"Draw",
						"Delete" 
				}) {

			private static final long serialVersionUID = -5059689400363849142L;

			Class<?>[] types = new Class[] { 
					java.lang.String.class,
					java.lang.String.class,
					java.lang.String.class,
					java.lang.String.class, 
					java.lang.String.class,
					java.lang.String.class, 
					Boolean.class, 
					java.lang.String.class
			};
			boolean[] canEdit = new boolean[] { 
					false,
					false,
					false, 
					false, 
					false, 
					true,
					true,
					false
			};

			public Class<?> getColumnClass(int columnIndex) {
				return types[columnIndex];
			}

			public boolean isCellEditable(int rowIndex, int columnIndex) {
				return canEdit[columnIndex];
			}
		};
	}
	
	
	/**
	 * Inits the table renderer and editor
	 */
	private void initChartTableRendererAndEditor() {
		TableColumnModel myColumnModel = jtYAxisValues.getColumnModel();
		TableColumn myTableColumn;		
		myTableColumn = myColumnModel.getColumn(DELETE);
		myTableColumn.setCellRenderer(new TableCellRenderer() {
			public Component getTableCellRendererComponent(JTable table,
					Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				JLabel jlDelete = new JLabel();
				jlDelete.setIcon(DELETE_ICON); 	
				jlDelete.setOpaque(true);
				jlDelete.setHorizontalAlignment(JLabel.CENTER);
				if (isSelected){
					Color bgrd = table.getSelectionBackground(); 
					jlDelete.setBackground(bgrd);
				}
				else{
					jlDelete.setBackground(Color.WHITE);
				}
				jlDelete.setBorder(null);
				return jlDelete;
			}
		});
		myTableColumn.setPreferredWidth(DELETE);
		jtYAxisValues.getColumnModel().getColumn(DELETE).setMaxWidth(10);
		jtYAxisValues.getColumnModel().getColumn(DELETE).setPreferredWidth(0);
		

		myTableColumn =  myColumnModel.getColumn(TO_DRAW);
		myTableColumn.setPreferredWidth(25);

		CheckBoxCellEditor editorDraw = new CheckBoxCellEditor();
		myTableColumn.setCellEditor(editorDraw);

		CheckBoxCellRenderer rendererDraw = new CheckBoxCellRenderer();
		myTableColumn.setCellRenderer(rendererDraw);
		
		
		//tip to hide the first column(chart id) : order is important
		jtYAxisValues.getColumnModel().getColumn(TO_DRAW).setPreferredWidth(40);
		jtYAxisValues.getColumnModel().getColumn(TO_DRAW).setMinWidth(40);
		jtYAxisValues.getColumnModel().getColumn(TO_DRAW).setMaxWidth(40);
		
		
		//tip to hide the first column(chart id) : order is important
		jtYAxisValues.getColumnModel().getColumn(CHART_ID).setPreferredWidth(0);
		jtYAxisValues.getColumnModel().getColumn(CHART_ID).setMinWidth(0);
		jtYAxisValues.getColumnModel().getColumn(CHART_ID).setMaxWidth(0);
		
	}


	/**
	 * Update statistics view.
	 */
	private void updateStatisticsView() {
		jtaStat.setText("Update Statistics View default text");
		String[] allYAxis = (String[]) statistics.keySet().toArray(
				new String[0]);
		String[] allStat;

		for (int i = 0; i < allYAxis.length; i++) {
			jtaStat.setText(jtaStat.getText()
					+ "---------------------------------------------\n");
			jtaStat.setText(jtaStat.getText() + allYAxis[i] + "\n");
			jtaStat.setText(jtaStat.getText()
					+ "---------------------------------------------\n");

			allStat = (String[]) statistics.get(allYAxis[i]).keySet().toArray(
					new String[0]);

			for (int j = 0; j < allStat.length; j++) {
				jtaStat.setText(jtaStat.getText() + allStat[j] + ": ");
				jtaStat.setText(jtaStat.getText()
						+ statistics.get(allYAxis[i]).get(allStat[j])
								.toString() + "\n");
			}
			jtaStat.setText(jtaStat.getText()
					+ "=======================================\n\n");
		}
	}

	
	/**
	 * RadioButtonRenderer class for the case to draw in jtable
	 */
	class RadioButtonRenderer implements TableCellRenderer {
		  public Component getTableCellRendererComponent(JTable table, Object value,
		                   boolean isSelected, boolean hasFocus, int row, int column) {
		    if (value==null) return null;
		    return (Component)value;
		  }
		}

		class RadioButtonEditor extends    DefaultCellEditor
		                        implements ItemListener {
		  /**
			 * 
			 */
			private static final long serialVersionUID = 6545107310799957554L;
		private JRadioButton button;

		  public RadioButtonEditor(JCheckBox checkBox) {
		    super(checkBox);
		  }

		  public Component getTableCellEditorComponent(JTable table, Object value,
		                   boolean isSelected, int row, int column) {
		    if (value==null) return null;
		    button = (JRadioButton)value;
		    button.addItemListener(this);
		    return (Component)value;
		  }

		  public Object getCellEditorValue() {
		    button.removeItemListener(this);
		    return button;
		  }

		  public void itemStateChanged(ItemEvent e) {
		    super.fireEditingStopped();
		  }
		}

	// ------------------------ Package Methods ----------------------//
	/**
	 * Sets the line chart color.
	 * 
	 * @param color
	 *            the new line chart color
	 */
	void setLineChartColor(Color color) {
		if (jtYAxisValues.getSelectedRow() != -1) {
			// Get selected Y line
			String selPath = yAxisTableModel.getValueAt(
					jtYAxisValues.getSelectedRow(), 3).toString();
			String[] splitPath = selPath.split("/");

			// Update color
			jpDiagram.setVisible(false);
			this.globalGUI.setChartLineColor(splitPath[1], splitPath[2],
					splitPath[3], splitPath[4], color);
			jpDiagram.setVisible(true);

			// Update yAxisProperties
			yAxisProperties.get(selPath).set(
					1,
					new String(color.getRed() + ":" + color.getGreen() + ":"
							+ color.getBlue()));
		} else {
			JOptionPane.showMessageDialog(this.globalGUI.getInternalFrame(),
					"Please selected a row in Y axis table.", "Warning",
					JOptionPane.INFORMATION_MESSAGE);
		}
	}

	/**
	 * Delete selected Y axis.
	 */
	void deleteSelectedYAxis() {
		if (jtYAxisValues.getSelectedRow() != -1) {
			String selPath = yAxisTableModel.getValueAt(
					jtYAxisValues.getSelectedRow(), 3).toString();
			String[] splitPath = selPath.split("/");

			// Remove chart line
			this.globalGUI.removeYAxis(splitPath[1], splitPath[2],
					splitPath[3], splitPath[4]);

			// Remove property
			yAxisProperties.remove(selPath);

			// Remove JTable row
			yAxisTableModel.removeRow(jtYAxisValues.getSelectedRow());

			// Update statistics view
			statistics.remove(selPath);
			this.updateStatisticsView();

			// Update Slider, max point
			long max = this.globalGUI.getMaxTimeFor(splitPath[1], splitPath[2]);
			if (this.maxTime == max) {
				this.maxTime = 0;

				for (int i = 0; i < jtYAxisValues.getRowCount(); i++) {
					splitPath = yAxisTableModel.getValueAt(i, 3).toString()
							.split("/");
					max = this.globalGUI.getMaxTimeFor(splitPath[1],
							splitPath[2]);
					if (i != jtYAxisValues.getSelectedRow()
							&& this.maxTime < max) {
						this.maxTime = max;
					}
				}

			

				// Set Default Background
				jtfTimeStart.setBackground(DEFAULT_TEXTFIELD_COLOR);
				jtfTimeEnd.setBackground(DEFAULT_TEXTFIELD_COLOR);

				// Set validate button visible to false
				jbValidateTimeDisplay.setVisible(false);
			}

			// Empty Y Axis JTable
			if (jtYAxisValues.getRowCount() == 0) {
				jtfTimeStart.setEnabled(false);
				jsTime.setEnabled(false);
				jtfTimeEnd.setEnabled(false);

				jpDiagram.setVisible(false);
				jpDiagram.removeAll();
				jpDiagram.setVisible(true);

				this.isFirstYAxis = true;
			}

			jpYAxisProperties.setVisible(false);
		} else {
			JOptionPane.showMessageDialog(this.globalGUI.getInternalFrame(),
					"Please selected a row in Y axis table.", "Warning",
					JOptionPane.INFORMATION_MESSAGE);
		}
	}



	
	/**
	 * This method is called from within the constructor to initialize the form.
	 */
	private void initComponents() {
		bgChartDataType = new javax.swing.ButtonGroup();
		bgMovingStatType = new javax.swing.ButtonGroup();
		jpmRightClickTree = new javax.swing.JPopupMenu();
		jmiAddToYAxis = new javax.swing.JMenuItem();
		jmiRefresh = new javax.swing.JMenuItem();
		jsSepPopupMenu = new javax.swing.JSeparator();
		jpMain = new javax.swing.JPanel();
		jpTreeButtons = new javax.swing.JPanel();
		jspExecutionsTree = new javax.swing.JScrollPane();
		jpButtonsForTree = new javax.swing.JPanel();
		jpAddYAxis = new javax.swing.JPanel();
		jbAddToYAxis = new javax.swing.JButton();
		jpGraphManager = new javax.swing.JPanel();
		jpGraphParameters = new javax.swing.JPanel();
		jpXAxis = new javax.swing.JPanel();
		jlXLabelTitle = new javax.swing.JLabel();
		jtfXLabel = new javax.swing.JTextField();
		jbValidateXLabel = new javax.swing.JButton();
		jpYAxis = new javax.swing.JPanel();
		jspYAxisValues = new javax.swing.JScrollPane();
		jtYAxisValues = new javax.swing.JTable();
		jpYAxisProperties = new javax.swing.JPanel();
		jpYLabel = new javax.swing.JPanel();
		jlYAxisLabel = new javax.swing.JLabel();
		jtfYAxisLabel = new javax.swing.JTextField();
		jpGraphType = new javax.swing.JPanel();
		jlGraphTypeTitle = new javax.swing.JLabel();
		jcbGraphType = new javax.swing.JComboBox();
		jpOptions = new javax.swing.JPanel();
		jpDataTypeTitle = new javax.swing.JPanel();
		jlDataType = new javax.swing.JLabel();
		jpRowData = new javax.swing.JPanel();
		jrbRowData = new javax.swing.JRadioButton();
		jpMovingStat = new javax.swing.JPanel();
		jrbMovingStat = new javax.swing.JRadioButton();
		jpMovingType = new javax.swing.JPanel();
		jpTypeMean = new javax.swing.JPanel();
		jlSepEmpty = new javax.swing.JLabel();
		jrbTypeMean = new javax.swing.JRadioButton();
		jpTypeMin = new javax.swing.JPanel();
		jrbTypeMin = new javax.swing.JRadioButton();
		jpTypeStandardDeviation = new javax.swing.JPanel();
		jlSepEmpty2 = new javax.swing.JLabel();
		jrbTypeStandardDeviation = new javax.swing.JRadioButton();
		jpTypeMax = new javax.swing.JPanel();
		jrbTypeMax = new javax.swing.JRadioButton();
		jpValidateOptions = new javax.swing.JPanel();
		jbValidateYAxisOptions = new javax.swing.JButton();
		jpTimeConf = new javax.swing.JPanel();
		jpMaxPoints = new javax.swing.JPanel();
		jlMaxPointTitle = new javax.swing.JLabel();
		jtfMaxPoint = new javax.swing.JTextField();
		jbValidateMaxPoint = new javax.swing.JButton();
		jpGrpahComment = new javax.swing.JPanel();
		jpGraph = new javax.swing.JPanel();
		jpDiagram = new javax.swing.JPanel();
		jpGraphOptions = new javax.swing.JPanel();
		jpTimeStart = new javax.swing.JPanel();
		jpITimeStartL = new javax.swing.JPanel();
		jlTimeStartTitle = new javax.swing.JLabel();
		jpTimeStartTF = new javax.swing.JPanel();
		jtfTimeStart = new javax.swing.JTextField();
		jlTimeStart_ms = new javax.swing.JLabel();
		jpScreenTime = new javax.swing.JPanel();
		jsTime = new javax.swing.JSlider();
		jpTimeEnd = new javax.swing.JPanel();
		jpTimeEndL = new javax.swing.JPanel();
		jlTimeEndTitle = new javax.swing.JLabel();
		jpTimeEndTF = new javax.swing.JPanel();
		jtfTimeEnd = new javax.swing.JTextField();
		jlTimeEnd_ms = new javax.swing.JLabel();
		jpValidatedTimeDisplay = new javax.swing.JPanel();
		jbValidateTimeDisplay = new javax.swing.JButton();
		jpComment = new javax.swing.JPanel();
		jpStat = new javax.swing.JPanel();
		jspStat = new javax.swing.JScrollPane();
		jtaStat = new javax.swing.JTextArea();
		jbRefresh = new javax.swing.JButton();
		jpChartTitle = new javax.swing.JPanel();
		jtfChartTitle = new javax.swing.JTextField();
		jpanel1 = new javax.swing.JPanel();
		jtPChart = new JTabbedPane();
		// dataset options tab
		jpDatasetOptionsTab = new javax.swing.JPanel();
		// page options tab
		jpPageOptionsTab = new javax.swing.JPanel();
		jpanelsub2 = new javax.swing.JPanel();
		jpanelsub3 = new javax.swing.JPanel();
		jtPsubChart = new JTabbedPane();
		jspGraph = new javax.swing.JSplitPane();
		jspGraphNorth = new javax.swing.JSplitPane();
		jbFilter = new javax.swing.JButton();
		jbDrawGraph = new javax.swing.JButton();
		jbRefreshGraph = new javax.swing.JButton();
		jpDraw1 = new javax.swing.JPanel();
		jcbRaw = new javax.swing.JCheckBox();
		jcbMoving = new javax.swing.JCheckBox();
		jcbHistogram = new javax.swing.JCheckBox();
		jbgDraw1 = new javax.swing.ButtonGroup();
		jpDraw2 = new javax.swing.JPanel();
		jcbMin = new javax.swing.JCheckBox();
		jcbMax = new javax.swing.JCheckBox();
		jcbMean = new javax.swing.JCheckBox();
		jcbStdDvt = new javax.swing.JCheckBox();
		jcbThroughput = new javax.swing.JCheckBox();
		jbgDraw2 = new javax.swing.ButtonGroup();
		jpDrawConfig = new javax.swing.JPanel();
		jpDrawButtons = new javax.swing.JPanel();    	
		jTreeComment = new JTree(getCommentTree());
		jScrollPaneComment = new javax.swing.JScrollPane();
		jScrollPaneComment.setViewportView(jTreeComment);
		jtaNewComment = new javax.swing.JTextArea();
		jbSaveComment = new javax.swing.JButton();
		jScrollPaneGraphOptions = new JScrollPane(jpDatasetOptionsTab);
		jsPageTime = new JSlider();
		
		// page options tab content
		jcbPageNoChart = new JCheckBox();
		jcbPageTimeChart = new JCheckBox();
		jcbPageHistogramChart = new JCheckBox();
		jcbPageQuantil = new JCheckBox();
		jbgPageChartType = new ButtonGroup();
		jlPageTimeStart = new JLabel();
		jlPageTimeEnd = new JLabel();
		jtfPageTimeStart = new JTextField();
		jtfPageTimeEnd = new JTextField();
		jpPageChartType = new JPanel();
		jpPageOptionsTime = new JPanel();
		jlPageSelected = new JLabel();
		
		jlDatasetTimeWindow = new JLabel();
		jlDatasetStep = new JLabel();
		jtfDatasetTimeWindow = new JTextField();
		jtfDatasetStep = new JTextField();
		jpDatasetWindowStep = new JPanel();
		jtfDatasetMaxPoints = new JTextField();
		jlDatasetMaxPoints = new JLabel();
		jpDatasetMaxPoints = new JPanel();
		jlDatasetNumberOfSlices = new JLabel();
		jtfDatasetNumberOfSlices = new JTextField();
		jpDatasetNumberOfSlices = new JPanel();
		
		jmiAddToYAxis.setFont(new java.awt.Font("Dialog", 0, 12));
		jmiAddToYAxis.setText("Add to Y axis");
		jmiAddToYAxis.setIcon(ADD_Y_AXIS_ICON);
		jmiAddToYAxis.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jmiAddToYAxisActionPerformed(evt);
			}
		});

		jpmRightClickTree.add(jmiAddToYAxis);
		jpmRightClickTree.add(jsSepPopupMenu);
		jmiRefresh.setFont(new java.awt.Font("Dialog", 0, 12));
		jmiRefresh.setText("Refresh");
		jmiRefresh.setIcon(REFRESH_ICON);
		jmiRefresh.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jmiRefreshActionPerformed(evt);
			}
		});

		jpmRightClickTree.add(jmiRefresh);
		this.setPreferredSize(new java.awt.Dimension(950, 691));
		this.setLayout(new java.awt.BorderLayout());
		this.setVisible(true);
		jpMain.setLayout(new java.awt.BorderLayout());
		jbRefresh.setFont(new java.awt.Font("Dialog", 0, 12));
		jbRefresh.setIcon(this.REFRESH_ICON);
		jbRefresh.setText("Refresh");
		jbRefresh.setToolTipText("Refresh tree executions.");
		jbRefresh.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jbRefreshActionPerformed(evt);
			}
		});
		jpTreeButtons.setLayout(new java.awt.BorderLayout());
		jpTreeButtons.add(jbRefresh, java.awt.BorderLayout.NORTH);

		jspExecutionsTree.setMinimumSize(new Dimension(200, 0));
		jspExecutionsTree.setPreferredSize(new Dimension(200, 400));

		jpTreeButtons.add(jspExecutionsTree, java.awt.BorderLayout.CENTER);

		jpButtonsForTree.setLayout(new javax.swing.BoxLayout(jpButtonsForTree,
				javax.swing.BoxLayout.Y_AXIS));

		jbAddToYAxis.setFont(new java.awt.Font("Dialog", 0, 12));
		jbAddToYAxis.setIcon(ADD_Y_AXIS_ICON);
		jbAddToYAxis.setText("Add to Y axis");
		jbAddToYAxis.setToolTipText("Add event field in a new Y axis.");
		jpAddYAxis.add(jbAddToYAxis, java.awt.BorderLayout.WEST);
		jpAddYAxis.add(jbFilter, java.awt.BorderLayout.EAST);
		jpButtonsForTree.add(jpAddYAxis);
		jpTreeButtons.add(jpButtonsForTree, java.awt.BorderLayout.SOUTH);
		jpGraphManager.setLayout(new java.awt.BorderLayout());
		jpGraphParameters.setLayout(new javax.swing.BoxLayout(
				jpGraphParameters, javax.swing.BoxLayout.Y_AXIS));
		jpGraphParameters.setBorder(javax.swing.BorderFactory
				.createTitledBorder(null, "Analysis Section",
						javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
						javax.swing.border.TitledBorder.DEFAULT_POSITION,
						new java.awt.Font("Dialog", 1, 11), new java.awt.Color(
								0, 102, 204)));
		jpXAxis.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		jpXAxis.setBorder(javax.swing.BorderFactory.createTitledBorder(null,
				"X Axis",
				javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
				javax.swing.border.TitledBorder.DEFAULT_POSITION,
				new java.awt.Font("Dialog", 1, 11), new java.awt.Color(51, 153,
						0)));		// GREEN
		jlXLabelTitle.setFont(new java.awt.Font("Dialog", 0, 12));
		jlXLabelTitle.setText("Label:");
		jpXAxis.add(jlXLabelTitle);

		jtfXLabel.setMinimumSize(new java.awt.Dimension(160, 19));
		jtfXLabel.setPreferredSize(new java.awt.Dimension(160, 19));
		jtfXLabel.setText("Date (ms)");
		jtfXLabel.addKeyListener(new java.awt.event.KeyAdapter() {
			public void keyPressed(java.awt.event.KeyEvent evt) {
				jtfXLabelKeyPressed(evt);
			}
		});

		jpXAxis.add(jtfXLabel);

		jbValidateXLabel.setFont(new java.awt.Font("Dialog", 0, 12));
		jbValidateXLabel.setToolTipText("Validate X axis label.");
		jbValidateXLabel.setContentAreaFilled(false);
		jbValidateXLabel.setMargin(new java.awt.Insets(0, 1, 0, 1));
		jbValidateXLabel.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jbValidateXLabelActionPerformed(evt);
			}
		});

		jpXAxis.add(jbValidateXLabel);
		jpYAxis.setLayout(new java.awt.BorderLayout());
		jspYAxisValues.setMinimumSize(new java.awt.Dimension(300, 80));
		jspYAxisValues.setPreferredSize(new java.awt.Dimension(300, 80));
		initChartTableModel();
		jtYAxisValues.setModel(chartTableModel);
		jtYAxisValues.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		initChartTableRendererAndEditor();
		jtYAxisValues.setForeground(DataQuickViewContainer.FORE_AREA_COLOR);
		jtYAxisValues
		.setSelectionForeground(DataQuickViewContainer.FORE_AREA_COLOR);
		jtYAxisValues
		.setSelectionBackground(DataQuickViewContainer.BACK_AREA_COLOR);
		jtYAxisValues.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				int row = jtYAxisValues.getSelectedRow();
				int key = Integer.parseInt(jtYAxisValues.getValueAt(row, CHART_ID).toString());
				int page = Integer.parseInt(jtYAxisValues.getValueAt(row, PAGE).toString());
				//update the statistics
				List<StatisticDataImpl> sdiList = statisticMap.get(key);
				updateStatisticsTab(sdiList);
				SimpleDataset sd = simpleDatasetMap.get(key);

				if(e.getClickCount() == 2){
					jmiSimpleDatasetActionPerformed();
				}
				//if field selected enable the panel for the graph options
				if(sd.isFieldSelected()){
					jpDraw1.setVisible(true);
					jpDraw2.setVisible(true);
					jpDrawButtons.setVisible(true);
					jpDrawConfig.setVisible(true);
				}
				else {
					jpDraw1.setVisible(false);
					jpDraw2.setVisible(false);
					jpDrawButtons.setVisible(false);
					jpDrawConfig.setVisible(false);
				}
				//go to the right page
				//set this page selected
				Component[] comList = jtPChart.getComponents();
				for (int i = 0; i < comList.length; i++) {
					if(comList[i].getName().equalsIgnoreCase(String.valueOf(page))){
						jtPChart.setSelectedComponent(comList[i]);
					}
				}
				//update graph options
				DatasetOptions go = datasetIdAndDatasetOptions.get(key);
				if(go != null){
					if(go.isDrawn() == false){
						jcbRaw.setSelected(true);
					}
					else{
						String chartType = go.getGraphType();
						if(chartType.equalsIgnoreCase("RAW")){
							jcbRaw.setSelected(true);
//							jbgDraw2.clearSelection();

							Component[] compList = jpDraw2.getComponents();
							for (int i = 0; i < compList.length; i++) {
								compList[i].setEnabled(false);
							}
						}
						else if(chartType.equalsIgnoreCase("MOVINGMIN")){
							jcbMoving.setSelected(true);
							jcbMin.setSelected(true);
							Component[] compList = jpDraw2.getComponents();
							for (int i = 0; i < compList.length; i++) {
								compList[i].setEnabled(true);
							}
						}
						else if(chartType.equalsIgnoreCase("MOVINGMAX")){
							jcbMoving.setSelected(true);
							jcbMax.setSelected(true);
							Component[] compList = jpDraw2.getComponents();
							for (int i = 0; i < compList.length; i++) {
								compList[i].setEnabled(true);
							}
						}
						else if(chartType.equalsIgnoreCase("MOVINGMEAN")){
							jcbMoving.setSelected(true);
							jcbMean.setSelected(true);
							Component[] compList = jpDraw2.getComponents();
							for (int i = 0; i < compList.length; i++) {
								compList[i].setEnabled(true);
							}
						}
						else if(chartType.equalsIgnoreCase("MOVINGSTDDVT")){
							jcbMoving.setSelected(true);
							jcbStdDvt.setSelected(true);
							Component[] compList = jpDraw2.getComponents();
							for (int i = 0; i < compList.length; i++) {
								compList[i].setEnabled(true);
							}
						}
					}
				}
				
				if(jtYAxisValues.getSelectedColumn() == TO_DRAW){
					int viewRow = jtYAxisValues.getSelectedRow();
					String value = jtYAxisValues.getValueAt(viewRow, TO_DRAW).toString();
					//get the id to delete it from the dataset list 
					int keyDataset = Integer.parseInt(jtYAxisValues.getValueAt(viewRow, CHART_ID).toString());
					int pageId = Integer.parseInt(jtYAxisValues.getValueAt(viewRow, PAGE).toString());
					if(value.equalsIgnoreCase("true")){
						jbDrawGraphPerformed();
					}
					else{
						try {
							//get current chartData and remove corresponding xyDataset from the tab
							ChartData chartDataCurrent = chartDataList.get(pageId);
							if(chartDataCurrent != null){
								JFreeChart jfc = chartDataCurrent.removeDatasetFromTab(keyDataset);
								updateChart(jfc, pageId, chartDataCurrent, keyDataset);
							}
						} catch (Exception ex1) {
						}	
					}
		
				}
				
				
				
				
				//remove a row
				if(jtYAxisValues.getSelectedColumn() == DELETE){
					if(chartTableModel.getRowCount()>=1){
						int viewRow = jtYAxisValues.getSelectedRow();
						//get the id to delete it from the dataset list 
						int keyDataset = Integer.parseInt(jtYAxisValues.getValueAt(viewRow, CHART_ID).toString());
						int pageId = Integer.parseInt(jtYAxisValues.getValueAt(viewRow, PAGE).toString());
						datasetIdAndPage.remove(keyDataset);
						//remove data 
						List<Integer> inList = pageIdAndDatasetIds.get(pageId);
						try {
							if(inList != null){
								inList.remove(keyDataset);
								pageIdAndDatasetIds.put(pageId, inList);
							}
						} catch (Exception ex) {

						}
						//compare for page existence 
						modifyPageExistence();
						//delete obsolete page
						deleteNonExistencePage();
						try {
							//get current chartData and remove corresponding xyDataset from the tab
							ChartData chartDataCurrent = chartDataList.get(pageId);
							if(chartDataCurrent != null){
								JFreeChart jfc = chartDataCurrent.removeDatasetFromTab(keyDataset);
								updateChart(jfc, pageId, chartDataCurrent, keyDataset);
							}
						} catch (Exception ex1) {
						}				

						//delete row 
						chartTableModel.removeRow(viewRow);
						// update other maps
						simpleDatasetMap.remove(keyDataset);
						statisticMap.remove(keyDataset);
						datasetIdAndDatasetOptions.remove(keyDataset);
						
						udpatepageIdAndDatasetIds();
						//update comment tree
						initCommentTree();
						cleanMapTreeComment();
//						jTreeComment.clearSelection();
						jtaNewComment.setText("new empty comment");
					}
				}
			}
		});

		jtYAxisValues.getModel().addTableModelListener(jtYAxisValues);

		jspYAxisValues.setViewportView(jtYAxisValues);

		jpYAxis.add(jspYAxisValues, java.awt.BorderLayout.CENTER);

		jpYAxisProperties.setLayout(new javax.swing.BoxLayout(
				jpYAxisProperties, javax.swing.BoxLayout.Y_AXIS));

		jpYAxisProperties.setBorder(javax.swing.BorderFactory
				.createTitledBorder(null, "Selected Y Axis Properties",
						javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
						javax.swing.border.TitledBorder.DEFAULT_POSITION,
						new java.awt.Font("Dialog", 0, 11), new java.awt.Color(
								51, 153, 0)));		// GREEN

		jpYLabel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

		jlYAxisLabel.setFont(new java.awt.Font("Dialog", 0, 12));
		jlYAxisLabel.setText("Label:");
		jpYLabel.add(jlYAxisLabel);

		jtfYAxisLabel.setMinimumSize(new java.awt.Dimension(200, 19));
		jtfYAxisLabel.setPreferredSize(new java.awt.Dimension(200, 19));
		jtfYAxisLabel.addKeyListener(new java.awt.event.KeyAdapter() {
			public void keyPressed(java.awt.event.KeyEvent evt) {
				jtfYAxisLabelKeyPressed(evt);
			}
		});
		jpYLabel.add(jtfYAxisLabel);

		jpYAxisProperties.add(jpYLabel);

		jpGraphType
		.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

		jlGraphTypeTitle.setFont(new java.awt.Font("Dialog", 0, 12));
		jlGraphTypeTitle.setText("Chart type:");
		jpGraphType.add(jlGraphTypeTitle);

		jcbGraphType.setFont(new java.awt.Font("Dialog", 0, 12));
		jcbGraphType.setMinimumSize(new java.awt.Dimension(170, 23));
		jcbGraphType.setPreferredSize(new java.awt.Dimension(170, 23));
		jcbGraphType.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(java.awt.event.MouseEvent evt) {
				jcbGraphTypeMouseClicked(evt);
			}
		});
		jpGraphType.add(jcbGraphType);

		jpYAxisProperties.add(jpGraphType);
		jpYAxisProperties.setVisible(false);

		jpOptions.setLayout(new javax.swing.BoxLayout(jpOptions,
				javax.swing.BoxLayout.Y_AXIS));

		jpDataTypeTitle.setLayout(new java.awt.FlowLayout(
				java.awt.FlowLayout.LEFT));

		jlDataType.setFont(new java.awt.Font("Dialog", 0, 12));
		jlDataType.setText("Data type:");
		jpDataTypeTitle.add(jlDataType);

		jpOptions.add(jpDataTypeTitle);

		jpRowData.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

		bgChartDataType.add(jrbRowData);
		jrbRowData.setFont(new java.awt.Font("Dialog", 0, 10));
		jrbRowData.setSelected(true);
		jrbRowData.setText("Raw data");
		jrbRowData.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0,
				0, 0));
		jrbRowData.setMargin(new java.awt.Insets(0, 0, 0, 0));
		jrbRowData.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jrbRowDataActionPerformed(evt);
			}
		});

		jpRowData.add(jrbRowData);

		jpOptions.add(jpRowData);

		jpMovingStat
		.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

		bgChartDataType.add(jrbMovingStat);
		jrbMovingStat.setFont(new java.awt.Font("Dialog", 0, 10));
		jrbMovingStat.setText("Moving statistic:");
		jrbMovingStat.setBorder(javax.swing.BorderFactory.createEmptyBorder(0,
				0, 0, 0));
		jrbMovingStat.setMargin(new java.awt.Insets(0, 0, 0, 0));
		jrbMovingStat.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jrbMovingStatActionPerformed(evt);
			}
		});

		jpMovingStat.add(jrbMovingStat);

		jpOptions.add(jpMovingStat);

		jpMovingType.setLayout(new java.awt.GridLayout(2, 2));

		jpTypeMean.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

		jlSepEmpty.setFont(new java.awt.Font("Dialog", 0, 12));
		jlSepEmpty.setText("     ");
		jlSepEmpty.setText(" (empty) ");
		jpTypeMean.add(jlSepEmpty);

		bgMovingStatType.add(jrbTypeMean);
		jrbTypeMean.setFont(new java.awt.Font("Dialog", 0, 10));
		jrbTypeMean.setSelected(true);
		jrbTypeMean.setText("mean");
		jrbTypeMean.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0,
				0, 0));
		jrbTypeMean.setEnabled(false);
		jrbTypeMean.setMargin(new java.awt.Insets(0, 0, 0, 0));
		jrbTypeMean.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jrbTypeMeanActionPerformed(evt);
			}
		});
		jpTypeMean.add(jrbTypeMean);

		jpMovingType.add(jpTypeMean);

		jpTypeMin.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

		bgMovingStatType.add(jrbTypeMin);
		jrbTypeMin.setFont(new java.awt.Font("Dialog", 0, 10));
		jrbTypeMin.setText("min");
		jrbTypeMin.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0,
				0, 0));
		jrbTypeMin.setEnabled(false);
		jrbTypeMin.setMargin(new java.awt.Insets(0, 0, 0, 0));
		jrbTypeMin.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jrbTypeMinActionPerformed(evt);
			}
		});
		jpTypeMin.add(jrbTypeMin);

		jpMovingType.add(jpTypeMin);

		jpTypeStandardDeviation.setLayout(new java.awt.FlowLayout(
				java.awt.FlowLayout.LEFT));

		jlSepEmpty2.setFont(new java.awt.Font("Dialog", 0, 12));
		jlSepEmpty2.setText("     ");
		jlSepEmpty2.setText(" (empty 2) ");
		jpTypeStandardDeviation.add(jlSepEmpty2);

		bgMovingStatType.add(jrbTypeStandardDeviation);
		jrbTypeStandardDeviation.setFont(new java.awt.Font("Dialog", 0, 10));
		jrbTypeStandardDeviation.setText("standard deviation");
		jrbTypeStandardDeviation.setBorder(javax.swing.BorderFactory
				.createEmptyBorder(0, 0, 0, 0));
		jrbTypeStandardDeviation.setEnabled(false);
		jrbTypeStandardDeviation.setMargin(new java.awt.Insets(0, 0, 0, 0));
		jrbTypeStandardDeviation
		.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jrbTypeStandardDeviationActionPerformed(evt);
			}
		});
		jpTypeStandardDeviation.add(jrbTypeStandardDeviation);

		jpMovingType.add(jpTypeStandardDeviation);

		jpTypeMax.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

		bgMovingStatType.add(jrbTypeMax);
		jrbTypeMax.setFont(new java.awt.Font("Dialog", 0, 10));
		jrbTypeMax.setText("max");
		jrbTypeMax.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0,
				0, 0));
		jrbTypeMax.setEnabled(false);
		jrbTypeMax.setMargin(new java.awt.Insets(0, 0, 0, 0));
		jrbTypeMax.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jrbTypeMaxActionPerformed(evt);
			}
		});
		jpTypeMax.add(jrbTypeMax, java.awt.BorderLayout.CENTER);

		jpMovingType.add(jpTypeMax, java.awt.BorderLayout.CENTER);

		jpOptions.add(jpMovingType, java.awt.BorderLayout.CENTER);

		jpYAxisProperties.add(jpOptions, java.awt.BorderLayout.CENTER);

		jpValidateOptions.setLayout(new java.awt.FlowLayout(
				java.awt.FlowLayout.RIGHT));

		jbValidateYAxisOptions.setFont(new java.awt.Font("Dialog", 0, 12));
		jbValidateYAxisOptions.setToolTipText("Validate Y axis options.");
		jbValidateYAxisOptions.setContentAreaFilled(false);
		jbValidateYAxisOptions.setMargin(new java.awt.Insets(0, 1, 0, 1));
		jbValidateYAxisOptions
		.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jbValidateYAxisOptionsActionPerformed(evt);
			}
		});

		jpValidateOptions.add(jbValidateYAxisOptions, java.awt.BorderLayout.CENTER);

		jpYAxisProperties.add(jpValidateOptions, java.awt.BorderLayout.CENTER);

		jpYAxis.add(jpYAxisProperties, java.awt.BorderLayout.SOUTH);

		jpGraphParameters.add(jpYAxis, java.awt.BorderLayout.CENTER);

		jpTimeConf.setLayout(new javax.swing.BoxLayout(jpTimeConf,
				javax.swing.BoxLayout.Y_AXIS));
		jpMaxPoints
		.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

		jlMaxPointTitle.setFont(new java.awt.Font("Dialog", 0, 12));
		jlMaxPointTitle.setText("Maximum number of point:");
		jpMaxPoints.add(jlMaxPointTitle, java.awt.BorderLayout.CENTER);

		jtfMaxPoint.setText("1000");
		jtfMaxPoint.setMinimumSize(new java.awt.Dimension(50, 19));
		jtfMaxPoint.setPreferredSize(new java.awt.Dimension(50, 19));
		jtfMaxPoint.addFocusListener(new java.awt.event.FocusAdapter() {
			public void focusLost(java.awt.event.FocusEvent evt) {
				jtfMaxPointFocusLost(evt);
			}
		});
		jtfMaxPoint.addKeyListener(new java.awt.event.KeyAdapter() {
			public void keyPressed(java.awt.event.KeyEvent evt) {
				jtfMaxPointKeyPressed(evt);
			}
		});

		jpMaxPoints.add(jtfMaxPoint, java.awt.BorderLayout.CENTER);

		jbValidateMaxPoint.setFont(new java.awt.Font("Dialog", 0, 12));
		jbValidateMaxPoint
		.setToolTipText("Validate maximum number of point by interval.");
		jbValidateMaxPoint.setContentAreaFilled(false);
		jbValidateMaxPoint.setMargin(new java.awt.Insets(0, 1, 0, 1));


//		jpMaxPoints.add(jbValidateMaxPoint, java.awt.BorderLayout.CENTER);

		jpTimeConf.add(jpMaxPoints, java.awt.BorderLayout.CENTER);
		jpGrpahComment.setLayout(new java.awt.BorderLayout());

		jpChartTitle.setLayout(new java.awt.BorderLayout());

		jpChartTitle.setBorder(javax.swing.BorderFactory.createTitledBorder(
				null, "Chart Title",
				javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
				javax.swing.border.TitledBorder.DEFAULT_POSITION,
				new java.awt.Font("Dialog", 1, 11), new java.awt.Color(0, 102,
						204)));	// DEEP BLUE
		jpChartTitle.add(jtfChartTitle, java.awt.BorderLayout.CENTER);
		jpGraph.setLayout(new java.awt.BorderLayout());

		jpGraph.setBorder(javax.swing.BorderFactory.createTitledBorder(null,
				"Analysis",
				javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
				javax.swing.border.TitledBorder.DEFAULT_POSITION,
				new java.awt.Font("Dialog", 1, 11), new java.awt.Color(0, 102,
						204)));		// DEEP BLUE
		jpDiagram.setLayout(new java.awt.BorderLayout());
		jpDiagram.setBackground(new java.awt.Color(255, 255, 255));		// WHITE
		jpDiagram.setBackground(new java.awt.Color(225, 238, 211));		// CREAMY
		jpGraphOptions.setLayout(new java.awt.BorderLayout());
		jpGraphOptions.setBorder(javax.swing.BorderFactory.createTitledBorder(
				null, "Analysis Parameters",
				javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
				javax.swing.border.TitledBorder.DEFAULT_POSITION,
				new java.awt.Font("Dialog", 1, 11), new java.awt.Color(51, 153,
						0))); // GREEN

		jpTimeStart.setLayout(new javax.swing.BoxLayout(jpTimeStart,
				javax.swing.BoxLayout.Y_AXIS));

		jpITimeStartL.setLayout(new java.awt.FlowLayout(
				java.awt.FlowLayout.LEFT, 5, 0));

		jlTimeStartTitle.setFont(new java.awt.Font("Dialog", 0, 12));
		jlTimeStartTitle.setText("Start:");
		jpITimeStartL.add(jlTimeStartTitle);

		jpTimeStart.add(jpITimeStartL);

		jpTimeStartTF.setLayout(new java.awt.FlowLayout(
				java.awt.FlowLayout.LEFT));

		jtfTimeStart.setText("");
		jtfTimeStart.setText(" (empty time start)");
		jtfTimeStart.setMinimumSize(new java.awt.Dimension(55, 19));
		jtfTimeStart.setPreferredSize(new java.awt.Dimension(55, 19));
		jtfTimeStart.setEnabled(true);
		jpTimeStartTF.add(jtfTimeStart);

		jlTimeStart_ms.setFont(new java.awt.Font("Dialog", 0, 12));
		jlTimeStart_ms.setText("ms");
		jpTimeStartTF.add(jlTimeStart_ms);

		jpTimeStart.add(jpTimeStartTF);
		//
		// jpGraphOptions.add(jpTimeStart, java.awt.BorderLayout.WEST);

		jpScreenTime.setLayout(new java.awt.BorderLayout());

		jsTime.setPaintLabels(true);
		jsTime.setPaintTicks(true);
		jsTime.setMaximumSize(new java.awt.Dimension(32767, 32767));
		jsTime.setPreferredSize(new java.awt.Dimension(250, 150));
		jsTime.setValueIsAdjusting(true);
		jsTime.setEnabled(true);
		jpScreenTime.add(jsTime, java.awt.BorderLayout.CENTER);
		jpTimeEnd.setLayout(new javax.swing.BoxLayout(jpTimeEnd,
				javax.swing.BoxLayout.Y_AXIS));
		jpTimeEndL.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT,
				5, 0));

		jlTimeEndTitle.setFont(new java.awt.Font("Dialog", 0, 12));
		jlTimeEndTitle.setText("End:");
		jpTimeEndL.add(jlTimeEndTitle);

		jpTimeEnd.add(jpTimeEndL);

		jpTimeEndTF
		.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

		jtfTimeEnd.setText("0");
		jtfTimeEnd.setMinimumSize(new java.awt.Dimension(55, 19));
		jtfTimeEnd.setPreferredSize(new java.awt.Dimension(55, 19));
		jtfTimeEnd.setEnabled(true);
		jpTimeEndTF.add(jtfTimeEnd);

		jlTimeEnd_ms.setFont(new java.awt.Font("Dialog", 0, 12));
		jlTimeEnd_ms.setText("ms");
		jpTimeEndTF.add(jlTimeEnd_ms);

		jpValidatedTimeDisplay.setLayout(new java.awt.FlowLayout(
				java.awt.FlowLayout.CENTER, 0, 0));
		jpValidatedTimeDisplay.setPreferredSize(new java.awt.Dimension(25, 24));

		jbValidateTimeDisplay.setFont(new java.awt.Font("Dialog", 0, 12));
		jbValidateTimeDisplay.setToolTipText("Validate time display.");
		jbValidateTimeDisplay.setContentAreaFilled(false);
		jbValidateTimeDisplay.setMargin(new java.awt.Insets(0, 1, 0, 1));

		jpValidatedTimeDisplay.add(jbValidateTimeDisplay);

		jpTimeEndTF.add(jpValidatedTimeDisplay);

		jpTimeEnd.add(jpTimeEndTF);

		// jpGraphOptions.add(jpTimeEnd, java.awt.BorderLayout.EAST);

		jpGraph.add(jpGraphOptions, java.awt.BorderLayout.SOUTH);

		// jpGrpahComment.add(jpGraph, java.awt.BorderLayout.CENTER);

		// jpComment.setLayout(new java.awt.BorderLayout());

		// jpComment.setBorder(javax.swing.BorderFactory.createTitledBorder(null,
		// "Graph Options",
		// javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
		// javax.swing.border.TitledBorder.DEFAULT_POSITION, new
		// java.awt.Font("Dialog", 1, 11), new java.awt.Color(0, 102, 204)));
		//jpDrawConfig
//		jpComment.setLayout(new GridBagLayout());
//		GridBagConstraints cComment = new GridBagConstraints();
//		//natural height, maximum width
//		cComment.fill = GridBagConstraints.VERTICAL;
//		cComment.gridx = 0;
//		cComment.gridy = 0;
//		jpComment.add(jScrollPaneComment, cComment);


//		jspComment.setViewportView(jtaComment);
//		jtaComment.setColumns(60);
//		jtaComment.setRows(10);
//		cComment.fill = GridBagConstraints.VERTICAL;
//		cComment.gridx = 1;
//		cComment.gridy = 0;
//		jpComment.add(jspComment, cComment);


		jpComment.add(jScrollPaneComment);
//		jpComment.add(jspComment, java.awt.BorderLayout.CENTER);

		// jpGrpahComment.add(jpComment, java.awt.BorderLayout.SOUTH);

		// jpGraphManager.add(jpGrpahComment, java.awt.BorderLayout.CENTER);

//		jspTreeGraph.setRightComponent(jpGraphManager);

		jpMain.add(jpGraphManager, java.awt.BorderLayout.CENTER);

//		jspTopBottom.setTopComponent(jpMain);

		jpStat.setLayout(new java.awt.BorderLayout());

		// jpStat.setBorder(javax.swing.BorderFactory.createTitledBorder(null,
		// "Graph Analyze",
		// javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
		// javax.swing.border.TitledBorder.DEFAULT_POSITION, new
		// java.awt.Font("Dialog", 1, 11), new java.awt.Color(0, 102, 204)));

		// jpStat.setPreferredSize(new java.awt.Dimension(100, 200));
		jtaStat.setColumns(20);
		jtaStat.setEditable(false);
		jtaStat.setRows(5);
		jtaStat.setText(" (jtaStat empty text) ");
		jspStat.setViewportView(jtaStat);

		jpStat.add(jspStat, java.awt.BorderLayout.CENTER);

		// jpStat.add(jpGraph);

		// jpStat.add(jpComment, java.awt.BorderLayout.CENTER);
		// add a tabbed pane : one for the chart comment and one for the chart
		// statistics

//		jtPChart.add(jpanel1, "Page1");
//		jpanel1.add(jpDiagram, java.awt.BorderLayout.CENTER);
//		jpanel1.add(jvpGraph, java.awt.BorderLayout.CENTER);
//		jvpGraph.setView(jpComment);
		//1. Create the frame.
//		javax.swing.JInternalFrame frame = new javax.swing.JInternalFrame("FrameDemo", false, false, false, false);
//		frame.setVisible(true);
//		frame.pack();
//		frame.add(jtPChart);
//		jpanel1.add(frame);
		// jspChartPanel.setViewportView(jpDiagram);
		//jpanel1.add(jspChartPanel, java.awt.BorderLayout.CENTER);
//		jspPaneContenu.setMinimumSize(new Dimension(200, 0));
//		jspPaneContenu.setPreferredSize(new Dimension(600, 200));
//		jspPaneContenu.setPreferredSize(jpanel1.getSize());
//		jspPaneContenu.setViewportView(jpDiagram);
//		jpanel1.add(jspPaneContenu, java.awt.BorderLayout.CENTER);


//		jpanel2.add(jcpChartPanel);
//		chartPanel.setLayout(new java.awt.BorderLayout());
		jpanel1.setLayout(new java.awt.BorderLayout());
//		jpanel1.add(chartPanel, java.awt.BorderLayout.CENTER);
//		jtPChart.add(jpanel2, "Page2");
		jtPChart.addMouseListener(new MouseListener() {
			int x0, y0, x1, y1, x2, y2;

			public void mousePressed(MouseEvent e) {
				x0 = e.getX();
				y0 = e.getY();
			}

			public void mouseReleased(MouseEvent e) {
				x1 = e.getX();
				y1 = e.getY();

				if (jtPChart.indexAtLocation(x0, y0) != jtPChart
						.indexAtLocation(x1, y1)) {
					// insert in order
					if (jtPChart.indexAtLocation(x0, y0) < jtPChart
							.indexAtLocation(x1, y1)) {
						if (jtPsubChart.indexAtLocation(x0, y0) >= 0
								&& jtPsubChart.indexAtLocation(x0, y0) < jtPChart
								.getTabCount()) {
							jtPChart.add(jtPChart.getComponentAt(jtPChart
									.indexAtLocation(x0, y0)), jtPChart
									.getTitleAt(jtPChart
											.indexAtLocation(x0, y0)), jtPChart
											.indexAtLocation(x1, y1) + 1);
							jtPChart.setSelectedIndex(jtPChart.indexAtLocation(
									x1, y1));
						}
					}
					// insert in reverse order
					else if (jtPChart.indexAtLocation(x0, y0) > jtPChart
							.indexAtLocation(x1, y1)) {
						// System.out.println("inverse \n");
						if (jtPsubChart.indexAtLocation(x0, y0) >= 0
								&& jtPsubChart.indexAtLocation(x0, y0) < jtPChart
								.getTabCount()) {
							jtPChart.add(jtPChart.getComponentAt(jtPChart
									.indexAtLocation(x0, y0)), jtPChart
									.getTitleAt(jtPChart
											.indexAtLocation(x0, y0)), jtPChart
											.indexAtLocation(x1, y1));
							jtPChart.setSelectedIndex(jtPChart.indexAtLocation(
									x1, y1));
						}
					}
				} else {
					// System.out.println("it's same index location
					// \n"+jtPChart.indexAtLocation(x1,y1) );
				}
			}

			public void mouseClicked(MouseEvent e) {
				// delete a page : molette click
				if (e.getButton() == 2) {
					x2 = e.getX();
					y2 = e.getY();
					if (jtPChart.getComponentAt(jtPChart
							.indexAtLocation(x2, y2)) != null) {
						// delete item in the JComboBox
						// AnalysisSection
						pageComboBox.removeItem(String.valueOf(jtPChart
								.indexAtLocation(x2, y2) + 1));
						// delete tab
						jtPChart.removeTabAt(jtPChart.indexAtLocation(x2, y2));
					}

				}
			}

			public void mouseEntered(MouseEvent e) {
			}

			public void mouseExited(MouseEvent e) {
			}

		});


		// jpComment.add(jtPChart,java.awt.BorderLayout.CENTER);
		jpGraph.add(jtPChart, java.awt.BorderLayout.CENTER);

		jScrollPaneGraphOptions.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		jScrollPaneGraphOptions.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
//		jtPsubChart.add(jScrollPaneGraphOptions, "Graph Options");
		
		// page options creation
		jpPageChartType.setLayout(new GridLayout(0,1));
		jcbPageNoChart.setFont(new java.awt.Font("Dialog", 0, 12));
		jcbPageNoChart.setText("No Chart");
		jpPageChartType.add(jcbPageNoChart);
		
		jcbPageTimeChart.setFont(new java.awt.Font("Dialog", 0, 12));
		jcbPageTimeChart.setText("Time Chart");
		jpPageChartType.add(jcbPageTimeChart);
		
		jcbPageHistogramChart.setFont(new java.awt.Font("Dialog", 0, 12));
		jcbPageHistogramChart.setText("Histogram Chart (Not Implemented Yet)");
		jpPageChartType.add(jcbPageHistogramChart);
		
		jcbPageQuantil.setFont(new java.awt.Font("Dialog", 0, 12));
		jcbPageQuantil.setText("Quantil Chart (Not Implemented Yet)");
		jpPageChartType.add(jcbPageQuantil);
		
		jbgPageChartType.add(jcbPageNoChart);
		jbgPageChartType.add(jcbPageTimeChart);
		jbgPageChartType.add(jcbPageHistogramChart);
		jbgPageChartType.add(jcbPageQuantil);
		
		jcbPageNoChart.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				int selectedPage = getSelectedPage();
				if(selectedPage != -1){
					// get matching page options
					PageOptions po = pageOptionsMap.get(selectedPage);
					if(po != null){
						po.setGraphType("NoChart");
						pageOptionsMap.put(selectedPage, po);
						disableAllDatasetOptions();
						// hide the chart associate with this page
						Component[] comList = jtPChart.getComponents();
						for (int i = 0; i < comList.length; i++) {
							if(comList[i].getName().equalsIgnoreCase(String.valueOf(selectedPage))){
								JScrollPane jp = (JScrollPane) comList[i];
								Component[] tmpList = jp.getComponents();
								for (int j = 0; j < tmpList.length; j++) {
									tmpList[i].setVisible(false);
								}
							}
						}
						jtPChart.repaint();
					}
				}
			}
		});
		jcbPageTimeChart.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				int selectedPage = getSelectedPage();
				if(selectedPage != -1){
					PageOptions po = pageOptionsMap.get(selectedPage);
					if(po != null){
						po.setGraphType("TimeChart");
						pageOptionsMap.put(selectedPage, po);
						enableOnlyTimeChart();
						// set visible the chart associate with this page
						Component[] comList = jtPChart.getComponents();
						for (int i = 0; i < comList.length; i++) {
							if(comList[i].getName().equalsIgnoreCase(String.valueOf(selectedPage))){
								JScrollPane jp = (JScrollPane) comList[i];
								Component[] tmpList = jp.getComponents();
								for (int j = 0; j < tmpList.length; j++) {
									tmpList[i].setVisible(true);
								}
							}
						}
						jtPChart.repaint();
					}
				}
			}
		});
		jcbPageHistogramChart.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				int selectedPage = getSelectedPage();
				if(selectedPage != -1){
					PageOptions po = pageOptionsMap.get(selectedPage);
					if(po != null){
						po.setGraphType("HistogramChart");
						pageOptionsMap.put(selectedPage, po);
						enableOnlyFrequencyChart();
						// set visible the chart associate with this page
						Component[] comList = jtPChart.getComponents();
						for (int i = 0; i < comList.length; i++) {
							if(comList[i].getName().equalsIgnoreCase(String.valueOf(selectedPage))){
								JScrollPane jp = (JScrollPane) comList[i];
								Component[] tmpList = jp.getComponents();
								for (int j = 0; j < tmpList.length; j++) {
									tmpList[i].setVisible(true);
								}
							}
						}
						jtPChart.repaint();
					}
				}
			}
		});
		jcbPageQuantil.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				int selectedPage = getSelectedPage();
				if(selectedPage != -1){
					PageOptions po = pageOptionsMap.get(selectedPage);
					if(po != null){
						po.setGraphType("QuantilChart");
						pageOptionsMap.put(selectedPage, po);
						enableOnlyQuantilChart();
						// set visible the chart associate with this page
						Component[] comList = jtPChart.getComponents();
						for (int i = 0; i < comList.length; i++) {
							if(comList[i].getName().equalsIgnoreCase(String.valueOf(selectedPage))){
								JScrollPane jp = (JScrollPane) comList[i];
								Component[] tmpList = jp.getComponents();
								for (int j = 0; j < tmpList.length; j++) {
									tmpList[i].setVisible(true);
								}
							}
						}
						jtPChart.repaint();
					}
				}
			}
		});
		
		
		jlPageSelected.setFont(new java.awt.Font("Dialog", 0, 12));
		jlPageSelected.setText("Page selected : " );
		
		// jpPageOptionsTime : right component of the page options tab
		jpPageOptionsTime.setLayout(new GridBagLayout());
		GridBagConstraints potConstraints = new GridBagConstraints();
		//natural height, maximum width
		potConstraints.fill = GridBagConstraints.VERTICAL;
		potConstraints.gridx = 0;
		potConstraints.gridy = 0;
		jlPageTimeStart.setFont(new java.awt.Font("Dialog", 0, 12));
		jlPageTimeStart.setText("Time Start  ");
		jpPageOptionsTime.add(jlPageTimeStart, potConstraints);

		potConstraints.fill = GridBagConstraints.VERTICAL;
		potConstraints.gridx = 1;
		potConstraints.gridy = 0;
		jtfPageTimeStart.setPreferredSize(new Dimension(140, 17));
		jpPageOptionsTime.add(jtfPageTimeStart, potConstraints);

		potConstraints.fill = GridBagConstraints.VERTICAL;
		potConstraints.gridx = 3;
		potConstraints.gridy = 0;
		jlPageTimeEnd.setFont(new java.awt.Font("Dialog", 0, 12));
		jlPageTimeEnd.setText("Time End  ");
		jpPageOptionsTime.add(jlPageTimeEnd, potConstraints);
		
		potConstraints.fill = GridBagConstraints.VERTICAL;
		potConstraints.gridx = 4;
		potConstraints.gridy = 0;
		jtfPageTimeEnd.setPreferredSize(new Dimension(140, 17));
		jpPageOptionsTime.add(jtfPageTimeEnd, potConstraints);
		
//		potConstraints.fill = GridBagConstraints.VERTICAL;
//		potConstraints.gridx = 2;
//		potConstraints.gridy = 0;
//		jpPageOptionsTime.add(jsPageTime, potConstraints);		
		
		
		
		
		jpPageOptionsTab.setLayout(new GridBagLayout());
		GridBagConstraints pageOptionsConstraints = new GridBagConstraints();
		//natural height, maximum width
		pageOptionsConstraints.fill = GridBagConstraints.HORIZONTAL;
		pageOptionsConstraints.gridx = 0;
		pageOptionsConstraints.gridy = 0;
		jpPageOptionsTab.add(jlPageSelected, pageOptionsConstraints);
		
		pageOptionsConstraints.fill = GridBagConstraints.HORIZONTAL;
		pageOptionsConstraints.gridx = 0;
		pageOptionsConstraints.gridy = 1;
		jpPageOptionsTab.add(jpPageChartType, pageOptionsConstraints);

		pageOptionsConstraints.fill = GridBagConstraints.HORIZONTAL;
		pageOptionsConstraints.gridx = 1;
		pageOptionsConstraints.gridy = 1;
		jpPageOptionsTab.add(jpPageOptionsTime, pageOptionsConstraints);
		
		jtPsubChart.add(jpPageOptionsTab, "Page Options");
		
		disableAllPageOptions();
		
		jtPsubChart.add(jpDatasetOptionsTab, "Dataset Options");

		jtPsubChart.add(jpanelsub2, "Statistic");
		jpanelsub2.setLayout(new java.awt.BorderLayout());
		jpStat.setPreferredSize(new java.awt.Dimension(1000, 150));
		jpStat.setMinimumSize(new java.awt.Dimension(200, 50));
		jpanelsub2.add(jpStat);

		jtPsubChart.add(jpanelsub3, "Comment");
		jpanelsub3.setLayout(new java.awt.BorderLayout());
//		jpComment.setPreferredSize(new java.awt.Dimension(200, 200));

		//put a gridbaglayout for this component

//		jpanelsub3.add(jpComment);


//		jScrollPaneComment.setMinimumSize(new Dimension(200, 200));
		jScrollPaneComment.setPreferredSize(new Dimension(400, 400));
		jScrollPaneComment.setMaximumSize(new Dimension(Short.MAX_VALUE,
				Short.MAX_VALUE));
		jpanelsub3.setLayout(new BoxLayout(jpanelsub3, BoxLayout.X_AXIS));
		jpanelsub3.setAlignmentX(Component.CENTER_ALIGNMENT);
		jpanelsub3.add(jScrollPaneComment);

//		jtaNewComment.setMinimumSize(new Dimension(500, 500));
		jtaNewComment.setPreferredSize(new Dimension(500, 500));
		jtaNewComment.setMaximumSize(new Dimension(Short.MAX_VALUE,
				Short.MAX_VALUE));
		jtaNewComment.setBorder(BorderFactory.createLineBorder(Color.GRAY));
		jtaNewComment.setText("");
		jtaNewComment.setText(" (new comment empty text) ");
		jpanelsub3.add(jtaNewComment);



		jbSaveComment.setFont(new java.awt.Font("Dialog", 0, 12));
		jbSaveComment.setText("Save");
		jbSaveComment.setToolTipText("save comment");

		jpanelsub3.add(jbSaveComment);


		jbSaveComment.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jbSaveCommentPerformed(evt);
			}
		});


		jtPsubChart.addMouseListener(new MouseListener() {
			int x0, y0, x1, y1;

			public void mousePressed(MouseEvent e) {
				x0 = e.getX();
				y0 = e.getY();
//				System.out.println("mouse clicked" + "x=" + x0 + "y=" + y0
//				+ "\n");
			}

			public void mouseReleased(MouseEvent e) {
				x1 = e.getX();
				y1 = e.getY();
//				System.out.println("mouse released" + "x=" + x1 + "y=" + y1
//				+ "\n");
				if (jtPsubChart.indexAtLocation(x0, y0) != jtPsubChart
						.indexAtLocation(x1, y1)) {
					// System.out.println("not same index
					// location\n"+jtPsubChart.indexAtLocation(x0,y0)+" et
					// "+jtPsubChart.indexAtLocation(x1,y1)+"\n");
					// inserer a la bonne position
					if (jtPsubChart.indexAtLocation(x0, y0) < jtPsubChart
							.indexAtLocation(x1, y1)) {
						if (jtPsubChart.indexAtLocation(x0, y0) >= 0
								&& jtPsubChart.indexAtLocation(x0, y0) < jtPChart
								.getTabCount()) {
							jtPsubChart.add(jtPsubChart
									.getComponentAt(jtPsubChart
											.indexAtLocation(x0, y0)),
											jtPsubChart.getTitleAt(jtPsubChart
													.indexAtLocation(x0, y0)),
													jtPsubChart.indexAtLocation(x1, y1) + 1);
							jtPsubChart.setSelectedIndex(jtPsubChart
									.indexAtLocation(x1, y1));
						}
					}
					// inserer dans l'autre sens
					else if (jtPsubChart.indexAtLocation(x0, y0) >= 0
							&& jtPsubChart.indexAtLocation(x0, y0) < jtPChart
							.getTabCount()) {
						// System.out.println("inverse \n");
						jtPsubChart.add(jtPsubChart.getComponentAt(jtPsubChart
								.indexAtLocation(x0, y0)),
								jtPsubChart.getTitleAt(jtPsubChart
										.indexAtLocation(x0, y0)), jtPsubChart
										.indexAtLocation(x1, y1));
						jtPsubChart.setSelectedIndex(jtPsubChart
								.indexAtLocation(x1, y1));
					}
				} else {
					// System.out.println("it's same index location
					// \n"+jtPsubChart.indexAtLocation(x1,y1) );
				}
			}

			public void mouseClicked(MouseEvent e) {

			}

			public void mouseEntered(MouseEvent e) {
			}

			public void mouseExited(MouseEvent e) {
			}

		});

		jtPsubChart.addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseDragged(MouseEvent e) {
				// System.out.print("Dragged to "+e.getPoint()+"\n" );
			}
		});

		jpGraphOptions.add(jtPsubChart, java.awt.BorderLayout.CENTER);
		// jpanel1.add(jtPsubChart,java.awt.BorderLayout.WEST);
//		jpGraphOptionsTab.add(jpTimeStart, java.awt.BorderLayout.CENTER);
//		jpGraphOptionsTab.add(jpScreenTime, java.awt.BorderLayout.CENTER);
//		jpGraphOptionsTab.add(jpTimeEnd, java.awt.BorderLayout.CENTER);
//		jpGraphOptionsTab.add(jpTimeConf, java.awt.BorderLayout.CENTER);


		jbDrawGraph.setFont(new java.awt.Font("Dialog", 0, 12));
		jbDrawGraph.setText("Refresh");
		jbDrawGraph.setToolTipText("Refresh the graph");
		jbDrawGraph.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jbDrawGraphPerformed();
//				System.out.println("TO DRAW");
			}
		});

//		jpGraphOptionsTab.add(jbFilter, java.awt.BorderLayout.CENTER);


		////////////////////////////
		// MANAGEMENT GRAPH OPTIONS
		jbRefreshGraph.setFont(new java.awt.Font("Dialog", 0, 12));
		jbRefreshGraph.setText("Refresh");
		jbRefreshGraph.setToolTipText("Refresh the graph");

		//jpDraw1
		jcbRaw.setSelected(true);
		jpDraw1.setLayout(new GridLayout(0,1));
		jcbRaw.setFont(new java.awt.Font("Dialog", 0, 12));
		jcbRaw.setText("Raw");
		jcbRaw.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				JCheckBox jcb = (JCheckBox) e.getSource();
				if(jcb.isSelected()){
					if(jcbMoving.isSelected()==false){
						Component[] compList = jpDraw2.getComponents();
						for (int i = 0; i < compList.length; i++) {
							compList[i].setEnabled(false);
						}
					}
				}
				enableOnlyTimeChart();
			}
		});
		jpDraw1.add(jcbRaw);
		jcbMoving.setFont(new java.awt.Font("Dialog", 0, 12));
		jcbMoving.setText("Moving");
		jcbMoving.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				JCheckBox jcb = (JCheckBox) e.getSource();
				if(jcb.isSelected()){
					Component[] compList = jpDraw2.getComponents();
					for (int i = 0; i < compList.length; i++) {
						compList[i].setEnabled(true);
					}
				}
				else{
					Component[] compList = jpDraw2.getComponents();
					for (int i = 0; i < compList.length; i++) {
						compList[i].setEnabled(false);
					}
				}
				enableOnlyTimeMovingChart();
			}

		});
		jpDraw1.add(jcbMoving);
		jcbHistogram.setFont(new java.awt.Font("Dialog", 0, 12));
		jcbHistogram.setText("Histogram");
		jcbHistogram.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				JCheckBox jcb = (JCheckBox) e.getSource();
				if(jcb.isSelected()){
					if(jcbMoving.isSelected()==false){
						Component[] compList = jpDraw2.getComponents();
						for (int i = 0; i < compList.length; i++) {
							compList[i].setEnabled(false);
						}
					}
				}
			}
		});
		jbgDraw1.add(jcbRaw);
		jbgDraw1.add(jcbMoving);
//		jbgDraw1.add(jcbHistogram);
//		jpDraw1.add(jcbHistogram);
		//jpDraw2

		jpDraw2.setLayout(new GridLayout(0,1));
		jcbMin.setFont(new java.awt.Font("Dialog", 0, 12));
		jcbMin.setText("Minimum");
		jpDraw2.add(jcbMin);
		jcbMax.setFont(new java.awt.Font("Dialog", 0, 12));
		jcbMax.setText("Maximum");
		jpDraw2.add(jcbMax);
		jcbMean.setFont(new java.awt.Font("Dialog", 0, 12));
		jcbMean.setText("Mean");
		jpDraw2.add(jcbMean);
//		jcbMedian.setFont(new java.awt.Font("Dialog", 0, 12));
//		jcbMedian.setText("Median");
//		jpDraw2.add(jcbMedian);
		jcbStdDvt.setFont(new java.awt.Font("Dialog", 0, 12));
		jcbStdDvt.setText("Standard deviation");
		jpDraw2.add(jcbStdDvt);
		
		jcbThroughput.setFont(new java.awt.Font("Dialog", 0, 12));
		jcbThroughput.setText("Throughput");
		jpDraw2.add(jcbThroughput);

		jbgDraw2.add(jcbMin);
		jbgDraw2.add(jcbMax);
		jbgDraw2.add(jcbMean);
//		jbgDraw2.add(jcbMedian);
		jbgDraw2.add(jcbStdDvt);
		jbgDraw2.add(jcbThroughput);
		//disable all components
		Component[] compList = jpDraw2.getComponents();
		for (int i = 0; i < compList.length; i++) {
			compList[i].setEnabled(false);
		}
		//jpDrawConfig
		jpDrawConfig.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
////		natural height, maximum width
//		c.fill = GridBagConstraints.HORIZONTAL;
//		c.gridx = 0;
//		c.gridy = 0;
//		jpDrawConfig.add(jpTimeStart, c);
//
//		c.fill = GridBagConstraints.HORIZONTAL;
//		c.gridx = 1;
//		c.gridy = 0;
//		jpDrawConfig.add(jpScreenTime, c);
//
//		c.fill = GridBagConstraints.HORIZONTAL;
//		c.gridx = 2;
//		c.gridy = 0;
//		jpDrawConfig.add(jpTimeEnd, c);

		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1;
		c.gridy = 1;
		jlDatasetMaxPoints.setFont(new java.awt.Font("Dialog", 0, 12));
		jlDatasetMaxPoints.setText("Maximum number of points ");
		jtfDatasetMaxPoints.setPreferredSize(new Dimension(80, 17));
		jtfDatasetMaxPoints.setMaximumSize(new Dimension(80, 17));
		jtfDatasetMaxPoints.setMinimumSize(new Dimension(80, 17));
		jpDatasetMaxPoints.setLayout(new BoxLayout(jpDatasetMaxPoints, BoxLayout.X_AXIS));
		jpDatasetMaxPoints.add(jlDatasetMaxPoints);
		jtfDatasetMaxPoints.setText(String.valueOf(maxPointsToDisplay));
		jtfDatasetMaxPoints.addKeyListener(new java.awt.event.KeyListener(){
			public void keyPressed(KeyEvent evt) {
				//JTextField modification
				jtfDatasetMaxPoints.setBackground(Color.YELLOW);
				if(evt.getKeyCode() == KeyEvent.VK_ENTER){
					//put focus on the parent
					maxPointsToDisplay = Integer.parseInt(jtfDatasetMaxPoints.getText());
					jtfDatasetMaxPoints.setBackground(Color.WHITE);
					jtfDatasetMaxPoints.getParent().requestFocus();
				}
			}
			public void keyReleased(KeyEvent e) {

			}
			public void keyTyped(KeyEvent e) {

			}
		});
		jpDatasetMaxPoints.add(jtfDatasetMaxPoints);
		jpDrawConfig.add(jpDatasetMaxPoints, c);
				
		
		
		jpDatasetWindowStep.setLayout(new BoxLayout(jpDatasetWindowStep, BoxLayout.X_AXIS));
		
		jlDatasetTimeWindow.setFont(new java.awt.Font("Dialog", 0, 12));
		jlDatasetTimeWindow.setText("Time Window  ");
		jpDatasetWindowStep.add(jlDatasetTimeWindow);
		
		jtfDatasetTimeWindow = new JTextField();
		jtfDatasetTimeWindow.setPreferredSize(new Dimension(80, 17));
		jtfDatasetTimeWindow.setMaximumSize(new Dimension(80, 17));
		jtfDatasetTimeWindow.setMinimumSize(new Dimension(80, 17));
		jpDatasetWindowStep.add(jtfDatasetTimeWindow);
		
		jlDatasetStep.setFont(new java.awt.Font("Dialog", 0, 12));
		jlDatasetStep.setText("  Step  ");
		jpDatasetWindowStep.add(jlDatasetStep);

		jtfDatasetStep = new JTextField();
		jtfDatasetStep.setPreferredSize(new Dimension(80, 17));
		jtfDatasetStep.setMaximumSize(new Dimension(80, 17));
		jtfDatasetStep.setMinimumSize(new Dimension(80, 17));
		jpDatasetWindowStep.add(jtfDatasetStep);

		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1;
		c.gridy = 3;
		c.insets = new Insets(10,0,0,0); 
		jpDrawConfig.add(jpDatasetWindowStep, c);
		
		jpDatasetNumberOfSlices.setLayout(new BoxLayout(jpDatasetNumberOfSlices, BoxLayout.X_AXIS));
		jlDatasetNumberOfSlices.setFont(new java.awt.Font("Dialog", 0, 12));
		jlDatasetNumberOfSlices.setText("Number of slices ");
		jpDatasetNumberOfSlices.add(jlDatasetNumberOfSlices);
		jpDatasetNumberOfSlices.add(jtfDatasetNumberOfSlices);
		
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1;
		c.gridy = 4;
		c.insets = new Insets(10,0,0,0); 
		jpDrawConfig.add(jpDatasetNumberOfSlices, c);


//		jpDrawConfig.add(jpTimeStart, java.awt.BorderLayout.CENTER);
//		jpDrawConfig.add(jpScreenTime, java.awt.BorderLayout.CENTER);
//		jpDrawConfig.add(jpTimeEnd, java.awt.BorderLayout.CENTER);
//		jpDrawConfig.add(jpTimeConf, java.awt.BorderLayout.CENTER);

		//jpDrawButtons
		jpDrawButtons.add(jbDrawGraph, java.awt.BorderLayout.CENTER);
//		jpDrawButtons.add(jbRefreshGraph, java.awt.BorderLayout.CENTER);

//		jpGraphOptionsTab.add(jpDraw1);
//		jpGraphOptionsTab.add(jpDraw2);
//		jpGraphOptionsTab.add(jpDrawConfig);
//		jpGraphOptionsTab.add(jpDrawButtons);

//		jpDraw1.setBorder(javax.swing.BorderFactory
//		.createTitledBorder(null, "1",
//		javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
//		javax.swing.border.TitledBorder.DEFAULT_POSITION,
//		new java.awt.Font("Dialog", 1, 11), new java.awt.Color(
//		0, 102, 204)));
//		jpDraw2.setBorder(javax.swing.BorderFactory
//		.createTitledBorder(null, "2",
//		javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
//		javax.swing.border.TitledBorder.DEFAULT_POSITION,
//		new java.awt.Font("Dialog", 1, 11), new java.awt.Color(
//		0, 102, 204)));
//		jpDrawConfig.setBorder(javax.swing.BorderFactory
//		.createTitledBorder(null, "3",
//		javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
//		javax.swing.border.TitledBorder.DEFAULT_POSITION,
//		new java.awt.Font("Dialog", 1, 11), new java.awt.Color(
//		0, 102, 204)));
//		jpDrawButtons.setBorder(javax.swing.BorderFactory
//		.createTitledBorder(null, "4",
//		javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
//		javax.swing.border.TitledBorder.DEFAULT_POSITION,
//		new java.awt.Font("Dialog", 1, 11), new java.awt.Color(
//		0, 102, 204)));

//		jpGraphOptionsTab.setLayout(new BoxLayout(jpGraphOptionsTab, BoxLayout.X_AXIS));
//		jpGraphOptionsTab.setAlignmentX(Component.CENTER_ALIGNMENT);
//		jpGraphOptionsTab.add(jpDraw1);
//		jpGraphOptionsTab.add(jpDraw2);
//		jpGraphOptionsTab.add(jpDrawConfig);
//		jpGraphOptionsTab.add(jpDrawButtons);

		jpDatasetOptionsTab.setLayout(new GridBagLayout());
		GridBagConstraints d = new GridBagConstraints();
		//natural height, maximum width
		d.fill = GridBagConstraints.VERTICAL;
		d.gridx = 0;
		d.gridy = 0;
		jpDatasetOptionsTab.add(jpDraw1, d);

		d.fill = GridBagConstraints.VERTICAL;
		d.gridx = 1;
		d.gridy = 0;
		jpDatasetOptionsTab.add(jpDraw2, d);

		d.fill = GridBagConstraints.VERTICAL;
		d.gridx = 2;
		d.gridy = 0;
		jpDatasetOptionsTab.add(jpDrawConfig, d);

		d.fill = GridBagConstraints.VERTICAL;
		d.gridx = 3;
		d.gridy = 0;
		jpDatasetOptionsTab.add(jpDrawButtons, d);

		jpDraw1.setVisible(false);
		jpDraw2.setVisible(false);
		jpDrawButtons.setVisible(false);
		jpDrawConfig.setVisible(false);

		jpGraph.add(jpGraphParameters, java.awt.BorderLayout.CENTER);
		jpGraph.add(jtPChart, java.awt.BorderLayout.CENTER);
//		jspTopBottom.setDividerLocation(700);
//		this.add(jspTopBottom, java.awt.BorderLayout.CENTER);
		this.add(jpMain, java.awt.BorderLayout.CENTER);

		jspGraph.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);

		jspGraph.setTopComponent(jpGraph);
//		jspGraph.setTopComponent(frame);
		jspGraph.setBottomComponent(jtPChart);
		
		
		//TO DO
//		jtPChart.addChangeListener(new ChangeListener(){
//			public void stateChanged(ChangeEvent e) {
//						JScrollPane jp = (JScrollPane) jtPChart.getSelectedComponent();
//						String tabName = jp.getName();
//						int pageIndex = Integer.parseInt(tabName);
//						PageOptions po = pageOptionsMap.get(pageIndex);
//						if(po != null){
//							updatePageOptions(po);	
//						}
//			}
//		});
		
		jspGraph.setDividerLocation(350);
		jpGraphManager.add(jspGraph, java.awt.BorderLayout.CENTER);

		jspGraphNorth.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);

		jspGraphNorth.setTopComponent(jpGraphParameters);
		jspGraphNorth.setBottomComponent(jpGraphOptions);
		jpGraph.add(jspGraphNorth, java.awt.BorderLayout.CENTER);

	}
	
	
	// ///////////////////////////////
	// Listeners //
	// ///////////////////////////////
	


	/**
	 * Save comment in a map
	 * @param evt
	 */
	protected void jbSaveCommentPerformed(ActionEvent evt) {
		if(jTreeComment.isSelectionEmpty() == false){
			TreePath[] tpArray = jTreeComment.getSelectionPaths();
			String tpString = "";
			for (int i = 0; i < tpArray.length; i++) {
				tpString += tpArray[i].toString();
			}
			String comment = jtaNewComment.getText();
			System.out.println("path :" + tpString);
			mapTreeComment.put(tpString, comment);
		}
		//no selected a value  
		else{
//			JOptionPane.showMessageDialog(this.globalGUI	// removed (irritating)
//					.getInternalFrame(),
//					"You haven't selected a value for this comment",
//					"Warning", JOptionPane.WARNING_MESSAGE);
		}
	}

	/**
	 * Draw a chart associated with a dataset 
	 * @param ActionEvent
	 */
	private void jbDrawGraphPerformed() {
		//to modify with the dataset Type
//		try {
			//depending on XField and YField, XField will often be the date(ms) 
			
			//get the row selected in the table
			int rowSelected = jtYAxisValues.getSelectedRow();
			if(rowSelected != -1 && 
					(jtYAxisValues.getValueAt(rowSelected, FIELD) != null) && 
					(jtYAxisValues.getValueAt(rowSelected, PAGE).toString()!="New Page")){
				//get associate id
				int chartId = Integer.parseInt(jtYAxisValues.getValueAt(rowSelected, CHART_ID).toString());
				//get the simpleDataset associated
				SimpleDataset sd = simpleDatasetMap.get(chartId);
				//get associate page
				int pageId = Integer.parseInt(jtYAxisValues.getValueAt(rowSelected, PAGE).toString());
				//field selected
				String field = jtYAxisValues.getValueAt(rowSelected, FIELD).toString();
				//get the statistic 
				List<StatisticDataImpl> sdi = statisticMap.get(chartId);
				//JFreeChart created
				JFreeChart jfChart = null;
				//Chart data created
				ChartData chartDataCurrent = chartDataList.get(pageId);
				
				StatisticDataImpl sdiSimple;
				//JFREE CHART CREATION
				
				//set draw selected
				jtYAxisValues.setValueAt(true, rowSelected, TO_DRAW);
				
				//set the time window and the step conformed to Jmenu
				timeWindow = globalGUI.getTimeWindow();
				step = globalGUI.getStep();
			
				
				//simple or aggregate dataset
				if(sd.getDatasetType() == 0 ||
						sd.getDatasetType() == 2){
					int timeStart = sd.getDrawTimeStart();
					int timeEnd = sd.getDrawTimeEnd();
					//only one in the list
					Iterator<StatisticDataImpl> it = sdi.iterator();
					while (it.hasNext()) {
						sdiSimple = (StatisticDataImpl) it.next();
					//create a new chart if it's null
					if(chartDataCurrent == null){
						//create a new chart data
						chartDataCurrent = new ChartData(chartId, pageId);
					}
					//raw data
					if(jcbRaw.isSelected() == true){
						//update the chartData with this this chart
						jfChart = chartDataCurrent.createSimpleChart("ms", field, sdiSimple, chartId, "RAW", timeWindow, step, maxPointsToDisplay,
								timeStart,
								timeEnd);	
						DatasetOptions go = new DatasetOptions(chartId, true, "RAW", maxPointsToDisplay, timeStart, timeEnd);
						datasetIdAndDatasetOptions.put(chartId, go);
					}
					//moving data
					else if(jcbMoving.isSelected() == true){
						if(jcbMin.isSelected() == true){
							jfChart = chartDataCurrent.createSimpleChart(
									"ms", 
									field, 
									sdiSimple, 
									chartId, 
									"MOVINGMIN", 
									timeWindow, 
									step, 
									maxPointsToDisplay,
									timeStart,
									timeEnd);	
							DatasetOptions go = new DatasetOptions(chartId, true, "MOVINGMIN", maxPointsToDisplay, timeStart, timeEnd);
							datasetIdAndDatasetOptions.put(chartId, go);
						}
						else if(jcbMax.isSelected() == true){
							jfChart = chartDataCurrent.createSimpleChart("ms", field, sdiSimple, chartId, "MOVINGMAX", timeWindow, step, maxPointsToDisplay, timeStart, timeEnd);	
							DatasetOptions go = new DatasetOptions(chartId, true, "MOVINGMAX", maxPointsToDisplay, timeStart, timeEnd);
							datasetIdAndDatasetOptions.put(chartId, go);
						}
						else if(jcbMean.isSelected() == true){
							jfChart = chartDataCurrent.createSimpleChart("ms", field, sdiSimple, chartId, "MOVINGMEAN", timeWindow, step, maxPointsToDisplay, timeStart, timeEnd);	
							DatasetOptions go = new DatasetOptions(chartId, true, "MOVINGMEAN",maxPointsToDisplay, timeStart, timeEnd);
							datasetIdAndDatasetOptions.put(chartId, go);
						}
						else if(jcbStdDvt.isSelected() == true){
							jfChart = chartDataCurrent.createSimpleChart("ms", field, sdiSimple, chartId, "MOVINGSTDDVT", timeWindow, step, maxPointsToDisplay, timeStart, timeEnd);	
							DatasetOptions go = new DatasetOptions(chartId, true, "MOVINGSTDDVT",maxPointsToDisplay, timeStart, timeEnd);
							datasetIdAndDatasetOptions.put(chartId, go);
						}
						else if(jcbThroughput.isSelected() == true){
							jfChart = chartDataCurrent.createSimpleChart("ms", "requestsPerSecond", sdiSimple, chartId, "MOVINGTHROUGHPUT", timeWindow, step, maxPointsToDisplay, timeStart, timeEnd);
							DatasetOptions go = new DatasetOptions(chartId, true, "MOVINGTHROUGHPUT", maxPointsToDisplay, timeStart, timeEnd);
							datasetIdAndDatasetOptions.put(chartId, go);
						}
					}	
					//histogram
					else if(jcbPageHistogramChart.isSelected() == true){
//						jfChart = chartDataCurrent.createHistoChart("ms", "slice", sdiSimple, chartId, "HISTOGRAM", timeWindow, step, maxPointsToDisplay, timeStart, timeEnd);
						DatasetOptions go = new DatasetOptions(chartId, true, "MOVINGTHROUGHPUT", maxPointsToDisplay, timeStart, timeEnd);
						datasetIdAndDatasetOptions.put(chartId, go);
					}
					}
				}
				//multiple dataset
				else{
					int timeStart = sd.getDrawTimeStart();
					int timeEnd = sd.getDrawTimeEnd();
					//create a new chart if it's null
					if(chartDataCurrent == null){
						//create a new chart data
						chartDataCurrent = new ChartData(chartId, pageId);
					}
					//raw data
					if(jcbRaw.isSelected() == true){
						//update the chartData with this this chart
						jfChart = chartDataCurrent.createMultipleChart("ms", field, sdi, chartId, "RAW", timeWindow, step, maxPointsToDisplay, timeStart, timeEnd);	
						DatasetOptions go = new DatasetOptions(chartId, true, "RAW", maxPointsToDisplay, timeStart, timeEnd);
						datasetIdAndDatasetOptions.put(chartId, go);
					}
					//moving data
					if(jcbMoving.isSelected() == true){
						if(jcbMin.isSelected() == true){
							jfChart = chartDataCurrent.createMultipleChart("ms", field, sdi, chartId, "MOVINGMIN", timeWindow, step, maxPointsToDisplay, timeStart, timeEnd);	
							DatasetOptions go = new DatasetOptions(chartId, true, "MOVINGMIN", maxPointsToDisplay, timeStart, timeEnd);
							datasetIdAndDatasetOptions.put(chartId, go);
						}
						else if(jcbMax.isSelected() == true){
							jfChart = chartDataCurrent.createMultipleChart("ms", field, sdi, chartId, "MOVINGMAX", timeWindow, step, maxPointsToDisplay, timeStart, timeEnd);	
							DatasetOptions go = new DatasetOptions(chartId, true, "MOVINGMAX", maxPointsToDisplay, timeStart, timeEnd);
							datasetIdAndDatasetOptions.put(chartId, go);
						}
						else if(jcbMean.isSelected() == true){
							jfChart = chartDataCurrent.createMultipleChart("ms", field, sdi, chartId, "MOVINGMEAN", timeWindow, step, maxPointsToDisplay, timeStart, timeEnd);	
							DatasetOptions go = new DatasetOptions(chartId, true, "MOVINGMEAN", maxPointsToDisplay, timeStart, timeEnd);
							datasetIdAndDatasetOptions.put(chartId, go);
						}
						else if(jcbStdDvt.isSelected() == true){
							jfChart = chartDataCurrent.createMultipleChart("ms", field, sdi, chartId, "MOVINGSTDDVT", timeWindow, step, maxPointsToDisplay, timeStart, timeEnd);	
							DatasetOptions go = new DatasetOptions(chartId, true, "MOVINGSTDDVT", maxPointsToDisplay, timeStart, timeEnd);
							datasetIdAndDatasetOptions.put(chartId, go);
						}
						else if(jcbThroughput.isSelected() == true){
							jfChart = chartDataCurrent.createMultipleChart("ms", "requestsPerSecond", sdi, chartId, "MOVINGTHROUGHPUT", timeWindow, step, maxPointsToDisplay, timeStart, timeEnd);	
							DatasetOptions go = new DatasetOptions(chartId, true, "MOVINGTHROUGHPUT", maxPointsToDisplay, timeStart, timeEnd);
							datasetIdAndDatasetOptions.put(chartId, go);
						}
					}	
					//moving data
					
					//histogram
					
				}
				
				//save in a map
				chartDataList.put(pageId, chartDataCurrent);
				
				//save the jFreeChart for this page 
				pageIdAndJFreeChart.put(pageId, jfChart);
				
				//JFREE CHART DISPLAY
				
				ChartPanel testPanel = new ChartPanel(chartDataCurrent.getJfChart());
				
				//display on correct page
				Component[] comList = jtPChart.getComponents();
				for (int i = 0; i < comList.length; i++) {
					if(comList[i].getName().equalsIgnoreCase(String.valueOf(pageId))){
						JScrollPane jp = (JScrollPane) comList[i];
						int v = ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED;
						int h = ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED;
						jp.setVerticalScrollBarPolicy(v);
						jp.setHorizontalScrollBarPolicy(h);
						jp.setViewportView(testPanel);
					}
				}
				jtPChart.repaint();
			}
	}
	
	/**
	 * Update chart
	 * @param jfc JFreeChart
	 * @param pageId the page Id
	 * @param chartDataCurrent ChartData
	 * @param chartId the chart Id
	 */
	private void updateChart(JFreeChart jfc,int  pageId, ChartData chartDataCurrent, int chartId){
		//save in a map
		chartDataList.put(pageId, chartDataCurrent);
		
		//save the jFreeChart for this page 
		pageIdAndJFreeChart.put(pageId, jfc);
		
		//JFREE CHART DISPLAY
		
		ChartPanel testPanel = new ChartPanel(chartDataCurrent.getJfChart());
		
		//display on correct page
		Component[] comList = jtPChart.getComponents();
		for (int i = 0; i < comList.length; i++) {
			if(comList[i].getName().equalsIgnoreCase(String.valueOf(pageId))){
				JScrollPane jp = (JScrollPane) comList[i];
				int v = ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED;
				int h = ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED;
				jp.setVerticalScrollBarPolicy(v);
				jp.setHorizontalScrollBarPolicy(h);
				jp.setViewportView(testPanel);
			}
		}
		jtPChart.repaint();
	}
	
	/**
	 * Refresh action
	 * 
	 * @param evt
	 */
	private void jbRefreshActionPerformed(java.awt.event.ActionEvent evt) {
		DefaultMutableTreeNode refreshRoot = this.globalGUI
				.refreshExecutionTree((DefaultMutableTreeNode) jtExecutionsTree
						.getModel().getRoot());
		executionsTreeModel.setRoot(refreshRoot);
		jtExecutionsTree.setModel(executionsTreeModel);
	}

	/**
	 * Max point focus lost
	 * 
	 * @param evt
	 */
	private void jtfMaxPointFocusLost(java.awt.event.FocusEvent evt) {
		if (jbValidateMaxPoint.isVisible() && jbValidateMaxPoint.isEnabled()) {
			// Number format test
			try {
				Integer.parseInt(jtfMaxPoint.getText());
				jbValidateMaxPoint.setEnabled(true);
			} catch (NumberFormatException e) {
				jbValidateMaxPoint.setEnabled(false);
				JOptionPane.showMessageDialog(
						this.globalGUI.getInternalFrame(),
						"Only integer value for maximum number of point.",
						"Warning", JOptionPane.WARNING_MESSAGE);
				jtfMaxPoint.setBackground(ERROR_AREA_COLOR);
				return;
			}

			jtfMaxPoint.setBackground(MODIFIED_AREA_COLOR);
		}
	}



	/**
	 * Graph type management
	 * 
	 * @param evt
	 */
	private void jcbGraphTypeMouseClicked(java.awt.event.MouseEvent evt) {
		// Set Foreground
		jlGraphTypeTitle.setForeground(MODIFIED_AREA_COLOR);
		jlGraphTypeTitle.setFont(new java.awt.Font("Dialog", 1, 12));
	}

	/**
	 * Row data
	 * 
	 * @param evt
	 */
	private void jrbRowDataActionPerformed(java.awt.event.ActionEvent evt) {
		jrbTypeMean.setEnabled(false);
		jrbTypeStandardDeviation.setEnabled(false);
		jrbTypeMin.setEnabled(false);
		jrbTypeMax.setEnabled(false);

		// Set Foreground
		jlDataType.setForeground(MODIFIED_AREA_COLOR);
		jlDataType.setFont(new java.awt.Font("Dialog", 1, 12));
	}

	/**
	 * Moving data
	 * 
	 * @param evt
	 */
	private void jrbMovingStatActionPerformed(java.awt.event.ActionEvent evt) {
		jrbTypeMean.setEnabled(true);
		jrbTypeStandardDeviation.setEnabled(true);
		jrbTypeMin.setEnabled(true);
		jrbTypeMax.setEnabled(true);

		// Set Foreground
		jlDataType.setForeground(MODIFIED_AREA_COLOR);
		jlDataType.setFont(new java.awt.Font("Dialog", 1, 12));
	}

	/**
	 * Mean
	 * 
	 * @param evt
	 */
	private void jrbTypeMeanActionPerformed(java.awt.event.ActionEvent evt) {
		jrbMovingStatActionPerformed(evt);
	}

	/**
	 * Minimum
	 * 
	 * @param evt
	 */
	private void jrbTypeMinActionPerformed(java.awt.event.ActionEvent evt) {
		jrbMovingStatActionPerformed(evt);
	}

	/**
	 * Standard deviation
	 * 
	 * @param evt
	 */
	private void jrbTypeStandardDeviationActionPerformed(
			java.awt.event.ActionEvent evt) {
		jrbMovingStatActionPerformed(evt);
	}

	/**
	 * Maximum
	 * 
	 * @param evt
	 */
	private void jrbTypeMaxActionPerformed(java.awt.event.ActionEvent evt) {
		jrbMovingStatActionPerformed(evt);
	}
	

	/**
	 * Yoptions 
	 * 
	 * @param evt
	 */
	private void jbValidateYAxisOptionsActionPerformed(
			java.awt.event.ActionEvent evt) {
		if (jtYAxisValues.getSelectedRow() != -1) {
			// Save options
			ArrayList<String> tmpPropLst = yAxisProperties.get(yAxisTableModel
					.getValueAt(jtYAxisValues.getSelectedRow(), 3).toString());
			// Save Y Label
			tmpPropLst.set(0, jtfYAxisLabel.getText());
			yAxisTableModel.setValueAt(tmpPropLst.get(0).toString(),
					jtYAxisValues.getSelectedRow(), 1);

			// Save Chart type
			tmpPropLst.set(2, jcbGraphType.getSelectedItem().toString());

			// Save Data Type
			if (jrbRowData.isSelected()) {
				tmpPropLst.set(3, "rawdata:");
			} else if (jrbMovingStat.isSelected()) {
				if (jrbTypeMean.isSelected()) {
					tmpPropLst.set(3, "movingstat:mean");
				} else if (jrbTypeStandardDeviation.isSelected()) {
					tmpPropLst.set(3, "movingstat:stdeviation");
				} else if (jrbTypeMin.isSelected()) {
					tmpPropLst.set(3, "movingstat:min");
				} else if (jrbTypeMax.isSelected()) {
					tmpPropLst.set(3, "movingstat:max");
				}
			}

			// Options panel non visible
			jpYAxisProperties.setVisible(false);

			// Set validate button visible to false
			jbValidateYAxisOptions.setVisible(false);

			// Do actions
			String[] splitPath = yAxisTableModel.getValueAt(
					jtYAxisValues.getSelectedRow(), 3).toString().split("/");

			if (jlGraphTypeTitle.getForeground() == MODIFIED_AREA_COLOR) {
				// ComboBox was modified
				this.globalGUI.setChartType(splitPath[1], splitPath[2],
						splitPath[3], splitPath[4], chartsTypes.get(
								tmpPropLst.get(2)).toString());

				// Set line color
				String[] splitColor = tmpPropLst.get(1).toString().split(":");
				this.setLineChartColor(new Color(Integer
						.parseInt(splitColor[0]), Integer
						.parseInt(splitColor[1]), Integer
						.parseInt(splitColor[2])));
			}

			if (jlDataType.getForeground() == MODIFIED_AREA_COLOR) {
				// Set Y axis data type
				String movingStatistic = "";
				if (jrbRowData.isSelected()) {
					movingStatistic = "";
				} else if (jrbMovingStat.isSelected()) {
					if (jrbTypeMean.isSelected()) {
						movingStatistic = DataSupplier.STAT_MEANS;
					} else if (jrbTypeStandardDeviation.isSelected()) {
						movingStatistic = DataSupplier.STAT_STD_DEVIATION;
					} else if (jrbTypeMin.isSelected()) {
						movingStatistic = DataSupplier.STAT_MINIMUMS;
					} else if (jrbTypeMax.isSelected()) {
						movingStatistic = DataSupplier.STAT_MAXIMUMS;
					}
				}
				this.globalGUI.setYAxisMovingStatistics(splitPath[1],
						splitPath[2], splitPath[3], splitPath[4],
						movingStatistic);
			}

			if (jtfYAxisLabel.getBackground() == MODIFIED_AREA_COLOR) {
				// Set Y Axis Label
				this.globalGUI.setYAxisLabel(splitPath[1], splitPath[2],
						splitPath[3], splitPath[4], tmpPropLst.get(0)
								.toString());
			}

			// Set Default Foreground
			jlGraphTypeTitle.setForeground(javax.swing.UIManager.getDefaults()
					.getColor("Label.foreground"));
			jlGraphTypeTitle.setFont(new java.awt.Font("Dialog", 0, 12));
			jlDataType.setForeground(javax.swing.UIManager.getDefaults()
					.getColor("Label.foreground"));
			jlDataType.setFont(new java.awt.Font("Dialog", 0, 12));

			jtfYAxisLabel
					.setBackground(DataQuickViewContainer.DEFAULT_TEXTFIELD_COLOR);
			jtYAxisValues.setForeground(DataQuickViewContainer.FORE_AREA_COLOR);
			jtYAxisValues
					.setSelectionForeground(DataQuickViewContainer.FORE_AREA_COLOR);
			jtYAxisValues
					.setSelectionBackground(DataQuickViewContainer.BACK_AREA_COLOR);
//			jtYAxisValues.getSelectionModel().clearSelection();
		}
	}

	/**
	 * Max points
	 * 
	 * @param evt
	 */
	private void jtfMaxPointKeyPressed(java.awt.event.KeyEvent evt) {
		// Set Background
		jtfMaxPoint.setBackground(MODIFIED_AREA_COLOR);

		if (!jbValidateMaxPoint.isVisible()
				&& evt.getKeyCode() != KeyEvent.VK_ENTER) {
			// Set validate button visible to true
			jbValidateMaxPoint.setVisible(true);
		} else {
			// Enter key
			if (jbValidateMaxPoint.isVisible()
					&& evt.getKeyCode() == KeyEvent.VK_ENTER) {
				// Number format test
				try {
					Integer.parseInt(jtfMaxPoint.getText());
					jbValidateMaxPoint.setEnabled(true);
				} catch (NumberFormatException e) {
					jbValidateMaxPoint.setEnabled(false);
					JOptionPane.showMessageDialog(this.globalGUI
							.getInternalFrame(),
							"Only integer value for maximum number of point.",
							"Warning", JOptionPane.WARNING_MESSAGE);
					jtfMaxPoint.setBackground(ERROR_AREA_COLOR);
					return;
				}
				// It's a number
				jtfMaxPoint.setBackground(MODIFIED_AREA_COLOR);
			}
		}
	}

	/**
	 * Xlabel
	 * 
	 * @param evt
	 */
	private void jtfXLabelKeyPressed(java.awt.event.KeyEvent evt) {
		// Set Background
		jtfXLabel.setBackground(MODIFIED_AREA_COLOR);

		if (!jbValidateXLabel.isVisible()) {
			// Set validate button visible to true
			jbValidateXLabel.setVisible(true);
		}

		if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
			jbValidateXLabelActionPerformed(null);
		}
	}

	/**
	 * X label
	 * 
	 * @param evt
	 */
	private void jbValidateXLabelActionPerformed(java.awt.event.ActionEvent evt) {
		// Set Default Background
		jtfXLabel.setBackground(DEFAULT_TEXTFIELD_COLOR);

		// Set validate button visible to false
		jbValidateXLabel.setVisible(false);

		// Do action
		jpDiagram.setVisible(false);
		this.globalGUI.setXAxisLabel(jtfXLabel.getText());
		jpDiagram.setVisible(true);
	}

	
	
	
	/**
	 * ADD TO Y AXIS
	 * 
	 * @param evt
	 */
	private void jbAddToYAxisActionPerformed(java.awt.event.ActionEvent evt) {
		TreeSelectionModel curentSelModel = jtExecutionsTree
				.getSelectionModel();
		// if curentSelModel exists and the selected model is an field
		// attribute( == 5)
		if (curentSelModel != null
				&& !curentSelModel.isSelectionEmpty()
				&& ((TreePath) curentSelModel.getLeadSelectionPath())
						.getPathCount() == 5) {
			Vector<String> addSel = new Vector<String>();
			String selPath = "";

			for (int i = 0; i < 5; i++) {
				selPath = selPath
						+ ((TreePath) curentSelModel.getLeadSelectionPath())
								.getPathComponent(i).toString() + "/";
			}
			selPath = selPath.substring(0, (selPath.length() - 1));
			// check if the chart isn't already in the tab
			if (!yAxisProperties.containsKey(selPath)) {
				// Update chart
				String[] splitPath = selPath.split("/");
				globalGUI.addYAxisInChart(splitPath[1], splitPath[2],
						splitPath[3], splitPath[4]);
				System.out.println("XYSeries__");
				if (this.isFirstYAxis) {
					// Add Chart Panel
					jpDiagram.setVisible(false);
					jpDiagram.add(globalGUI.getChartPanel(),
							BorderLayout.CENTER);
					jpDiagram.setVisible(true);
					this.isFirstYAxis = false;
				}

				// Update line chart color
				Color lineChartColor = this.globalGUI.getChartLineColor(
						splitPath[1], splitPath[2], splitPath[3], splitPath[4]);
				int r = lineChartColor.getRed();
				int g = lineChartColor.getGreen();
				int b = lineChartColor.getBlue();

				// Update JTable
				// Event field
				addSel.add(((TreePath) curentSelModel.getLeadSelectionPath())
						.getPathComponent(4).toString());
				// Y axis label
				addSel.add(((TreePath) curentSelModel.getLeadSelectionPath())
						.getPathComponent(4).toString());
				// Line color
				addSel.add(r + ":" + g + ":" + b);
				// Delete button
				addSel.add(selPath);
				yAxisTableModel.addRow(addSel);

				// Update Y Label
				jtfYAxisLabel
						.setText(((TreePath) curentSelModel
								.getLeadSelectionPath()).getPathComponent(4)
								.toString());
				// Set Y Axis Label
				this.globalGUI.setYAxisLabel(splitPath[1], splitPath[2],
						splitPath[3], splitPath[4], ((TreePath) curentSelModel
								.getLeadSelectionPath()).getPathComponent(4)
								.toString());

				// Update statistics view
				Map<String, Double> mapTmp = 
					this.globalGUI.getStatisticsFor(
						splitPath[1], 
						splitPath[2], 
						splitPath[3],
						splitPath[4]);
				statistics
						.put(selPath, mapTmp );
				this.updateStatisticsView();

				// Update Slider, max point
				long max = this.globalGUI.getMaxTimeFor(splitPath[1],
						splitPath[2]);
				if (this.maxTime < max) {
					this.maxTime = max;

				
					this.globalGUI
							.setDomainRange(Integer.parseInt(jtfTimeStart
									.getText()), Integer.parseInt(jtfTimeEnd
									.getText()));

					// Set Default Background
					jtfTimeStart.setBackground(DEFAULT_TEXTFIELD_COLOR);
					jtfTimeEnd.setBackground(DEFAULT_TEXTFIELD_COLOR);

					// Set validate button visible to false
					jbValidateTimeDisplay.setVisible(false);
				}

				// Add properties in HashMap
				ArrayList<String> tmpLst = new ArrayList<String>();
				tmpLst.add(((TreePath) curentSelModel.getLeadSelectionPath())
						.getPathComponent(4).toString());
				tmpLst.add(r + ":" + g + ":" + b);
				tmpLst
						.add(chartsTypes.keySet().toArray()[DataQuickViewContainer.DEFAULT_CHART_TYPE]
								.toString());
				tmpLst.add("rawdata:");
				yAxisProperties.put(selPath, tmpLst);

				yAxisAndNode.put(selPath, curentSelModel.getSelectionPath());

				jtfTimeStart.setEnabled(true);
				jsTime.setEnabled(true);
				jtfTimeEnd.setEnabled(true);
			} else {
				JOptionPane.showMessageDialog(
						this.globalGUI.getInternalFrame(),
						"Event Field already exists in chart.", "Warning",
						JOptionPane.INFORMATION_MESSAGE);
			}
		} else {
			JOptionPane.showMessageDialog(this.globalGUI.getInternalFrame(),
					"Please select an event field first.", "Warning",
					JOptionPane.INFORMATION_MESSAGE);
		}
	}
	
	/**
	 * Generate Id
	 * 
	 * @return id 
	 */
	private int generateId(){
		int id = generatedId;
		generatedId = generatedId + 1;
		return id;
	}
	
	
    
	/**
	 * Map this dataset
	 * @param simpleDataset
	 */
	public void addNewDataset(SimpleDataset simpleDataset) {
		//generate an id
		int idChart = generateId();
		this.currentIdDataset = idChart;
		//associate ID with the dataset, add the dataset to the dataset List
		simpleDatasetMap.put(idChart, simpleDataset);
		udpatepageIdAndDatasetIds();
	}
	
	/**
	 * @return current dataset id
	 */
	public int getCurrentIdDataset(){
		return this.currentIdDataset;
	}
	
	/**
	 * Modify dataset
	 * 
	 * @param simpleDataset
	 * @param key
	 */
	public void modifyDataset(SimpleDataset simpleDataset, int key) {
		//associate ID with the dataset, add the dataset to the dataset List
		simpleDatasetMap.put(key, simpleDataset);
	}
	
    /**
     * Simple dataset
     */
    private void jmiSimpleDatasetActionPerformed() {
    	int selectedRow = jtYAxisValues.getSelectedRow();
    	int col = 0;
    	int key = Integer.parseInt(jtYAxisValues.getValueAt(selectedRow, col).toString());
    	
    	boolean isDrawSelected;
    	if(jtYAxisValues.getValueAt(selectedRow, TO_DRAW).toString() == "true"){
    		isDrawSelected = true;
    	}
    	else {
    		isDrawSelected = false;
    	}
    	
//    	verifyDataset();
    	
    	SimpleDataset sds = simpleDatasetMap.get(key);
    	sds = new SimpleDataset(
    			sds.getTestName(), 
    			sds.getBladeId(), 
    			sds.getBladeIdList(),
    			sds.getEventSelected(),
    			sds.isFieldSelected(),
    			sds.getFieldSelected(),
    			sds.getFilterList(),
    			sds.getLogicalOp(),
    			sds.getTimeStartFilter(),
    			sds.getTimeEndFilter(),
    			sds.getKParam(),
    			sds.getPercentPointsKeptParam(),
    			sds.getMaxTime(),
    			sds.getDatasetType(),
    			sds.getBladeAlias(),
    			isDrawSelected,
    			sds.getDrawTimeStart(),
    			sds.getDrawTimeEnd(),
    			this.globalGUI
    			);
    	JFrame jSimpleFrame = new JFrame();
    	jSimpleFrame = sds;
    	jSimpleFrame.getContentPane().repaint();
    	jSimpleFrame.pack();
    	jSimpleFrame.setVisible(true);
	}
	
	public int getDatasetRowSelected(){
		return jtYAxisValues.getSelectedRow();
	}
	
	public void setDatasetValueAt(String value, int row, int col){
		jtYAxisValues.setValueAt(value, row, col);
	}
	
	private void jtfYAxisLabelKeyPressed(java.awt.event.KeyEvent evt) {
		jtfYAxisLabel.setBackground(DataQuickViewContainer.MODIFIED_AREA_COLOR);
	}

	private void jmiAddToYAxisActionPerformed(java.awt.event.ActionEvent evt) {
		jbAddToYAxisActionPerformed(evt);
	}

	private void jmiRefreshActionPerformed(java.awt.event.ActionEvent evt) {
		jbRefreshActionPerformed(evt);
	}

		  
	
	// ///////////////////////////////
	// 		   COMMENT              //
	// ///////////////////////////////
	
	
	/**
	 * Initialize comment tree : root is the report and his childs are pages and page's childs are dataset
	 * @return DefaultMutableTreeNode
	 */
	private DefaultMutableTreeNode getCommentTree(){
		DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode("Report");
		//pages
		DefaultMutableTreeNode childNode = null;
		//dataset
		DefaultMutableTreeNode childChildNode = null;
	
		Set<Entry<Integer, List<Integer>>> set = pageIdAndDatasetIds.entrySet();
			Iterator<Entry<Integer, List<Integer>>> iterator = set.iterator();
			while (iterator.hasNext()) {
				Map.Entry<java.lang.Integer, java.util.List<java.lang.Integer>> entry = 
					(Map.Entry<java.lang.Integer, java.util.List<java.lang.Integer>>) iterator
						.next();
				int key = entry.getKey();
				List<Integer> inList = entry.getValue();
				childNode = new DefaultMutableTreeNode("Page "+String.valueOf(key));
				Iterator<Integer> it = inList.iterator();
				while (it.hasNext()) {
					Integer datasetId = (Integer) it.next();
					childChildNode = new DefaultMutableTreeNode(
							simpleDatasetMap.get(datasetId).getBladeIdAlias()
							);
					childNode.add(childChildNode);
				}
				rootNode.add(childNode);
			}
//		cleanMapTreeComment();
		return rootNode;
	}
	
	/**
	 * Clean the map : remove suppressed dataset
	 */
	private void cleanMapTreeComment(){
		//get all paths in jTreeComment
		try {
			Set<String> stringList = new HashSet<String>();
			TreeModel treeModel = jTreeComment.getModel();
			Object root = treeModel.getRoot();
			stringList.add("[" + root.toString() + "]");
			for (int i = 0; i < treeModel.getChildCount(root); i++) {
				Object page = (Object) treeModel.getChild(root, i);
				stringList.add("[" + root.toString() + ", " + page.toString() + "]");
					for (int j = 0; j < treeModel.getChildCount(page); j++) {
						Object dataset =  (Object) treeModel.getChild(page, j);
						stringList.add("[" + root.toString() + ", " + page.toString() + "," + dataset.toString() + "]");
					}
			}
			
//			System.out.println("clean __________________");
//			for (Iterator iterator = stringList.iterator(); iterator.hasNext();) {
//				String string = (String) iterator.next();
//				System.out.println(string);
//			}
//			System.out.println(":::::::::::::::::::::::::");
			
			//for each value in mapTreeComment 
			//if not in the path of jtreecomment remove from the map
			Set<Entry<String, String>> set = mapTreeComment.entrySet();
			Iterator<Entry<String, String>> iterator = set.iterator();
			while (iterator.hasNext()) {
				Map.Entry<java.lang.String, String> entry = iterator
						.next();
				String key = entry.getKey();
				if( stringList.contains(key.toString()) == false){
					System.out.println("removed " + key);
					mapTreeComment.remove(key);
				}
			}
		} catch (Exception e) {
			
		}
		
	}
	
	/**
	 * Init the comment tree and update it
	 */
	public void initCommentTree(){
		jTreeComment = new JTree(getCommentTree());
		jTreeComment.getSelectionModel().setSelectionMode(
				//single selection possible
				TreeSelectionModel.SINGLE_TREE_SELECTION);
    	jScrollPaneComment.setViewportView(jTreeComment);
    	jTreeComment.addMouseListener(new MouseListener (){
			public void mouseClicked(MouseEvent e) {

				javax.swing.JTree jtTmp = (JTree) e.getSource();
				if(jtTmp.isSelectionEmpty() == false){
					TreePath[] tpTmp = jtTmp.getSelectionPaths();
					String tpString = "";
					for (int i = 0; i < tpTmp.length; i++) {
						tpString += tpTmp[i].toString();
					}
					String sTmp = mapTreeComment.get(tpString);
					if(sTmp != null){
						jtaNewComment.setText(sTmp);
					}
					else{
						jtaNewComment.setText("");
						jtaNewComment.setText( "(empty comment)");
					}
				}
			}

			public void mouseEntered(MouseEvent e) {
			}

			public void mouseExited(MouseEvent e) {
				String name = "Mouse Event";
				// TPS 2010 03 01 before path switching, must save current text  -->
//				jbSaveCommentPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "save"));
				jbSaveCommentPerformed(new ActionEvent(this, currentIdDataset, name ));
				// TPS 2010 03 01 before path switching, must save current text  <--
			}

			public void mousePressed(MouseEvent e) {
			}

			public void mouseReleased(MouseEvent e) {
			}
    		
    	});
    	jpComment.updateUI();
	}
	

	// ///////////////////////////////
	// Page Management              //
	// ///////////////////////////////

	/**
	 * Create a new page for this dataset
	 * @param chartId
	 * @return page key
	 */
	public int updatePageExistence(int chartId) {
		// if no page exists create new one
		if(pageExistence.size() == 0){
			pageExistence.put(0, true);
			//create new page in tabbed pane
			datasetIdAndPage.put(chartId, 0);
			//add data to map pageIdsAndDatasetIds 
			List<Integer> inList = pageIdAndDatasetIds.get(0);
			if(inList != null){
				inList.add(chartId);
			}
			else{
				inList = new ArrayList<Integer>();
			}
			pageIdAndDatasetIds.put(0, inList);
			//create a new tab
			addPageToTab(0);
			updatePageComboBox(0);
			enableAllPageOptions();
			createNewPageOptions(0, chartId);
			return 0;
		}
		// try to find a non existent page
		else{
			int key = -1;
			Set<Entry<Integer, Boolean>> set = pageExistence.entrySet();
			Iterator<Entry<Integer, Boolean>> iterator = set.iterator();
			while(iterator.hasNext()){
				Map.Entry<Integer,Boolean> me = iterator.next();
				key = me.getKey();
				//if value is false means the page doesn't exist
				if(!me.getValue()){
					//create existence for this page : override
					pageExistence.put(key, true);
					//associate the dataset with this page
					datasetIdAndPage.put(chartId, key);
					//add data to map pageIdsAndDatasetIds 
					List<Integer> inList = pageIdAndDatasetIds.get(key);
					if(inList != null){
						inList.add(chartId);
					}
					else{
						inList = new ArrayList<Integer>();
						inList.add(chartId);
					}
					pageIdAndDatasetIds.put(key, inList);
					//create new page in tabbed pane
					addPageToTab(key);
					updatePageComboBox(key);
					enableAllPageOptions();
					createNewPageOptions(key, chartId);
					return key;
				}
			}
			key = key + 1;
			//create new page
			pageExistence.put(key, true);
			//associate the dataset with this page
			datasetIdAndPage.put(chartId, key);
			//add data to map pageIdsAndDatasetIds 
			List<Integer> inList = pageIdAndDatasetIds.get(key);
			if(inList != null){
				inList.add(chartId);
			}
			else{
				inList = new ArrayList<Integer>();
				inList.add(chartId);
			}
			pageIdAndDatasetIds.put(key, inList);
			//create new page in tabbed pane
			addPageToTab(key);
			updatePageComboBox(key);
			enableAllPageOptions();
			createNewPageOptions(key, chartId);
			return key;
		}
	}
	
	
	public void addNewPage(SimpleDataset simpleDataset, int currentId) {
		datasetIdAndPage.put(currentId, currentId);
	}
	
	
	/**
	 * Update table combo box(associated with pages)
	 * @param key
	 */
	private void updatePageComboBox(int key) {
		TableColumnModel myColumnModel = jtYAxisValues.getColumnModel();
		TableColumn myTableColumn;
		myTableColumn = myColumnModel.getColumn(PAGE);
		pageComboBox = new JComboBox();
		Set<Entry<Integer, Boolean>> set = pageExistence.entrySet();
		Iterator<Entry<Integer, Boolean>> iterator = set.iterator();
		while(iterator.hasNext()){
			Map.Entry<Integer,Boolean> me = iterator.next();
			if(me.getValue() == true){
				pageComboBox.addItem(me.getKey());
			}
		}
		pageComboBox.addItem("New Page");
		pageComboBox.setSelectedItem(key);
		 
		ActionListener myActionListener = new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
		        JComboBox jcBox = (JComboBox) e.getSource();
		        int selectedPage;
		        for (int i = 0; i < jtYAxisValues.getRowCount(); i++) {
					
					//new dataset created
					if(jtYAxisValues.getValueAt(i, PAGE).toString().equalsIgnoreCase("New Page")){
						//get the first free page
						selectedPage = getFirstFreePage();
					
						//insert the page in page existence
						pageExistence.put(selectedPage, true);
						//update the combobox
						jcBox.removeAllItems();
						//page existence is sorted already
						Set<Entry<Integer, Boolean>> set = pageExistence.entrySet();
						Iterator<Entry<Integer, Boolean>> iterator = set.iterator();
						while(iterator.hasNext()){
							Map.Entry<Integer,Boolean> me = iterator.next();
							if(me.getValue() == true){
								jcBox.addItem(me.getKey().toString());
							}
						}
						jcBox.addItem("New Page");
						
						//display properties
						jtYAxisValues.setValueAt(selectedPage, i, PAGE);
						jcBox.setSelectedItem(String.valueOf(selectedPage));
						
						//insert value in the dataset id an page id map
						datasetIdAndPage.put(
								Integer.parseInt(jtYAxisValues.getValueAt(i, CHART_ID).toString()), 
								selectedPage);
						//add data to map pageIdsAndDatasetIds 
						List<Integer> inList = pageIdAndDatasetIds.get(selectedPage);
						if(inList != null){
							inList.add(Integer.parseInt(jtYAxisValues.getValueAt(i, CHART_ID).toString()));
						}
						else{
							inList = new ArrayList<Integer>();
							inList.add(Integer.parseInt(jtYAxisValues.getValueAt(i, CHART_ID).toString()));
						}
						pageIdAndDatasetIds.put(selectedPage, inList);
						//compare for page existence 
						modifyPageExistence();
						//delete obsolete page
						deleteNonExistencePage();
						//create new page
						createNewExistencePage();
						//draw if is selected
						if(jtYAxisValues.getValueAt(i, TO_DRAW).toString().equalsIgnoreCase("true")){
							jbDrawGraphPerformed();
						}
					}
					//put in an already existent page
					else{
							datasetIdAndPage.put(
									Integer.parseInt(jtYAxisValues.getValueAt(i, CHART_ID).toString()), 
									Integer.parseInt(jtYAxisValues.getValueAt(i, PAGE).toString()));
							pageExistence.put(Integer.parseInt(jtYAxisValues.getValueAt(i, PAGE).toString()), true);
							//add data to map pageIdsAndDatasetIds 
							List<Integer> inList = pageIdAndDatasetIds.get(
									Integer.parseInt(jtYAxisValues.getValueAt(i, PAGE).toString()));
							if(inList != null){
								inList.add(Integer.parseInt(jtYAxisValues.getValueAt(i, CHART_ID).toString()));
							}
							else{
								inList = new ArrayList<Integer>();
								inList.add(Integer.parseInt(jtYAxisValues.getValueAt(i, CHART_ID).toString()));
							}
							pageIdAndDatasetIds.put(
									Integer.parseInt(jtYAxisValues.getValueAt(i, PAGE).toString()),
									inList);
							//compare for page existence 
							modifyPageExistence();
							//delete obsolete page
							deleteNonExistencePage();
							//create new page
							createNewExistencePage();
							//draw if selected
							if(jtYAxisValues.getValueAt(i, TO_DRAW).toString().equalsIgnoreCase("true")){
								jbDrawGraphPerformed();
							}
					}
				}
		        //entry on page 
		        Set<Entry<Integer, List<Integer>>> set = pageIdAndDatasetIds.entrySet();
				Iterator<Entry<Integer, List<Integer>>> iterator = set.iterator();
				while(iterator.hasNext()){
					Map.Entry<Integer,List<Integer>> me = iterator.next();
					int pageId = me.getKey();
					List<Integer> inListTable = me.getValue();
					//get the correct chart data
					ChartData cd = chartDataList.get(pageId);
					if(cd != null){
						List<Integer> inListChart = cd.getAllDatasetId();
						if(inListChart != null){
							Iterator<Integer> inListIterator = inListChart.iterator();
							while (inListIterator.hasNext()) {
								Integer currentInt = inListIterator.next();
								//the dataset is in the chart but not in the page table, remove it
								if(inListTable.contains((int)currentInt) == false){
									cd.removeDatasetFromTab(currentInt);
								}
								//nothing to do 
								else{
									;
								}
							}
						}
					}
					else{
						//nothing to do since nothing in the chart
					}	
				}
				//if the data is in the map but not in the table : delete it
				Set<Entry<Integer, ChartData>> set1 = chartDataList.entrySet();
				Iterator<Entry<Integer, ChartData>> iterator1 = set1.iterator();
				while (iterator1.hasNext()) {
					Map.Entry<Integer, ChartData> entry = iterator1.next();
					int intPageChart = entry.getKey();
					if(pageIdAndDatasetIds.containsKey(intPageChart) == false){
						chartDataList.remove(intPageChart);
					}
				}
				udpatepageIdAndDatasetIds();
				// update comment tree
				initCommentTree();
				cleanMapTreeComment();
		    }
		}; 
					
		pageComboBox.addActionListener(myActionListener);
		
		myTableColumn.setCellEditor(new DefaultCellEditor(pageComboBox));

		}

	
	/**
	 * Clean the data in map pageIdAndDatasetIds
	 */
	public void udpatepageIdAndDatasetIds(){
		pageIdAndDatasetIds.clear();
		for (int i = 0; i < jtYAxisValues.getRowCount(); i++) {
			int pageId = Integer.parseInt(jtYAxisValues.getValueAt(i, PAGE).toString());
			List<Integer> inList = pageIdAndDatasetIds.get(pageId);
			if(inList == null){
				inList = new ArrayList<Integer>();
			}
			int chartId = Integer.parseInt(jtYAxisValues.getValueAt(i, CHART_ID).toString());
			inList.add(chartId);
			pageIdAndDatasetIds.put(pageId, inList);
		}	
	}
	
	/**
	 * Get the first free page according to natural order 
	 * @return Integer 
	 */
	private int getFirstFreePage(){
		Set<Entry<Integer, Boolean>> set = pageExistence.entrySet();
		Iterator<Entry<Integer, Boolean>> iterator = set.iterator();
		while(iterator.hasNext()){
			Map.Entry<Integer,Boolean> me = iterator.next();
			int pageId = me.getKey();
			boolean boo = me.getValue();
			if(boo == false){
				return pageId;
			}
		}
		return pageExistence.size();
	}
	
	private void createNewExistencePage(){	
		Set<Entry<Integer, Integer>> setDP = datasetIdAndPage.entrySet();
		Iterator<Entry<Integer, Integer>> iteratorDP = setDP.iterator();		
		while(iteratorDP.hasNext()){
			Map.Entry<Integer,Integer> me = iteratorDP.next();
			int value = me.getValue();
			//check into pageExistence : if value in pageExistence false : create new tab
			boolean booValue = pageExistence.get(value);
			if(booValue == true){
				//check if the page exist already created 
				Component[] compList = jtPChart.getComponents();
				Set<String> compSet = new HashSet<String>();
				for (int i = 0; i < compList.length; i++) {
					compSet.add(compList[i].getName());
				}
				if(!compSet.contains(String.valueOf(value))){
					addPageToTab(value);
				}
			}
		}
	}
	
	private void modifyPageExistence(){
		//add all the page in dataset in a set
		Set<Integer> datasetPage = new HashSet<Integer>();
		Set<Entry<Integer, Integer>> setDP = datasetIdAndPage.entrySet();
		Iterator<Entry<Integer, Integer>> iteratorDP = setDP.iterator();		
		while(iteratorDP.hasNext()){
			Map.Entry<Integer,Integer> me = iteratorDP.next();
			int value = me.getValue();
			datasetPage.add(value);
		}
		
		
		Set<Entry<Integer, Boolean>> set = pageExistence.entrySet();
		Iterator<Entry<Integer, Boolean>> iterator = set.iterator();	
		while(iterator.hasNext()){
			Map.Entry<Integer,Boolean> me = iterator.next();
			int pageId = me.getKey();
			boolean boo = me.getValue();
			if(boo){
				if(!datasetPage.contains(pageId)){
					//remove pageId from the dataset
					pageExistence.put(pageId, false);
					//and the tab
					addPageToTab(pageId);
				}
			}
		}		
	}
	
	/**
	 * Check the table and delete the tabs that are not use anymore
	 */
	private void deleteNonExistencePage(){
		//test if each existent page are in dataset table
		Set<Entry<Integer, Boolean>> set = pageExistence.entrySet();
		Iterator<Entry<Integer, Boolean>> iterator = set.iterator();
		while(iterator.hasNext()){
			Map.Entry<Integer,Boolean> me = iterator.next();
			int page = me.getKey();
			boolean boo = me.getValue();
			Set<Integer> v = new HashSet<Integer>();
			if(!boo){
				Set<Entry<Integer, Integer>> set2 = datasetIdAndPage.entrySet();
				Iterator<Entry<Integer, Integer>> iterator2 = set2.iterator();
				while(iterator2.hasNext()){
					Map.Entry<Integer,Integer> me2 = iterator2.next();
					int int2 = me2.getValue();
					v.add(int2);
				}
				//remove this page
				if(!(v.contains(page))){
					Component[] comList = jtPChart.getComponents();
					for (int i = 0; i < comList.length; i++) {
						if(comList[i].getName().equalsIgnoreCase(String.valueOf(page))){
							jtPChart.remove(comList[i]);
							chartDataList.remove(page);
						}
					}
				}
			}		
		}	
	}

	
	private void addPageToTab(int currentId) {
		    jpp = new JPanel();
		    jsppp = new JScrollPane(jpp);
		    //set a specific name (currentId)
		    jsppp.setName(String.valueOf(currentId));
		    jpp.setName(String.valueOf(currentId));
		    //add the new page
			jtPChart.add(jsppp, "Page "+ String.valueOf(currentId));
			//set this page selected
			Component[] comList = jtPChart.getComponents();
			for (int i = 0; i < comList.length; i++) {
				if(comList[i].getName().equalsIgnoreCase(String.valueOf(currentId))){
					jtPChart.setSelectedComponent(comList[i]);
				}
			}
			initCommentTree();
	}

	
	// ///////////////////////
	// JFreeChart utilities //
	// ///////////////////////
	
	public void addRowToTable(Vector<String> row){
		chartTableModel.addRow(row);
		udpatepageIdAndDatasetIds();
	}
	
	public void setValueAt(String field, int row, int col){
		chartTableModel.setValueAt(field, row, col);
	}
	
	public String getValueAt(int row, int col){
		return jtYAxisValues.getValueAt(row, col).toString();
	}
	
	public int getSelectedRow(){
		return jtYAxisValues.getSelectedRow();
	}
	
    public void addToDataQuickViewContainer(SimpleDataset simpleDataset){
    	this.simpleDataset = simpleDataset;
    }
    
	
    ////////////////////////////////////
    //	Util 						  //
    ////////////////////////////////////
    /**
     * 26 juin
     * Get the bladeEventArray from a bladeEventList
     * @param bladeEventList
     * @return bladeEventArray an array that contains the blade events
     */
    public BladeEvent[] toBladeEventArray(List<BladeEvent> bladeEventList){
    	int size = bladeEventList.size();
    	BladeEvent[] bladeEventArray = new BladeEvent[size];
    	Iterator<BladeEvent> iterator = bladeEventList.iterator();
    	int count = 0;
    	while (iterator.hasNext()) {
			BladeEvent bladeEvent = iterator.next();
			bladeEventArray[count] = bladeEvent;
			count++;
		}
    	return bladeEventArray;
    }


	public void setRowSelected(int currentId) {
		int rowSelected = -1;
		for (int i = 0; i < jtYAxisValues.getRowCount(); i++) {
			if( Integer.parseInt(jtYAxisValues.getValueAt(i, CHART_ID).toString()) ==  currentId){
				rowSelected = i;
			}
		}
		jtYAxisValues.changeSelection(rowSelected, CHART_ID, false, false);
	}
    

	//////////////////////////////////
	// STATISTIC TAB				//
	//////////////////////////////////

	/** 
	 * Calculate statistic for this dataset
	 */
    //used july 29
    public void addNewStatistic(SimpleDataset simpleDataset, int id, String field, String bladeIdAlias) {
    	String testName = simpleDataset.getTestName();
    	String bladeId = simpleDataset.getBladeId();
    	List<String> bladeIdList = simpleDataset.getBladeIdList();
    	String eventType = simpleDataset.getEventSelected();
    	Map<String, Map<String, String>> filter = simpleDataset.getFilter();
    	List<FieldFilterAndOp> ffo = simpleDataset.getFilterList();
    	String linkingOp = simpleDataset.getLogicalOpString();
    	List<BladeEvent> bladeEventList = new ArrayList<BladeEvent>();
    	StatisticDataImpl sdi = null;
    	List<StatisticDataImpl> sdiList = new ArrayList<StatisticDataImpl>();
    	List<List<BladeEvent>> bladeEventListList = new ArrayList<List<BladeEvent>>();
    	int start = (int) simpleDataset.getTimeStartFilter();
    	int end = (int) simpleDataset.getTimeEndFilter();
    	List<BladeEvent> bladeEventListTmp = new ArrayList<BladeEvent>();
    	List<BladeEvent> bladeEventListTmp2 = new ArrayList<BladeEvent>();
    	Comparator<BladeEvent> SORT_BY_TIME = new DateComp();
		TreeSet<BladeEvent> bladeTreeSetOr = new TreeSet<BladeEvent>(SORT_BY_TIME);
    	//aggregate
    	if(simpleDataset.getDatasetType() == 2){
    		Iterator<String> iter = bladeIdList.iterator();
    		while (iter.hasNext()) {
    			String string = iter.next();
    			try {
    				bladeEventList = this.globalGUI.getBladeOneEventValues(testName, string, eventType);
    				bladeEventList = this.globalGUI.getBladeByTime(bladeEventList, start, end);
    			} catch (ClifException e) {
    				e.printStackTrace();
    			}
    			bladeEventListList.add(bladeEventList);
    		}
    		bladeEventList = this.globalGUI.aggregateBladesOneSameEvent(bladeEventListList);
    		bladeEventList = this.globalGUI.getBladeByTime(bladeEventList, start, end);
    	}

    	//no logical op : means one filter or no filter
    	if(linkingOp == null){
    		//only one filter
    		if(filter.size()==1){
    			Set<Entry<String, Map<String, String>>> set = filter.entrySet();
    			Iterator<Entry<String, Map<String, String>>> iterator = set.iterator();
    			Map<String, String> operatorFilter = new HashMap<String, String>(); 
    			String fieldAlone;
    			String keywordAlone;
    			String operatorAlone;
    			while(iterator.hasNext()){
    				Map.Entry<String,Map<String, String>> me = 
    					iterator.next();
    				fieldAlone = me.getKey();
    				operatorFilter = me.getValue();
    				Set<Entry<String, String>> setF = operatorFilter.entrySet();
    				Iterator<Entry<String, String>> iteratorF = setF.iterator();
    				while (iteratorF.hasNext()) {
    					Map.Entry<String, String> entry = 
    						iteratorF.next();
    					keywordAlone = entry.getKey();
    					operatorAlone = entry.getValue();
    					if( (simpleDataset.getDatasetType() == 0)
    							|| (simpleDataset.getDatasetType() == 2)
    					){
    						if(simpleDataset.getDatasetType() == 0){
    							bladeEventList = this.globalGUI.fieldFilter(testName, bladeId, eventType, fieldAlone, keywordAlone, operatorAlone);
    							bladeEventList = this.globalGUI.getBladeByTime(bladeEventList, start, end);
    							sdi = createStatisticDataImpl(simpleDataset, id, bladeId, field, bladeEventList, "", 0, start, end);
    						}
    						else if(simpleDataset.getDatasetType() == 2){
    							bladeEventList = this.globalGUI.fieldFilter(bladeEventList, fieldAlone, keywordAlone, operatorAlone);
    							bladeEventList = this.globalGUI.getBladeByTime(bladeEventList, start, end);
    							sdi = createStatisticDataImpl(simpleDataset, id, bladeId, field, bladeEventList, bladeIdAlias, 2, start, end);
    						}
    						sdiList.add(sdi);
    						statisticMap.put(id, sdiList);
    						updateStatisticsTab(sdiList);
    					}
    					else if (simpleDataset.getDatasetType() == 1) {
    						Iterator<String> iter = bladeIdList.iterator();
    						while (iter.hasNext()) {
    							String string = iter.next();
    							bladeEventList = this.globalGUI.fieldFilter(testName, string, eventType, fieldAlone, keywordAlone, operatorAlone);
    							bladeEventList = this.globalGUI.getBladeByTime(bladeEventList, start, end);
    							sdi = createStatisticDataImpl(simpleDataset, id, string, field, bladeEventList, bladeIdAlias, 1, start, end);
    							sdiList.add(sdi);
    						}
    						statisticMap.put(id, sdiList);
    						updateStatisticsTab(sdiList);
    					}
    				}
    			}
    			
    		}

    		//no filter : return raw data
    		else {
    			try {
    				if( (simpleDataset.getDatasetType() == 0)
    						|| (simpleDataset.getDatasetType() == 2)
    				){
    					if(simpleDataset.getDatasetType() == 0){
    						bladeEventList = this.globalGUI.getBladeOneEventValues(testName, bladeId, eventType);
    						bladeEventList = this.globalGUI.getBladeByTime(bladeEventList, start, end);
    						sdi = createStatisticDataImpl(simpleDataset, id, bladeId, field, bladeEventList, "", 0, start, end);
    					}
    					else if(simpleDataset.getDatasetType() == 2){
    						//NOTHING TO DO
    						sdi = createStatisticDataImpl(simpleDataset, id, bladeId, field, bladeEventList, bladeIdAlias, 2, start, end);
    					}
    					
    					sdiList.add(sdi);
    					statisticMap.put(id, sdiList);
    					updateStatisticsTab(sdiList);
    				}
    				else if (simpleDataset.getDatasetType() == 1) {
    					Iterator<String> iter = bladeIdList.iterator();
    					while (iter.hasNext()) {
    						String string = iter.next();
    						bladeEventList = this.globalGUI.getBladeOneEventValues(testName, string, eventType);
    						bladeEventList = this.globalGUI.getBladeByTime(bladeEventList, start, end);
//    						createStatisticDataImpl(simpleDataset, id, string, field, bladeEventList, bladeIdAlias);
    						sdi = createStatisticDataImpl(simpleDataset, id, string, field, bladeEventList, bladeIdAlias, 1, start, end);
    						sdiList.add(sdi);
    					}
    					statisticMap.put(id, sdiList);
    					updateStatisticsTab(sdiList);
    				}
    				else{
    					if(DEBUG){
    						System.out.println("ERROR IN DATASET TYPE line 4095");
    					}
    				}

    			} catch (ClifException e) {
    				e.printStackTrace();
    			}
    		}
    	}
    	//many filters 
    	else{
    		if( (simpleDataset.getDatasetType() == 0)
    				|| (simpleDataset.getDatasetType() == 2)
    		){
    			if(simpleDataset.getDatasetType() == 0){
    				try {
    					bladeEventList = this.globalGUI.getBladeOneEventValues(testName, bladeId, eventType);
    				} catch (Exception e) {
    				
    				}
    			}

    			for (Iterator<FieldFilterAndOp> iterator = ffo.iterator(); iterator
    			.hasNext();) {
    				FieldFilterAndOp tmp = iterator
    				.next();
    				if(simpleDataset.getLogicalOpString().equalsIgnoreCase("AND")){
    					bladeEventList = this.globalGUI.fieldFilter(bladeEventList, tmp.getField(), tmp.getFilter(), tmp.getOperator());
    				}
    				//OR
    				else{
    					bladeEventListTmp = this.globalGUI.fieldFilter(bladeEventList, tmp.getField(), tmp.getFilter(), tmp.getOperator());
    					bladeTreeSetOr.addAll(bladeEventListTmp);
    				}
    				
    			}
    			if(simpleDataset.getLogicalOpString().equalsIgnoreCase("OR")){
    				Iterator<BladeEvent> iterator2 = bladeTreeSetOr.iterator();
    	        	while (iterator2.hasNext()) {
    					BladeEvent bladeEvent = iterator2.next();
    					bladeEventListTmp2.add(bladeEvent);
    				}
    				bladeEventList = bladeEventListTmp2;
    			}
    			bladeEventList = this.globalGUI.getBladeByTime(bladeEventList, start, end);
    			if(simpleDataset.getDatasetType() == 0){
    				sdi = createStatisticDataImpl(simpleDataset, id, bladeId, field, bladeEventList, "", 0, start, end);
    			}
    			else{
    				sdi = createStatisticDataImpl(simpleDataset, id, bladeId, field, bladeEventList, bladeIdAlias, 2, start, end);
    			}
    			sdiList.add(sdi);
    			statisticMap.put(id, sdiList);
    			updateStatisticsTab(sdiList);
    		}
    		else if (simpleDataset.getDatasetType() == 1) {
    			Iterator<String> iter = bladeIdList.iterator();
    			while (iter.hasNext()) {
    				//string corresponds to bladeId
    				String string = iter.next();
    				
    				try {
    					bladeEventList = this.globalGUI.getBladeOneEventValues(testName, string, eventType);
    				} catch (Exception e) {
    				
    				}
    				
    				//for each blade : filter with all the filters 
    				for (Iterator<FieldFilterAndOp> iterator = ffo.iterator(); iterator
        			.hasNext();) {
        				FieldFilterAndOp tmp = iterator
        				.next();
        				if(simpleDataset.getLogicalOpString().equalsIgnoreCase("AND")){
        					bladeEventList = this.globalGUI.fieldFilter(bladeEventList, tmp.getField(), tmp.getFilter(), tmp.getOperator());
        				}
        				//OR
        				else{
        					bladeEventListTmp = this.globalGUI.fieldFilter(bladeEventList, tmp.getField(), tmp.getFilter(), tmp.getOperator());
        					bladeTreeSetOr.addAll(bladeEventListTmp);
        				}
        				
        			}
        			if(simpleDataset.getLogicalOpString().equalsIgnoreCase("OR")){
        				Iterator<BladeEvent> iterator2 = bladeTreeSetOr.iterator();
        	        	while (iterator2.hasNext()) {
        					BladeEvent bladeEvent = iterator2.next();
        					bladeEventListTmp2.add(bladeEvent);
        				}
        				bladeEventList = bladeEventListTmp2;
        			}
    				bladeEventList = this.globalGUI.getBladeByTime(bladeEventList, start, end);
    				sdi = createStatisticDataImpl(simpleDataset, id, string, field, bladeEventList, bladeIdAlias, 1, start, end);
    				sdiList.add(sdi);
    			}
    			statisticMap.put(id, sdiList);
    			updateStatisticsTab(sdiList);
    		}
    		else{
    			if(DEBUG){
    				System.out.println("ERROR IN DATASET TYPE line 4119");
    			}
    		}
    	}
    }

	/////////////////////////////////
    // Class DateComp //
    /////////////////////////////////
    private class DateComp implements Comparator<BladeEvent> {
    	//sort by time
    	public int compare(BladeEvent bladeEvent1, BladeEvent bladeEvent2){
    		if(bladeEvent1.getDate()<bladeEvent2.getDate()){
    			return -1;
    		}
    		else if(bladeEvent1.getDate()>bladeEvent2.getDate()){
    			return 1;
    		}
    		else{
    			return 0;
    		}
    	}
    }
    
    
    private class CommentComp implements Comparator<String> {
    	//sort by time
    	public int compare(String string1, String string2){
    	    return -string1.compareToIgnoreCase(string2);
    	}
    }
    
	/**
	 * Create a statisticDataImpl
	 * @param simpleDataset
	 * @param id
	 * @param bladeId
	 * @param field
	 * @param bladeEventList
	 * @return StatisticDataImpl
	 */
	private StatisticDataImpl createStatisticDataImpl(
			SimpleDataset simpleDataset, 
			int id, 
			String bladeId,  
			String field, 
			List<BladeEvent> bladeEventList,
			String bladeIdAlias,
			int datasetType,
			int start,
			int end){
		StatOnLongs stolDate = new StatOnLongs();
		StatOnLongs stol = new StatOnLongs();
		Iterator<BladeEvent> iterator = bladeEventList.iterator();
		while (iterator.hasNext()) {
			BladeEvent bladeEvent = iterator.next();
			Long date = bladeEvent.getDate();
			stolDate.addLong(date);
			try {
				Long tmp = Long.parseLong(bladeEvent.getFieldValue(field).toString());
				stol.addLong(tmp);
			} catch (Exception e) {
				if(DEBUG){
//					System.out.println("WARNING NOT A LONG");	
				}
			}
		}
		StatisticDataImpl sdi = new StatisticDataImpl(
				simpleDataset.getTestName(), 
				bladeId,
				simpleDataset.getEventSelected(),
				simpleDataset.getFieldSelected(),
				stolDate,
				stol,
				simpleDataset.getFilterList(),
				simpleDataset.getLogicalOpString(),
				bladeIdAlias,
				datasetType,
				start,
				end);
		return sdi;
	}
   
	/**
	 * Update the statistic view in the main panel
	 * @param simpleDataset
	 * @param sdi
	 */
	private void updateStatisticsTab(List<StatisticDataImpl> sdiList) {
//		if(sdiList.get(0).getField() != null){
//			jtPsubChart.setSelectedComponent(jpanelsub2);
//		}
		jtaStat.setText("Statistics Tab default text");
		Iterator<StatisticDataImpl> it = sdiList.iterator();
		while (it.hasNext()) {
			StatisticDataImpl sdi = it.next();

			jtaStat.setText(jtaStat.getText()
					+ "  -------------------------\n");
			
			//simple dataset and multiple dataset 
			if(sdi.getDatasetType() == 0 || sdi.getDatasetType() == 1){
				jtaStat.setText(jtaStat.getText()
						+ sdi.getTestName() + ","
						+ sdi.getBladeId() + ","
						+ sdi.getEventType()
				);
			}
			//Aggregate dataset
			else if(sdi.getDatasetType() == 2){
				jtaStat.setText(jtaStat.getText()
						+ sdi.getTestName() + ","
						+ sdi.getBladeIdAlias() + ","
						+ sdi.getEventType()
				);
			}
			if(sdi.getField() != null && sdi.getField() != ""){
				jtaStat.setText(jtaStat.getText()
						+ ","
						+ sdi.getField()
				);
				
				// Time Interval
				
				//
				if(sdi.getStolSize() > 0){
					if(sdi.getFfo().size() > 0){
						jtaStat.setText(jtaStat.getText()+ "\n");
						jtaStat.setText(jtaStat.getText()+
								"\nFilters used :\n"
								);
						List<FieldFilterAndOp> ffo = sdi.getFfo();
						
						
						Iterator<FieldFilterAndOp> ffoIt = ffo.iterator();
						while (ffoIt.hasNext()) {
							FieldFilterAndOp tmp = ffoIt
									.next();
							jtaStat.setText(jtaStat.getText()
									+ 
									"Operator : "
									+ tmp.getField() + " " 
									+ tmp.getOperator() + " "
									+ tmp.getFilter()
									+ " \n"
							);
						}

						if(ffo.size() > 1){
							jtaStat.setText(jtaStat.getText() 
									+ "Logical Operator used :"
									+ sdi.getLogicalOp()
									);
							jtaStat.setText(jtaStat.getText() 
									+ "\n"
									);
						}
						
					}
					else{
						jtaStat.setText(jtaStat.getText()		
								+
								"\nNo filter used\n"
								);
					}
					
					jtaStat.setText(jtaStat.getText()
							+"\n"
							+ "Time Interval : " 
							+ sdi.gettimeIntervalStart() + " ms to "
							+ sdi.gettimeIntervalEnd() + " ms \n"
							+"\n"
							//request per second
							+ "Throughput : " +sdi.getThroughput() + "\n"
							+ "Minimum :"+sdi.getMinimum() + "\n"
							+ "Maximum :"+sdi.getMaximum() + "\n"
							+ "Mean :"+sdi.getMean() + "\n"
							+ "Median :"+sdi.getMedian() + "\n"
							+ "Standard deviation :"+sdi.getStdDvt() + "\n"
							+ "Number of values :"+ (int) sdi.getNumberOfValues() + "\n");
				}
				else{
					jtaStat.setText(jtaStat.getText()
							+ "\n"
							+ "Non numeric field chosen for the dataset or no result found\n");
				}
				
//				jtPsubChart.setSelectedComponent(jpanelsub2);
			}
			else{
				jtaStat.setText(jtaStat.getText()
						+"\n"
						+ "Throughput : " +sdi.getThroughput() + "\n"
						+ "Number of values :"+ (int) sdi.getNumberOfValues() + "\n");
				jtPsubChart.setSelectedComponent(jpanelsub2);
			}
			jtaStat.setText(jtaStat.getText()
					+ "\n"
					+ "=======================================\n\n");
		}
	}

	public String getNewStatistics() {
		return jtaStat.getText();
	}
	
	/**
	 * Update dataset options according to this dataset 
	 */
	public void updateDatasetOptions() {
		try {
			int row = jtYAxisValues.getSelectedRow();
			int key = Integer.parseInt(jtYAxisValues.getValueAt(row, CHART_ID).toString());
			int page =  Integer.parseInt(jtYAxisValues.getValueAt(row, PAGE).toString());
			// get the dataset associated 
			SimpleDataset sd = simpleDatasetMap.get(key);
			// TO MODIFY
			//if field selected enable the panel for the graph options
			if(sd.isFieldSelected()){
				jpDraw1.setVisible(true);
				jpDraw2.setVisible(true);
				jpDrawButtons.setVisible(true);
				jpDrawConfig.setVisible(true);
			}
			else {
				jpDraw1.setVisible(false);
				jpDraw2.setVisible(false);
				jpDrawButtons.setVisible(false);
				jpDrawConfig.setVisible(false);
			}
			enableAllPageOptions();

			if(sd.isFieldSelected()){
				// we want to draw the chart
				if(sd.isDrawSelected()){
					// retrieve the page options matching this page
					//  if it's a new page created, save it to time chart type
					PageOptions po = pageOptionsMap.get(page);

					// analyze the page associated
					if(po.getGraphType().equalsIgnoreCase("NoChart")){
						// nothing to do
						jcbPageNoChart.setSelected(true);
					}
					else if(po.getGraphType().equalsIgnoreCase("TimeChart")){
						// time chart chart
						jcbPageTimeChart.setSelected(true);
						jcbRaw.setSelected(true);
						enableOnlyTimeChart();
						jbDrawGraphPerformed();
					}
					else if(po.getGraphType().equalsIgnoreCase("FrequencyChart")){
						// frequency chart
						// TODO
					}
					else if(po.getGraphType().equalsIgnoreCase("QuantilChart")){
						// quantil charts
						// TODO
					}
				}
				else {
					PageOptions po = pageOptionsMap.get(page);
					if(po != null){
						updatePageOptions(po);
					}
				}
			}			

		} 
		catch (Exception ex) {
		}
	}
	
	private void updatePageOptions(PageOptions po){
		int pageOptionsId = po.getPageOptionsId();
		jlPageSelected.setText("Page Selected :" + pageOptionsId);
		
		String chartType = po.getGraphType();
		if(chartType.equalsIgnoreCase("NoChart")){
			jcbPageNoChart.setSelected(true);
		}
		else if(chartType.equalsIgnoreCase("TimeChart")){
			jcbPageTimeChart.setSelected(true);
		}
		else if(chartType.equalsIgnoreCase("HistogramChart")){
			jcbPageHistogramChart.setSelected(true);
		}
		else if(chartType.equalsIgnoreCase("QuantilChart")){
			jcbPageQuantil.setSelected(true);
		}
	}
	
	// PAGE OPTIONS ///////////////////////////::
	
	/**
	 * Create a new page options
	 * @param pageId
	 * @param datasetId
	 * @return PageOptions
	 */
	private void createNewPageOptions(int pageId, int datasetId){
		PageOptions po = new PageOptions();
		po.setPageOptionsId(pageId);
		jlPageSelected.setText("Page selected : "  + pageId);
		
		SimpleDataset sd = simpleDatasetMap.get(datasetId);
		int drawTimeStart = sd.getDrawTimeStart();
		int drawTimeEnd = sd.getDrawTimeEnd();
		
		if(sd.isDrawSelected() == false){
			po.setGraphType("NoChart");
			jcbPageNoChart.setSelected(true);
			disablePageOptionsTimeConf();
			disableAllPageOptions();
		}
		if(sd.isDrawSelected() == true){
			po.setGraphType("TimeChart");
			jcbPageTimeChart.setSelected(true);
			enablePageOptionsTimeConf();
			jtfPageTimeStart.setText(String.valueOf(drawTimeStart));
			jtfPageTimeStart.setEnabled(false);
			jtfPageTimeEnd.setText(String.valueOf(drawTimeEnd));
			jtfPageTimeEnd.setEnabled(false);
//			jsPageTime.setMinimum(drawTimeStart.intValue());
//			jsPageTime.setMaximum(drawTimeEnd.intValue());
			
			jsPageTime.setMajorTickSpacing(100000);
			jsPageTime.setMinorTickSpacing(50000);
			jsPageTime.setPaintTicks(true);
			enableOnlyTimeChart();
//			jsPageTime.addChangeListener(new ChangeListener() {
//				  public void stateChanged(ChangeEvent e) {
//				     JSlider source = (JSlider)e.getSource();  // get the slider
//				     int value;
//				     if (source.getValueIsAdjusting()) {
//				    	value = source.getValue();  // get slider value
////				        picture.setSomething(val);  //  picture must repaint itself
//				    	jtfPageTimeStart.setText(String.valueOf(value));
//				     }
//				  }
//				});
		}
		
		po.setTimeStart(drawTimeStart);
		po.setTimeEnd(drawTimeEnd);
		
		pageOptionsMap.put(pageId, po);
	}
	
	/**
	 * Disable the time start, time end and the slider 
	 */
	private void disablePageOptionsTimeConf(){
		jlPageTimeStart.setForeground(Color.GRAY);
		jlPageTimeEnd.setForeground(Color.GRAY);
		jtfPageTimeStart.setEnabled(false);
		jtfPageTimeEnd.setEnabled(false);
		jsPageTime.setEnabled(false);
	}
	
	/**
	 * Disable page options
	 */
	private void disableAllPageOptions(){
//		jbgPageChartType.clearSelection();
		jlPageSelected.setForeground(Color.GRAY);
		jcbPageNoChart.setEnabled(false);
		jcbPageTimeChart.setEnabled(false);
		jcbPageHistogramChart.setEnabled(false);
		jcbPageQuantil.setEnabled(false);
		jlPageTimeStart.setForeground(Color.GRAY);
		jlPageTimeEnd.setForeground(Color.GRAY);
		jtfPageTimeStart.setEnabled(false);
		jtfPageTimeEnd.setEnabled(false);
		jsPageTime.setEnabled(false);
	}
	
	/**
	 * Enable page options
	 */
	private void enableAllPageOptions(){
//		jbgPageChartType.clearSelection();
		jlPageSelected.setForeground(Color.BLACK);
		jcbPageNoChart.setEnabled(true);
		jcbPageTimeChart.setEnabled(true);
		jcbPageHistogramChart.setEnabled(true);
		jcbPageQuantil.setEnabled(true);
		jlPageTimeStart.setForeground(Color.BLACK);
		jlPageTimeEnd.setForeground(Color.BLACK);
		jtfPageTimeStart.setEnabled(true);
		jtfPageTimeEnd.setEnabled(true);
		jsPageTime.setEnabled(true);

		jcbPageHistogramChart.setEnabled(false);	// TPS Not Implemented Yet
		jcbPageHistogramChart.setForeground(UNIMPLEMENTED_COLOR);

		jcbPageQuantil.setEnabled(false);			// TPS Not Implemented Yet
		jcbPageQuantil.setForeground(UNIMPLEMENTED_COLOR);
}
	
	/**
	 * Enable the time start, time end and the slider 
	 */
	private void enablePageOptionsTimeConf(){
		jlPageTimeStart.setForeground(Color.BLACK);
		jlPageTimeEnd.setForeground(Color.BLACK);
		jtfPageTimeStart.setEnabled(true);
		jtfPageTimeEnd.setEnabled(true);
		jsPageTime.setEnabled(true);
	}
	
	
	/**
	 * Disable all components in dataset options
	 */
	private void disableAllDatasetOptions(){
//		jbgDraw1.clearSelection();
		jcbRaw.setEnabled(false);
		jcbMoving.setEnabled(false);
//		jbgDraw2.clearSelection();
		jcbMin.setEnabled(false);
		jcbMax.setEnabled(false);
		jcbMean.setEnabled(false);
		jcbStdDvt.setEnabled(false);
		jcbThroughput.setEnabled(false);
		jlTimeStartTitle.setForeground(Color.GRAY);
		jlTimeEndTitle.setForeground(Color.GRAY);
		jlTimeStart_ms.setForeground(Color.GRAY);
		jlTimeEnd_ms.setForeground(Color.GRAY);
		jtfTimeStart.setEnabled(false);
		jtfTimeEnd.setEnabled(false);
		jsTime.setEnabled(false);
		jlDatasetMaxPoints.setForeground(Color.GRAY);
		jtfDatasetMaxPoints.setEnabled(false);
		jtfDatasetTimeWindow.setEnabled(false);
		jtfDatasetStep.setEnabled(false);
		jlDatasetTimeWindow.setForeground(Color.GRAY);
		jlDatasetStep.setForeground(Color.GRAY);
		jlDatasetNumberOfSlices.setForeground(Color.GRAY);
		jtfDatasetNumberOfSlices.setEnabled(false);
		jbDrawGraph.setEnabled(false);
	}
	
	/**
	 * Enable only for time chart
	 */
	private void enableOnlyTimeChart(){
		disableAllDatasetOptions();
		jcbRaw.setEnabled(true);
		jcbMoving.setEnabled(true);
		jcbMin.setEnabled(false);
		jcbMax.setEnabled(false);
		jcbMean.setEnabled(false);
		jcbStdDvt.setEnabled(false);
		jcbThroughput.setEnabled(false);
		jtfTimeStart.setEnabled(true);
		jtfTimeEnd.setEnabled(true);
		jlTimeStartTitle.setForeground(Color.BLACK);
		jlTimeEndTitle.setForeground(Color.BLACK);
		jlTimeStart_ms.setForeground(Color.BLACK);
		jlTimeEnd_ms.setForeground(Color.BLACK);
		jsTime.setEnabled(true);
		jlDatasetMaxPoints.setForeground(Color.BLACK);
		jtfDatasetMaxPoints.setEnabled(true);
		jtfDatasetTimeWindow.setEnabled(false);
		jtfDatasetStep.setEnabled(false);
		jlDatasetTimeWindow.setForeground(Color.GRAY);
		jlDatasetStep.setForeground(Color.GRAY);
		jlDatasetNumberOfSlices.setForeground(Color.GRAY);
		jtfDatasetNumberOfSlices.setEnabled(false);
		jbDrawGraph.setEnabled(true);
	}
	
	/**
	 * Enable only for time chart
	 */
	private void enableOnlyTimeMovingChart(){
		disableAllDatasetOptions();
		jcbRaw.setEnabled(true);
		jcbMoving.setEnabled(true);
		jcbMin.setEnabled(false);
		jcbMax.setEnabled(false);
		jcbMean.setEnabled(false);
		jcbStdDvt.setEnabled(false);
		jcbThroughput.setEnabled(false);
		jtfTimeStart.setEnabled(true);
		jtfTimeEnd.setEnabled(true);
		jlTimeStartTitle.setForeground(Color.BLACK);
		jlTimeEndTitle.setForeground(Color.BLACK);
		jlTimeStart_ms.setForeground(Color.BLACK);
		jlTimeEnd_ms.setForeground(Color.BLACK);
		jsTime.setEnabled(true);
		jlDatasetMaxPoints.setForeground(Color.BLACK);
		jtfDatasetMaxPoints.setEnabled(true);
		jtfDatasetTimeWindow.setEnabled(true);
		jtfDatasetStep.setEnabled(true);
		jlDatasetTimeWindow.setForeground(Color.BLACK);
		jlDatasetStep.setForeground(Color.BLACK);
		jlDatasetNumberOfSlices.setForeground(Color.GRAY);
		jtfDatasetNumberOfSlices.setEnabled(false);
		jbDrawGraph.setEnabled(true);
	}
	
	/**
	 * Enable only for frequency chart
	 */
	private void enableOnlyFrequencyChart(){
		disableAllDatasetOptions();
		jlDatasetNumberOfSlices.setForeground(Color.BLACK);
		jtfDatasetNumberOfSlices.setEnabled(true);
		jbDrawGraph.setEnabled(true);
	}
	
	/**
	 * Enable only for quantil chart
	 */
	private void enableOnlyQuantilChart(){
		disableAllDatasetOptions();
		jlDatasetNumberOfSlices.setForeground(Color.BLACK);
		jtfDatasetNumberOfSlices.setEnabled(true);
		jbDrawGraph.setEnabled(true);
	}
	
	
	public void setGraphOptionsVisible(boolean b) {
		jpDraw1.setVisible(b);
		jpDraw2.setVisible(b);
		jpDrawButtons.setVisible(b);
		jpDrawConfig.setVisible(b);
	}
	
	public int getSelectedPage(){
		int selectedRow = jtYAxisValues.getSelectedRow();
		if(selectedRow != -1){
			int selectedPage = Integer.valueOf(jtYAxisValues.getValueAt(selectedRow, PAGE).toString());
			return selectedPage;
		}
		else{
			return -1;
		}
	}
	
	public Map<Integer, List<StatisticDataImpl>> getStatisticMap() {
		return statisticMap;
	}

	public Map<String, String> getMapTreeComment() {
		return mapTreeComment;
	}
	
	public String getComment()
	{
		String COMMENT = "Comment";
		String PAGE = "Page";
		String PAGE_ID = "id";
		String DATASET = "Dataset";
		String DATASET_TEST_NAME = "TestName";
		String DATASET_BLADE_ID = "BladeId";
		String DATASET_FIELD = "Field"; 
//		String DATASET_TYPE_ATT = "Type";
//		String DATASET_COMMENT = "Comment";
//		String DATASET_TYPE = "DatasetType";
//		String STATISTIC = "Statistic";
//		String VALUE = "value";
//		String URL = "url";
//
//		String FILTER = "Filter";
//		String FIELD_FILTER = "Field";
//		String OPERATOR_FILTER = "Operator";
//		String KEYWORD_FILTER = "Keyword";

		String returnComment = mapTreeComment.get("[Report]");
		// pages
		Element page;
		Attribute pageAtt;

		Set<Entry<Integer, List<Integer>>> setDataset = pageIdAndDatasetIds.entrySet();
		Iterator<Entry<Integer, List<Integer>>> iteratorDataset = setDataset.iterator();		
		while(iteratorDataset.hasNext()){
			Map.Entry<Integer, List<Integer>> meDS = iteratorDataset.next();
			int currentPage = meDS.getKey();
			page = new Element(PAGE);
			pageAtt = new Attribute(PAGE_ID, String.valueOf(currentPage));
			page.setAttribute(pageAtt);
			List<Integer> datasetList = meDS.getValue();

			// Dataset
			Element dataset;

			Iterator<Integer> iterator = datasetList.iterator();
			while (iterator.hasNext()) {
				Integer currentDataset = iterator.next();
				List<StatisticDataImpl> statList = statisticMap.get(currentDataset);
				Iterator<StatisticDataImpl> sdiIt = statList.iterator();

				dataset = new Element(DATASET);

				// Dataset Attributes
				Attribute datasetTestName;
				Attribute datasetAliasId;
				Attribute datasetField;
				Attribute datasetCommentAtt;

				while (sdiIt.hasNext()) {
					StatisticDataImpl sdi = sdiIt
							.next();
					String datasetTestNameN = sdi.getTestName();
					String datasetAliasIdN;
					String datasetCom;
					if(sdi.getDatasetType() == 0){
						datasetAliasIdN = sdi.getBladeId();
					}
					else{
						datasetAliasIdN = sdi.getBladeIdAlias();
					}
					String datasetFieldN = sdi.getField();
					if(datasetFieldN == null){
						datasetFieldN = "";
					}
					datasetTestName = new Attribute(DATASET_TEST_NAME, datasetTestNameN);
					datasetAliasId = new Attribute(DATASET_BLADE_ID, datasetAliasIdN);
					datasetField = new Attribute(DATASET_FIELD, datasetFieldN);

					dataset.setAttribute(datasetTestName);
					dataset.setAttribute(datasetAliasId);
					dataset.setAttribute(datasetField);
					datasetCom = mapTreeComment.get("[Report, Page "+String.valueOf(currentPage)+", "+datasetAliasIdN+"]");
					if(datasetCom != null){
						datasetCommentAtt = new Attribute(COMMENT, datasetCom);
						dataset.setAttribute(datasetCommentAtt);
					}
				}
			}
			returnComment += mapTreeComment.get("[Report, Page "+String.valueOf(currentPage)+"]");
		}
		return returnComment;
	}
	


	public Map<Integer, JFreeChart> getJFreeChartMap() {
		return pageIdAndJFreeChart;
	}
	
	public Map<Integer, List<Integer>> getPageIdAndDatasetIds() {
		return pageIdAndDatasetIds;
	}

	public Map<Integer, org.ow2.clif.analyze.lib.graph.ChartData> getChartDataList() {
		return chartDataList;
	}

	public Map<Integer, org.ow2.clif.analyze.lib.graph.DatasetOptions> getDatasetIdAndGraphOptions() {
		return datasetIdAndDatasetOptions;
	}


	
	//////////////////////////////////
	// GUI variable declaration 	//
	//////////////////////////////////

	private javax.swing.ButtonGroup bgChartDataType;
	private javax.swing.ButtonGroup bgMovingStatType;
	private javax.swing.JButton jbAddToYAxis;
	private javax.swing.JButton jbValidateMaxPoint;
	private javax.swing.JButton jbValidateTimeDisplay;
	private javax.swing.JButton jbValidateXLabel;
	private javax.swing.JButton jbValidateYAxisOptions;
	private javax.swing.JComboBox jcbGraphType;
	private javax.swing.JLabel jlDataType;
	private javax.swing.JLabel jlGraphTypeTitle;
	private javax.swing.JLabel jlMaxPointTitle;
	private javax.swing.JLabel jlSepEmpty;
	private javax.swing.JLabel jlSepEmpty2;
	private javax.swing.JLabel jlTimeEndTitle;
	private javax.swing.JLabel jlTimeEnd_ms;
	private javax.swing.JLabel jlTimeStartTitle;
	private javax.swing.JLabel jlTimeStart_ms;
	private javax.swing.JLabel jlXLabelTitle;
	private javax.swing.JLabel jlYAxisLabel;
	private javax.swing.JMenuItem jmiAddToYAxis;
	private javax.swing.JPanel jpAddYAxis;
	private javax.swing.JPanel jpButtonsForTree;
	private javax.swing.JPanel jpComment;
	private javax.swing.JPanel jpDataTypeTitle;
	private javax.swing.JPanel jpDiagram;
	private javax.swing.JPanel jpGraph;
	private javax.swing.JPanel jpGraphManager;
	private javax.swing.JPanel jpGraphOptions;
	private javax.swing.JPanel jpGraphParameters;
	private javax.swing.JPanel jpGraphType;
	private javax.swing.JPanel jpGrpahComment;
	private javax.swing.JPanel jpITimeStartL;
	private javax.swing.JPanel jpMain;
	private javax.swing.JPanel jpMaxPoints;
	private javax.swing.JPanel jpMovingStat;
	private javax.swing.JPanel jpMovingType;
	private javax.swing.JPanel jpOptions;
	private javax.swing.JPanel jpRowData;
	private javax.swing.JPanel jpScreenTime;
	private javax.swing.JPanel jpStat;
	private javax.swing.JPanel jpTimeConf;
	private javax.swing.JPanel jpTimeEnd;
	private javax.swing.JPanel jpTimeEndL;
	private javax.swing.JPanel jpTimeEndTF;
	private javax.swing.JPanel jpTimeStart;
	private javax.swing.JPanel jpTimeStartTF;
	private javax.swing.JPanel jpTreeButtons;
	private javax.swing.JPanel jpTypeMax;
	private javax.swing.JPanel jpTypeMean;
	private javax.swing.JPanel jpTypeMin;
	private javax.swing.JPanel jpTypeStandardDeviation;
	private javax.swing.JPanel jpValidateOptions;
	private javax.swing.JPanel jpValidatedTimeDisplay;
	private javax.swing.JPanel jpXAxis;
	private javax.swing.JPanel jpYAxis;
	private javax.swing.JPanel jpYAxisProperties;
	private javax.swing.JPanel jpYLabel;
	private javax.swing.JPopupMenu jpmRightClickTree;
	private javax.swing.JPanel jpChartTitle;
	private javax.swing.JRadioButton jrbMovingStat;
	private javax.swing.JRadioButton jrbRowData;
	private javax.swing.JRadioButton jrbTypeMax;
	private javax.swing.JRadioButton jrbTypeMean;
	private javax.swing.JRadioButton jrbTypeMin;
	private javax.swing.JRadioButton jrbTypeStandardDeviation;
	private javax.swing.JSlider jsTime;
	private javax.swing.JScrollPane jspExecutionsTree;
	private javax.swing.JScrollPane jspStat;
	private javax.swing.JScrollPane jspYAxisValues;
	private javax.swing.JTree jtExecutionsTree;
	private javax.swing.JTable jtYAxisValues;
	private javax.swing.JTextArea jtaStat;
	private javax.swing.JTextField jtfChartTitle;
	private javax.swing.JTextField jtfMaxPoint;
	private javax.swing.JTextField jtfTimeEnd;
	private javax.swing.JTextField jtfTimeStart;
	private javax.swing.JTextField jtfXLabel;
	private javax.swing.JButton jbRefresh;
	private javax.swing.JTextField jtfYAxisLabel;
	private javax.swing.JMenuItem jmiRefresh;
	private javax.swing.JSeparator jsSepPopupMenu;
	private javax.swing.JTabbedPane jtPChart;
	private javax.swing.JPanel jpanel1;
	private javax.swing.JTabbedPane jtPsubChart;
	private javax.swing.JPanel jpDatasetOptionsTab;
	private javax.swing.JPanel jpPageOptionsTab;
	private javax.swing.JPanel jpanelsub2;
	private javax.swing.JPanel jpanelsub3;
	private javax.swing.JSplitPane jspGraph;
	private javax.swing.JSplitPane jspGraphNorth;
	private javax.swing.JButton jbFilter;
	private javax.swing.JButton jbDrawGraph;
	private javax.swing.JButton jbRefreshGraph;
	private javax.swing.JComboBox pageComboBox;
	
	// Dataset Options TAB 
	private javax.swing.JPanel jpDraw1;
	private javax.swing.JCheckBox jcbRaw;
	private javax.swing.JCheckBox jcbMoving;
	private javax.swing.JCheckBox jcbHistogram;
	private javax.swing.ButtonGroup jbgDraw1;
	private javax.swing.JPanel jpDraw2;
	private javax.swing.JCheckBox jcbMin;
	private javax.swing.JCheckBox jcbMax;
	private javax.swing.JCheckBox jcbMean;
//	private javax.swing.JCheckBox jcbMedian;
	private javax.swing.JCheckBox jcbStdDvt;
	private javax.swing.JCheckBox jcbThroughput;
	private javax.swing.ButtonGroup jbgDraw2;
	private javax.swing.JPanel jpDrawConfig;
	private javax.swing.JPanel jpDrawButtons;
	private javax.swing.JPanel jpp;
	private javax.swing.JScrollPane jsppp;
	private javax.swing.JScrollPane jScrollPaneComment;
	private javax.swing.JTree jTreeComment;
	private javax.swing.JTextArea jtaNewComment;
	private javax.swing.JButton jbSaveComment;
	private javax.swing.JScrollPane jScrollPaneGraphOptions;
//	private javax.swing.JCheckBox jcbToDraw;
//	private JRadioButton jrbTmp;
	private javax.swing.JLabel jlDatasetTimeWindow;
	private javax.swing.JLabel jlDatasetStep;
	private javax.swing.JTextField jtfDatasetTimeWindow;
	private javax.swing.JTextField jtfDatasetStep;
	private javax.swing.JPanel jpDatasetWindowStep;
	private javax.swing.JTextField jtfDatasetMaxPoints;
	private javax.swing.JLabel jlDatasetMaxPoints;
	private javax.swing.JPanel jpDatasetMaxPoints;
	private javax.swing.JLabel jlDatasetNumberOfSlices;
	private javax.swing.JTextField jtfDatasetNumberOfSlices;
	private javax.swing.JPanel jpDatasetNumberOfSlices;
	
	
	// Page Options TAB
	private javax.swing.JLabel jlPageSelected;
	private javax.swing.JPanel jpPageChartType;
	private javax.swing.JPanel jpPageOptionsTime;
	private javax.swing.JCheckBox jcbPageNoChart;
	private javax.swing.JCheckBox jcbPageTimeChart;
	private javax.swing.JCheckBox jcbPageHistogramChart;
	private javax.swing.JCheckBox jcbPageQuantil;
	private javax.swing.ButtonGroup jbgPageChartType;
	private javax.swing.JLabel jlPageTimeStart;
	private javax.swing.JLabel jlPageTimeEnd;
	private javax.swing.JTextField jtfPageTimeStart;
	private javax.swing.JTextField jtfPageTimeEnd;
	private javax.swing.JSlider jsPageTime;
	
	final Color UNIMPLEMENTED_COLOR    = new Color(107, 108, 153);		// BLUE-GRAY // color for unimplemented functionalities

	
}
