/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Grégory CALONNIER
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.lib.egui.wizards;


import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.ow2.clif.analyze.api.AnalyzerExternalAccess;
import org.ow2.clif.deploy.ClifAnalyzerAppFacade;
import org.ow2.clif.storage.api.TestDescriptor;
import org.ow2.clif.supervisor.api.ClifException;


/**
 * Provide a wizard page to select local CLIF executions.
 * @author Grégory CALONNIER
 * @author Jordan BRUNIER
 */
public class CLIFReportExecutionsChoicePage extends WizardPage {
	
	private boolean completePage;
	
	// Lists for selection
	private List executionsList;
	private List executionsChoicesList;
	
	
	/**
	 * Constructor for CLIFReportExecutionsChoicePage.
	 */
	public CLIFReportExecutionsChoicePage() {
		super("wizardPage");
		setTitle("CLIF Report File");
		setDescription("Select local CLIF execution(s)...");
		
		this.completePage = false;
	}
	
	
	/**
	 * Create graphical interface.
	 * @param parent the container
	 */
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);		
		GridLayout containerLayout = new GridLayout();
		containerLayout.numColumns = 3;
		containerLayout.verticalSpacing = 9;
		container.setLayout(containerLayout);
		GridData gd;
		
		
		/*LEFT AREA : Label + List*/
		Composite leftArea = new Composite(container, SWT.NULL);
		GridLayout layoutLeftArea = new GridLayout();
		leftArea.setLayout(layoutLeftArea);
		layoutLeftArea.numColumns = 1;
		
		Label labExecutionsTitle = new Label(leftArea, SWT.NULL);
		labExecutionsTitle.setText("Local available CLIF execution(s) :");

		executionsList = new List(leftArea, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL);
		gd = new GridData(GridData.FILL_BOTH);
		gd.heightHint = 300;
		gd.widthHint = 200;
		executionsList.setLayoutData(gd);	

		
		/*MIDDLE AREA : Buttons*/
		Composite buttonArea = new Composite(container, SWT.NULL);
		GridLayout layoutButtonArea = new GridLayout();
		buttonArea.setLayout(layoutButtonArea);
		layoutButtonArea.numColumns = 1;
		
		Label labSeparator = new Label(buttonArea, SWT.NULL);
		labSeparator.setText("");
		
		Button butAdd = new Button(buttonArea, SWT.PUSH);
		butAdd.setText(">");
		butAdd.setToolTipText("Add execution(s)");
		butAdd.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				addExecutions();
			}
		});
		gd = new GridData();
		butAdd.setLayoutData(gd);
		gd.widthHint = 50;
		
		Button butAddAll = new Button(buttonArea, SWT.PUSH);
		butAddAll.setText(">>");
		butAddAll.setToolTipText("Add all executions");
		butAddAll.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				addAllExecutions();
			}
		});
		gd = new GridData();
		butAddAll.setLayoutData(gd);
		gd.widthHint = 50;
		
		Button butDelete = new Button(buttonArea, SWT.PUSH);
		butDelete.setText("<");
		butDelete.setToolTipText("Delete execution(s)");
		butDelete.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				deleteExecutions();
			}
		});
		gd = new GridData();
		butDelete.setLayoutData(gd);
		gd.widthHint = 50;
		
		Button butDeleteAll = new Button(buttonArea, SWT.PUSH);
		butDeleteAll.setText("<<");
		butDeleteAll.setToolTipText("Delete all executions");
		butDeleteAll.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				deleteAllExecutions();
			}
		});
		gd = new GridData();
		butDeleteAll.setLayoutData(gd);
		gd.widthHint = 50;
		
		
		/*RIGHT AREA : Label + List*/
		Composite rightArea = new Composite(container, SWT.NULL);
		GridLayout layoutRightArea = new GridLayout();
		rightArea.setLayout(layoutRightArea);
		layoutRightArea.numColumns = 1;
		
		Label labExecutionschoiceTitle = new Label(rightArea, SWT.NULL);
		labExecutionschoiceTitle.setText("Selected execution(s) :");

		executionsChoicesList = new List(rightArea, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL);
		gd = new GridData(GridData.FILL_BOTH);
		gd.heightHint = 300;
		gd.widthHint = 200;
		executionsChoicesList.setLayoutData(gd);
		
		initialize();
		dialogChanged();
		setControl(container);
	}
	
    
	/**
	 * Initialize executions list.
	 */
	private void initialize() {
		// Get all local executions
		AnalyzerExternalAccess analyzer = null;
		ClifAnalyzerAppFacade clifAnalyzerApp = ClifAnalyzerAppFacade.getInstance();
		
		try{
			Component analyzerComp = clifAnalyzerApp.getComponentByName("analyzer");
			analyzer = (AnalyzerExternalAccess) analyzerComp.getFcInterface(AnalyzerExternalAccess.ANALYZER_EXTERNAL_ACCESS);
			
		}
		catch (NoSuchInterfaceException ex)
		{
			System.err.println(ex);
		}		
		TestDescriptor[] executionsDescList = null;
		try{
			executionsDescList = analyzer.getTests(null);
		} catch(ClifException ce) {
			System.err.println(ce);
		}
		
		// Update list with executions
		for(int i = 0; i < executionsDescList.length; i++){
			executionsList.add(executionsDescList[i].getName());
		}
	}
	
	/**
	 * Ensures that executions choices list is set.
	 */
	private void dialogChanged() {		
		updateStatus(null);
	}
	
	/**
	 * Update error message.
	 * @param message
	 */
	private void updateStatus(String message) {
		setErrorMessage(message);
		setPageComplete(message == null);
	}
	
	
	/**
	 * Add selected executions to the right list 
	 */
	private void addExecutions() {
		// Get selected executions
		String[] lstSelectedExec = executionsList.getSelection();
		int[] lstSelectedInd = executionsList.getSelectionIndices();
		
		if(lstSelectedExec != null){
			// Move selected executions from left to right
			for(int i = 0; i < lstSelectedExec.length; i++) {
				executionsChoicesList.add(lstSelectedExec[i]);				
			}
			
			// Delete selected executions from left			
			for(int i = 0; i < lstSelectedExec.length; i++) {				
				executionsList.remove(lstSelectedInd[i] - i);
			}			
		
			// Update "Next" button
			completePage = (executionsChoicesList.getItemCount() > 0);
			getWizard( ).getContainer( ).updateButtons( );
		}else{
			System.out.println("Selection must be not empty !");
		}	
	}
	
	/**
	 * Delete selected executions from the right list 
	 */
	private void deleteExecutions() {
		// Get selected executions
		String[] lstSelectedExec = executionsChoicesList.getSelection();
		int[] lstSelectedInd = executionsChoicesList.getSelectionIndices();
		
		if(lstSelectedExec != null){
			// Move selected executions from right to left
			for(int i = 0; i < lstSelectedExec.length; i++) {
				executionsList.add(lstSelectedExec[i]);				
			}
			
			// Delete selected executions from right		
			for(int i = 0; i < lstSelectedExec.length; i++) {				
				executionsChoicesList.remove(lstSelectedInd[i] - i);
			}	
			
			// Update "Next" button
			completePage = (executionsChoicesList.getItemCount() > 0);
			getWizard( ).getContainer( ).updateButtons( );
		}else{
			System.out.println("Selection must be not empty !");
		}		
	}
	
	/**
	 * Add all executions to the right list 
	 */
	private void addAllExecutions() {
		// Get all executions
		String[] lstExec = executionsList.getItems();		
		
		if(lstExec != null){
			// Move all executions from left to right
			for(int i = 0; i < lstExec.length; i++) {
				executionsChoicesList.add(lstExec[i]);				
			}
			
			// Delete all executions from left			
			for(int i = 0; i < lstExec.length; i++) {				
				executionsList.removeAll();
			}			
			
			// Update "Next" button
			completePage = (executionsChoicesList.getItemCount() > 0);
			getWizard( ).getContainer( ).updateButtons( );
		}else{
			System.out.println("Selection must be not empty !");
		}		
	}
	
	/**
	 * Delete all executions from the right list 
	 */
	private void deleteAllExecutions() {
		// Get all executions
		String[] lstExec = executionsChoicesList.getItems();		
		
		if(lstExec != null){
			// Move all executions from right to left
			for(int i = 0; i < lstExec.length; i++) {
				executionsList.add(lstExec[i]);				
			}
			
			// Delete all executions from right			
			for(int i = 0; i < lstExec.length; i++) {				
				executionsChoicesList.removeAll();
			}			
			
			// Update "Next" button
			completePage = (executionsChoicesList.getItemCount() > 0);
			getWizard( ).getContainer( ).updateButtons( );
		}else{
			System.out.println("Selection must be not empty !");
		}		
	}
	
	public boolean isPageComplete() {		
		return(completePage);		
	}
	
}
