/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Grégory CALONNIER
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.analyze.lib.oda;

import org.eclipse.datatools.connectivity.oda.OdaException;
import org.ow2.clif.analyze.lib.oda.i18n.Messages;

/**
 * A helper class that helps ResultSetMetaData to parse column information and
 * provide a public access to ResultSet's original column names.
 */

public class ResultSetMetaDataHelper
{
	String tableName;
	String eventLabel;
	String[] columnNames;
	String[] originalColumnNames;
	String[] columnLabels;
	
	/**
	 * Constructor
	 * 
	 * @param tableName
	 * @param eventLabel
	 * @param colNames
	 * @param colLabels
	 * @throws OdaException
	 * 				if colNames is null
	 */
	ResultSetMetaDataHelper( String tableName, String eventLabel, String[] colNames, String[] colLabels ) throws OdaException
	{
		if ( colNames == null )
			throw new OdaException( Messages.getString( "common_ARGUMENT_CANNOT_BE_NULL" ) ); //$NON-NLS-1$
		this.tableName=tableName;
		this.eventLabel=eventLabel;
		this.columnNames = colNames;
		this.columnLabels = colLabels;
		this.originalColumnNames = colNames;
	}

	/**
     * Gets the table name.
     * 
     * @return the table name
     */
	String getTableName( )
	{
		return this.tableName;
	}
	
	/**
     * Gets the event label.
     * 
     * @return the event label
     */
	String getEventLabel( )
	{
		return this.eventLabel;
	}
	
	/**
     * Gets the column names.
     * 
     * @return the column names
     */
	String[] getColumnNames( )
	{
		
		return this.columnNames;
	}

	/**
     * Gets the column labels.
     * 
     * @return the column labels
     */
	String[] getColumnLabels( )
	{
		return this.columnLabels;
	}

	/**
	 * Get the orignal column names
	 * @return the array of the original column names
	 */
	public String[] getOriginalColumnNames( )
	{
		return this.originalColumnNames;
	}

	/**
	 * Get the original column name of the specified column name
	 * @param columnName
	 * @return return the original column name for the given column name
	 */
	public String getOriginalColumnName( String columnName )
	{
		String originName = null;

		for ( int i = 0; i < columnNames.length; i++ )
		{
			if ( columnName.equals( columnNames[i] ) )
			{
				originName = originalColumnNames[i];
			}
		}

		return originName;
	}
	
}
